﻿Imports System.IO
Imports System.Console
Imports System.Math
' The above two Imports elimitate the need to use COnsole. and Math. with console I/O and math function calls.

Public Class CDynArr
  Public Property arrSize As Integer
  Public Property arrName As String
  Public Property StoreIndex As Integer
  Public Property RecallIndex As Integer
  Public bIsSorted As Boolean
  Public data() As Double

  Public Sub New(ByVal varName As String, ByVal arrSize As Integer)
    Dim I As Integer

    ' Simple constructor
    arrName = varName
    Me.arrSize = arrSize
    ReDim data(arrSize)
    bIsSorted = False
    ' explicitly initialize the array
    For I = 0 To arrSize
      data(I) = 0
    Next
    StoreIndex = 0
    RecallIndex = 0
  End Sub

  Private Function RandomNormal(fMean As Double, fSdev As Double) As Double
    Dim X1 As Double, X2 As Double
    Dim Y1 As Double, Y2 As Double
    Dim Pi As Double

    X1 = Rnd(1)
    X2 = Rnd(1)
    Y1 = fMean + fSdev * Math.Sqrt(-2 * Math.Log(X1)) * Math.Sin(2 * Pi * X2)
    Y2 = fMean + fSdev * Math.Sqrt(-2 * Math.Log(X1)) * Math.Cos(2 * Pi * X2)
    RandomNormal = If(Rnd(1) > 0.5, Y1, Y2)
  End Function

  Public Sub Fill(ByVal bUniform As Boolean, ByVal bIntegers As Boolean,
                 ByVal Low As Double, ByVal High As Double)
    Dim I As Integer
    Dim iLow, iHigh As Long
    Dim Mean, Sdev As Double

    bIsSorted = False
    Randomize()

    If bUniform Then
      If bIntegers Then
        iLow = Fix(Low)
        iHigh = Fix(High)
        For I = 0 To arrSize
          data(I) = iLow + CLng((iHigh - iLow) * Rnd(1))
        Next
      Else
        For I = 0 To arrSize
          data(I) = Low + (High - Low) * Rnd(1)
        Next
      End If
    Else
      If bIntegers Then
        Mean = Fix(Low)
        Sdev = Fix(High)
        For I = 0 To arrSize
          data(I) = Fix(RandomNormal(Mean, Sdev))
        Next
      Else
        Mean = Low
        Sdev = High
        For I = 0 To arrSize
          data(I) = RandomNormal(Mean, Sdev)
        Next
      End If
    End If
  End Sub

  Public Sub New(ByVal varName As String, ByVal arrSize As Integer,
                 ByVal bUniform As Boolean, ByVal bIntegers As Boolean,
                 ByVal Low As Double, ByVal High As Double)
    ' Constructor for either array with uniformly-distrbuted or normall-distributed
    ' random numbers

    Me.arrName = arrName
    Me.arrSize = arrSize
    ReDim data(arrSize)

    Fill(bUniform, bIntegers, Low, High)
    StoreIndex = 0
    RecallIndex = 0
  End Sub

  Public Sub Fill(ByVal First As Double, ByVal Increment As Double)
    Dim I As Integer

    bIsSorted = False
    For I = 0 To arrSize
      data(I) = First
      First += Increment
    Next
  End Sub

  Public Sub New(ByVal varName As String, ByVal arrSize As Integer,
                 ByVal First As Double, ByVal Increment As Double)
    ' Constructor for arrays with a sequence of numbers
    Me.arrName = varName
    Me.arrSize = arrSize
    ReDim data(arrSize)

    Fill(First, Increment)
    StoreIndex = 0
    RecallIndex = 0
  End Sub

  Public Sub Fill(ByVal Low As Long, ByVal High As Long)
    ' constructor for array with uniformly-distributed random integers
    Dim I As Integer

    bIsSorted = False
    Randomize()

    For I = 0 To arrSize
      data(I) = Low + Fix((High - Low) * Rnd(1))
    Next
  End Sub

  Public Sub New(ByVal varName As String, ByVal arrSize As Integer,
                 ByVal Low As Long, ByVal High As Long)
    ' constructor for array with uniformly-distributed random integers

    Me.arrName = varName
    Me.arrSize = arrSize
    ReDim data(arrSize)

    Fill(Low, High)
    StoreIndex = 0
    RecallIndex = 0
  End Sub

  Public Function GetSize() As Integer
    Return arrSize
  End Function

  Public Sub Resize(ByVal newArrSize As Integer)

    bIsSorted = newArrSize <= arrSize And bIsSorted ' clear in-order state if we are adding new elements
    ReDim Preserve data(newArrSize)
    arrSize = newArrSize
  End Sub

  Public Sub Reset()
    arrSize = -1
    ReDim data(0)
    bIsSorted = False
  End Sub

  Public Sub Store(ByVal Index As Integer, ByVal X As Double)

    Index = Math.Abs(Index)
    If Index <= arrSize Then
      data(Index) = X
    Else
      arrSize = Index
      ReDim Preserve data(arrSize)
      data(arrSize) = X
    End If
    bIsSorted = False
  End Sub

  Public Sub StoreIncIndex(ByVal X As Double)
    Dim Index As Integer = Math.Abs(StoreIndex)

    If Index > arrSize Then Index = 0
    data(Index) = X
    StoreIndex = Index + 1
    bIsSorted = False
  End Sub

  Public Sub StoreDecIndex(ByVal X As Double)
    Dim Index As Integer = Math.Abs(StoreIndex)

    If Index > arrSize Then Index = 0
    data(Index) = X
    If Index > 0 Then
      StoreIndex = Index - 1
    Else
      StoreIndex = arrSize
    End If
    bIsSorted = False
  End Sub

  Public Sub StoreKeepIndex(ByVal X As Double)
    Dim Index As Integer = Math.Abs(StoreIndex)

    If Index > arrSize Then Index = 0
    data(Index) = X
    StoreIndex = Index
    bIsSorted = False
  End Sub


  Public Sub StorePlus(ByVal Index As Integer, ByVal X As Double)
    Index = Math.Abs(Index)
    If Index <= arrSize Then
      data(Index) += X
    Else
      arrSize = Index
      ReDim Preserve data(arrSize)
      data(arrSize) += X
    End If
    bIsSorted = False
  End Sub

  Public Sub StoreMinus(ByVal Index As Integer, ByVal X As Double)
    Index = Math.Abs(Index)
    If Index <= arrSize Then
      data(Index) -= X
    Else
      arrSize = Index
      ReDim Preserve data(arrSize)
      data(arrSize) -= X
    End If
    bIsSorted = False
  End Sub

  Public Sub StoreMult(ByVal Index As Integer, ByVal X As Double)
    Index = Math.Abs(Index)
    If Index <= arrSize Then
      data(Index) *= X
    Else
      arrSize = Index
      ReDim Preserve data(arrSize)
      data(arrSize) = 0 ' because the expanded array element is set to 0
    End If
    bIsSorted = False
  End Sub

  Public Sub StoreDiv(ByVal Index As Integer, ByVal X As Double)
    Index = Math.Abs(Index)
    If Index <= arrSize And X <> 0 Then
      data(Index) /= X
    ElseIf Index > arrSize Then
      arrSize = Index
      ReDim Preserve data(arrSize)
      data(arrSize) = 0 ' because the expanded array element is set to 0
    End If
    bIsSorted = False
  End Sub

  Public Function RecallIncIndex() As Double
    Dim Index = Math.Abs(RecallIndex)

    If Index >= 0 And Index <= arrSize Then
      RecallIndex += 1
      If RecallIndex > arrSize Then RecallIndex = 0
      Return data(Index)
    Else
      Return 0
    End If
  End Function

  Public Function RecallDecIndex() As Double
    Dim Index = Math.Abs(RecallIndex)

    If Index >= 0 And Index <= arrSize Then
      RecallIndex -= 1
      If RecallIndex < 0 Then RecallIndex = arrSize
      Return data(Index)
    Else
      Return 0
    End If
  End Function

  Public Function RecallKeepIndex() As Double
    Dim Index = Math.Abs(RecallIndex)

    If Index >= 0 And Index <= arrSize Then
      Return data(Index)
    Else
      Return 0
    End If
  End Function


  Public Function Recall(ByVal index As Integer) As Double
    If index >= 0 And index <= arrSize Then
      Return data(index)
    Else
      Return 0
    End If
  End Function

  Public Sub SortAscending()
    Dim I, J, Offset As Integer
    Dim bInOrder As Boolean
    Dim Buffer As Double

    Offset = arrSize
    Do While Offset > 1
      Offset = Fix(Offset * 5.0# / 11.0#)
      If Offset < 1 Then Offset = 1
      Do
        bInOrder = True
        For I = 1 To arrSize - Offset
          J = I + Offset
          If data(I) > data(J) Then
            Buffer = data(I)
            data(I) = data(J)
            data(J) = Buffer
            bInOrder = False
          End If
        Next I
      Loop Until bInOrder
    Loop
    bIsSorted = True
  End Sub

  Public Sub SortDescending()
    Dim I, J, Offset As Integer
    Dim bInOrder As Boolean
    Dim Buffer As Double

    Offset = arrSize
    Do While Offset > 1
      Offset = Fix(Offset * 5.0# / 11.0#)
      If Offset < 1 Then Offset = 1
      Do
        bInOrder = True
        For I = 1 To arrSize - Offset
          J = I + Offset
          If data(I) < data(J) Then
            bInOrder = False
            Buffer = data(I)
            data(I) = data(J)
            data(J) = Buffer
            bInOrder = False
          End If
        Next I
      Loop Until bInOrder
    Loop
    bIsSorted = True
  End Sub

  Public Function Search(ByVal FindX As Double, ByVal Optional First As Integer = 0,
                         ByVal Optional Tolerance As Double = 0,
                         ByVal Optional SearchMode As Integer = 0) As Integer
    Dim I As Integer

    If First >= 0 Then
      Select Case SearchMode
        Case Is > 0
          ' perform forward search
          For I = First To arrSize
            If data(I) > First Then
              Return I
            End If
          Next
        Case Is < 0
          ' perform forward search
          For I = First To arrSize
            If data(I) < First Then
              Return I
            End If
          Next
        Case Else ' = 0
          ' perform forward search
          For I = First To arrSize
            If Math.Abs(data(I) - FindX) <= Tolerance Then
              Return I
            End If
          Next
      End Select

    Else
      ' perform backward search
      First = -First
      If First > arrSize Then First = arrSize

      Select Case SearchMode
        Case Is > 0
          For I = First To 0 Step -1
            If data(I) > First Then
              Return I
            End If
          Next
        Case Is < 0
          For I = First To 0 Step -1
            If data(I) < First Then
              Return I
            End If
          Next
        Case Else
          For I = First To 0 Step -1
            If Math.Abs(data(I) - FindX) <= Tolerance Then
              Return I
            End If
          Next

      End Select
    End If

    Return -1
  End Function

  Public Function MiddleFirstSearch(ByVal FindX As Double, ByVal Optional Tolerance As Double = 0,
                         ByVal Optional SearchMode As Integer = 0) As Integer
    Dim Iup, Idown As Integer

    Iup = arrSize \ 2
    Idown = Iup

    Select Case SearchMode
      Case Is < 0
        ' start in the middle
        If data(Iup) < FindX Then
          Return Iup
        End If

        Do Until Idown <= 0 And Iup >= arrSize

          If Idown > 0 Then
            Idown -= 1
            If data(Iup) < FindX Then
              Return Idown
            End If
          End If

          If Iup < arrSize Then
            Iup += 1
            If data(Iup) < FindX Then
              Return Iup
            End If
          End If
        Loop

      Case Is > 0
        ' start in the middle
        If data(Iup) > FindX Then
          Return Iup
        End If

        Do Until Idown <= 0 And Iup >= arrSize

          If Idown > 0 Then
            Idown -= 1
            If data(Iup) > FindX Then
              Return Idown
            End If
          End If

          If Iup < arrSize Then
            Iup += 1
            If data(Iup) > FindX Then
              Return Iup
            End If
          End If
        Loop
      Case Else
        ' start in the middle
        If Math.Abs(data(Iup) - FindX) <= Tolerance Then
          Return Iup
        End If

        Do Until Idown <= 0 And Iup >= arrSize

          If Idown > 0 Then
            Idown -= 1
            If Math.Abs(data(Idown) - FindX) <= Tolerance Then
              Return Idown
            End If
          End If

          If Iup < arrSize Then
            Iup += 1
            If Math.Abs(data(Iup) - FindX) <= Tolerance Then
              Return Iup
            End If
          End If
        Loop

    End Select
    Return -1
  End Function

  Public Function CircularSearch(ByVal FindX As Double, ByVal Optional First As Integer = 0,
                         ByVal Optional Tolerance As Double = 0,
                         ByVal Optional SearchMode As Integer = 0) As Integer
    Dim I As Integer

    If First > 0 Then
      ' forward circular search 
      Select Case SearchMode
        Case Is < 0
          ' perform search from First element to the end
          For I = First To arrSize
            If data(I) < FindX Then
              Return I
            End If
          Next

          ' perform search from the bgeinning to the First-1 element
          For I = 0 To First - 1
            If data(I) < FindX Then
              Return I
            End If
          Next
        Case Is > 0
          ' perform search from First element to the end
          For I = First To arrSize
            If data(I) > FindX Then
              Return I
            End If
          Next

          ' perform search from the bgeinning to the First-1 element
          For I = 0 To First - 1
            If data(I) > FindX Then
              Return I
            End If
          Next
        Case Else
          ' perform search from First element to the end
          For I = First To arrSize
            If Math.Abs(data(I) - FindX) <= Tolerance Then
              Return I
            End If
          Next

          ' perform search from the bgeinning to the First-1 element
          For I = 0 To First - 1
            If Math.Abs(data(I) - FindX) <= Tolerance Then
              Return I
            End If
          Next
      End Select

    Else
      ' backward circular search 
      First = -First

      Select Case SearchMode
        Case Is < 0
          ' perform search from the bgeinning to the First-1 element
          For I = First To 0 Step -1
            If data(I) < FindX Then
              Return I
            End If
          Next

          ' perform search the end to the First element 
          For I = arrSize To First + 1 Step -1
            If data(I) < FindX Then
              Return I
            End If
          Next
        Case Is > 0
          ' perform search from the bgeinning to the First-1 element
          For I = First To 0 Step -1
            If data(I) > FindX Then
              Return I
            End If
          Next

          ' perform search the end to the First element 
          For I = arrSize To First + 1 Step -1
            If data(I) > FindX Then
              Return I
            End If
          Next
        Case Else
          ' perform search from the bgeinning to the First-1 element
          For I = First To 0 Step -1
            If Math.Abs(data(I) - FindX) <= Tolerance Then
              Return I
            End If
          Next

          ' perform search the end to the First element 
          For I = arrSize To First + 1 Step -1
            If Math.Abs(data(I) - FindX) <= Tolerance Then
              Return I
            End If
          Next
      End Select
    End If

    Return -1
  End Function

  Public Function HeuristicSearch(ByVal FindX As Double,
                                  ByRef NewPosition As Integer,
                                  ByVal Optional Shift As Integer = 1,
                                  ByVal Optional First As Integer = 0,
                                  ByVal Optional Tolerance As Double = 0) As Integer
    Dim I, J As Integer
    Dim Buffer As Double

    NewPosition = -1

    If First >= 0 Then
      ' perform forward search
      For I = First To arrSize
        If Math.Abs(data(I) - FindX) <= Tolerance Then
          J = I + Shift
          NewPosition = I
          If Shift < 0 Then
            If J >= 0 Then
              bIsSorted = False
              Buffer = data(I)
              data(I) = data(J)
              data(J) = Buffer
            End If
            NewPosition = J
          ElseIf Shift > 0 Then
            If J <= arrSize Then
              bIsSorted = False
              Buffer = data(I)
              data(I) = data(J)
              data(J) = Buffer
            End If
            NewPosition = J
          End If
          Return I
        End If
      Next
    Else
      ' perform Backward search
      First = -First
      If First > arrSize Then First = arrSize
      For I = First To 0 Step -1
        If Math.Abs(data(I) - FindX) <= Tolerance Then
          J = I + Shift
          NewPosition = I
          If Shift < 0 Then
            If J >= 0 Then
              bIsSorted = False
              Buffer = data(I)
              data(I) = data(J)
              data(J) = Buffer
            End If
            NewPosition = J
          ElseIf Shift > 0 Then
            If J <= arrSize Then
              bIsSorted = False
              Buffer = data(I)
              data(I) = data(J)
              data(J) = Buffer
            End If
            NewPosition = J
          End If
          Return I
        End If
      Next
    End If
    Return -1
  End Function

  Public Sub Reverse()
    Dim I As Integer = 0
    Dim J As Integer = arrSize
    Dim X As Double

    Do Until I >= J
      X = data(I)
      data(I) = data(J)
      data(J) = X
      I += 1
      J -= 1
    Loop

  End Sub

  Public Function BinarySearch(ByVal FindX As Double, ByVal Tolerance As Double,
                               ByRef CountAbove As Integer, ByRef CountBelow As Integer) As Integer
    Dim Low As Integer = 0
    Dim High As Integer = arrSize
    Dim I, Idx, Median As Integer
    Dim X As Double

    If Not bIsSorted Then SortAscending()

    Idx = -1
    CountAbove = 0
    CountBelow = 0

    ' binary-search for a match in the array
    Do Until Low > High
      Median = (Low + High) \ 2
      X = data(Median)
      If Math.Abs(FindX - X) <= Tolerance Then
        Idx = Median
        Exit Do
      ElseIf FindX > X Then
        Low = Median + 1
      Else
        High = Median - 1
      End If
    Loop

    ' found a match?
    If Idx > -1 Then
      ' look for similar elements above the matching elements
      I = Idx + 1
      Do While I <= arrSize
        If Math.Abs(FindX - data(I)) <= Tolerance Then
          CountAbove += 1
          I += 1
        Else
          Exit Do
        End If
      Loop

      ' look for similar elements below the matching elements
      I = Idx - 1
      Do While I >= 0
        If Math.Abs(FindX - data(I)) <= Tolerance Then
          CountBelow += 1
          I -= 1
        Else
          Exit Do
        End If
      Loop

    End If

    Return Idx
  End Function

End Class
