﻿Imports System.IO
Imports System.Console
Imports System.Math
' The above two Imports elimitate the need to use Console. and Math. with console I/O and math function calls.

Public Class CDynArrCollection
  Private count As Integer
  Private arrCollect() As CDynArr
  Private sErrMessage As String
  Private ArrHashTable As CHashTable

  Public Sub New()
    count = 0
    sErrMessage = ""
    ArrHashTable = New CHashTable
  End Sub

  Protected Function ParseInt(ByVal s As String) As Integer
    Dim I As Integer

    If Integer.TryParse(s, I) Then
      Return I
    Else
      sErrMessage = "Dynamic array error in converting '" & s & "' into an integer. Conversion yields 0"
      Return 0
    End If
  End Function

  Protected Function ParseDbl(ByVal s As String) As Double
    Dim X As Double

    If Double.TryParse(s, X) Then
      Return X
    Else
      sErrMessage = "Dynamic array error in converting '" & s & "' into an a real value. Conversion yields 1"
      Return 1
    End If
  End Function

  Public Function GetErrorMessage() As String
    Return sErrMessage
  End Function

  Public Sub ResetErrMessage()
    sErrMessage = ""
  End Sub

  Public Function DoesArrNameExist(ByVal varName As String) As Boolean
    Return FindVar(varName) > 0
  End Function

  Public Function GetArrSize(ByVal varName As String) As Integer
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      Return arrCollect(I).arrSize
    Else
      Return -1
    End If

  End Function

  Public Sub Reset(ByVal varName As String)
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      arrCollect(I).Reset()
    Else
      sErrMessage = "Array '" & varName & "' does not exist"
    End If
  End Sub

  Public Sub AddVar(ByVal varName As String, ByVal arrSize As Integer,
                    ByVal bUniform As Boolean, ByVal bIntegers As Boolean,
                    ByVal Low As Double, ByVal High As Double)
    Dim I As Integer

    I = FindVar(varName)
    If I < 1 Then
      count += 1
      ReDim Preserve arrCollect(count)
      arrCollect(count) = New CDynArr(varName.ToUpper, arrSize, bUniform, bIntegers, Low, High)
      arrCollect(count).arrSize = arrSize
      arrCollect(count).arrName = varName.ToUpper
      ArrHashTable.Insert(varName, count)
    Else
      arrCollect(I).Resize(arrSize)
      arrCollect(I).Fill(bUniform, bIntegers, Low, High)
    End If
  End Sub

  Public Sub AddVar(ByVal varName As String, ByVal arrSize As Integer,
                    ByVal First As Double, ByVal Increment As Double)
    Dim I As Integer

    I = FindVar(varName)
    If I < 1 Then
      count += 1
      ReDim Preserve arrCollect(count)
      arrCollect(count) = New CDynArr(varName.ToUpper, arrSize, First, Increment)
      arrCollect(count).arrSize = arrSize
      arrCollect(count).arrName = varName.ToUpper
      ArrHashTable.Insert(varName, count)
    Else
      arrCollect(I).Resize(arrSize)
      arrCollect(I).Fill(First, Increment)
    End If
  End Sub

  Public Sub AddVar(ByVal varName As String, ByVal arrSize As Integer)
    Dim I As Integer

    I = FindVar(varName)
    If I < 1 Then
      count += 1
      ReDim Preserve arrCollect(count)
      arrCollect(count) = New CDynArr(varName.ToUpper, arrSize)
      arrCollect(count).arrSize = arrSize
      arrCollect(count).arrName = varName.ToUpper
      ArrHashTable.Insert(varName, count)
    Else
      arrCollect(I).Resize(arrSize)
    End If
  End Sub

  Public Sub RemoveVar(ByVal varName As String)
    Dim I As Integer

    I = FindVar(varName)
    If I > 0 Then
      If I < Me.count Then
        ' write the Ith element with the last element
        arrCollect(I) = arrCollect(count)
      End If
      ' deallocate the data of the last element
      ReDim Preserve arrCollect(count).data(0)
      count -= 1 ' decrement array size
      ReDim Preserve arrCollect(count)
      ArrHashTable.Delete(varName)
    Else
      WriteLine("Named array " & varName & " does not exist in command ARDEL.")
    End If
  End Sub

  Public Sub SwapVars(ByVal varName1 As String, ByVal varName2 As String)
    Dim I1, I2 As Integer
    Dim s As String

    I1 = FindVar(varName1)
    I2 = FindVar(varName2)

    If I1 = 0 And I2 = 0 Then
      sErrMessage = "Named arrays " & varName1 & " and " & varName2 & " do not exist in command ARSWAP"
    ElseIf I1 = 0 Then
      sErrMessage = "Named array " & varName1 & " does not exist in command ARSWAP"
    ElseIf I2 = 0 Then
      sErrMessage = "Named array " & varName2 & " does not exist in command ARSWAP"
    ElseIf I1 = I1 Then
      sErrMessage = "Why swap the same named array (" & varName1 & ") in command ARSWAP?"
    Else
      ' simply swap the array names
      s = arrCollect(I1).arrName
      arrCollect(I1).arrName = arrCollect(I2).arrName
      arrCollect(I2).arrName = s
    End If

  End Sub

  Public Sub CopyVar(ByVal SourceVar As String, ByVal TargetVar As String)
    Dim I, J, K As Integer

    I = FindVar(SourceVar)

    If I > 0 Then
      J = FindVar(TargetVar)
      ' Target array doe snot exist?
      If J = 0 Then
        AddVar(TargetVar, arrCollect(I).arrSize)
        J = FindVar(TargetVar)
      Else
        ' if exist, just resize it
        Resize(TargetVar, arrCollect(I).arrSize)
      End If
      ' now copy the data
      For K = 0 To arrCollect(I).arrSize
        arrCollect(J).data(K) = arrCollect(I).data(K)
      Next
      arrCollect(J).bIsSorted = arrCollect(I).bIsSorted
      arrCollect(J).StoreIndex = arrCollect(I).StoreIndex
      arrCollect(J).RecallIndex = arrCollect(I).RecallIndex
    Else
      sErrMessage = "Variable " & SourceVar & " does not exist"
    End If

  End Sub

  Public Sub Fill(ByVal varName As String, ByVal bUniform As Boolean, ByVal bIntegers As Boolean,
                    ByVal Low As Double, ByVal High As Double)
    Dim I As Integer

    I = FindVar(varName.ToUpper)
    arrCollect(I).Fill(bUniform, bIntegers, Low, High)
  End Sub

  Public Sub Fill(ByVal varName As String, ByVal First As Double,
                  ByVal Increment As Double)
    Dim I As Integer

    I = FindVar(varName.ToUpper)
    arrCollect(I).Fill(First, Increment)
  End Sub

  Public Sub Resize(ByVal varName As String, ByVal newArrSize As Integer)
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      arrCollect(I).Resize(newArrSize)
      arrCollect(I).arrSize = newArrSize
    Else
      sErrMessage = "Array '" & varName & "' does not exsit"
    End If

  End Sub

  Public Function FindVar(ByVal varName As String) As Integer
    '''Dim I As Integer

    '''varName = varName.ToUpper
    '''For I = 1 To count
    '''  If varName = arrCollect(I).arrName Then
    '''    Return I
    '''  End If
    '''Next
    '''Return 0

    Return ArrHashTable.Find(varName)
  End Function

  Public Function GetSize(ByVal varName As String) As Integer
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      Return arrCollect(I).GetSize()
    Else
      Return -1
    End If

  End Function

  Public Sub Store(ByVal varName As String, ByVal Index As Integer, ByVal X As Double)
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      arrCollect(I).Store(Index, X)
    Else
      AddVar(varName, Index)
      I = FindVar(varName)
      arrCollect(I).Store(Index, X)
    End If
  End Sub

  Public Sub QStore(ByVal varIndex As Integer, ByVal Index As Integer, ByVal X As Double)
    ' This is a quick version where the idnex has been verified. Use it in loops
    arrCollect(varIndex).Store(Index, X)
  End Sub

  Public Sub SetStoreIndex(ByVal varName As String, ByVal Index As Integer)
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      If Index < 0 Or Index > arrCollect(I).arrSize Then Index = 0
      arrCollect(I).StoreIndex = Index
    End If

  End Sub

  Public Function GetStoreIndex(ByVal varName As String) As Integer
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      Return arrCollect(I).StoreIndex
    End If
    Return -1

  End Function

  Public Sub IncStoreIndex(ByVal varName As String, ByVal DeltaIndex As Integer)
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      arrCollect(I).StoreIndex += DeltaIndex
      ' check limit of the srorage index
      arrCollect(I).StoreIndex = Math.Max(0, arrCollect(I).StoreIndex)
      arrCollect(I).StoreIndex = Math.Min(arrCollect(I).arrSize, arrCollect(I).StoreIndex)
    End If

  End Sub

  Public Sub StoreIncIndex(ByVal varName As String, ByVal X As Double)
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      arrCollect(I).StoreIncIndex(X)
    Else
      ' do nothing
    End If
  End Sub

  Public Sub StoreDecIndex(ByVal varName As String, ByVal X As Double)
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      arrCollect(I).StoreDecIndex(X)
    Else
      ' do nothing
    End If
  End Sub

  Public Sub StoreKeepIndex(ByVal varName As String, ByVal X As Double)
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      arrCollect(I).StoreKeepIndex(X)
    Else
      ' do nothing
    End If
  End Sub

  Public Sub StorePlus(ByVal varName As String, ByVal Index As Integer, ByVal X As Double)
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      arrCollect(I).StorePlus(Index, X)
    Else
      AddVar(varName, Index)
      I = FindVar(varName)
      arrCollect(I).StorePlus(Index, X)
    End If
  End Sub

  Public Sub StoreMinus(ByVal varName As String, ByVal Index As Integer, ByVal X As Double)
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      arrCollect(I).StoreMinus(Index, X)
    Else
      AddVar(varName, Index)
      I = FindVar(varName)
      arrCollect(I).StoreMinus(Index, X)
    End If
  End Sub

  Public Sub StoreMult(ByVal varName As String, ByVal Index As Integer, ByVal X As Double)
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      arrCollect(I).StoreMult(Index, X)
    Else
      AddVar(varName, Index)
      I = FindVar(varName)
      arrCollect(I).StoreMult(Index, X)
    End If
  End Sub

  Public Sub StoreDiv(ByVal varName As String, ByVal Index As Integer, ByVal X As Double)
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      arrCollect(I).StoreDiv(Index, X)
    Else
      AddVar(varName, Index)
      I = FindVar(varName)
      arrCollect(I).StoreDiv(Index, X)
    End If
  End Sub

  Public Sub IncRecallIndex(ByVal varName As String, ByVal DeltaIndex As Integer)
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      arrCollect(I).RecallIndex += DeltaIndex
      ' check limit of the srorage index
      arrCollect(I).RecallIndex = Math.Max(0, arrCollect(I).RecallIndex)
      arrCollect(I).RecallIndex = Math.Min(arrCollect(I).arrSize, arrCollect(I).RecallIndex)
    End If

  End Sub

  Public Function GetRecallIndex(ByVal varName As String) As Integer
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      Return arrCollect(I).RecallIndex
    End If
    Return -1

  End Function


  Public Sub SetRecallIndex(ByVal varName As String, ByVal Index As Integer)
    Dim I As Integer

    I = FindVar(varName)

    If I > 0 Then
      If Index < 0 Or Index > arrCollect(I).arrSize Then Index = 0
      arrCollect(I).RecallIndex = Index
    End If
  End Sub


  Public Function RecallIncIndex(ByVal varName As String) As Double
    Dim I As Integer

    I = FindVar(varName.ToUpper)

    If I > 0 Then
      Return arrCollect(I).RecallIncIndex()
    Else
      Return 0
    End If
  End Function

  Public Function RecallDecIndex(ByVal varName As String) As Double
    Dim I As Integer

    I = FindVar(varName.ToUpper)

    If I > 0 Then
      Return arrCollect(I).RecallDecIndex()
    Else
      Return 0
    End If
  End Function

  Public Function RecallKeepIndex(ByVal varName As String) As Double
    Dim I As Integer

    I = FindVar(varName.ToUpper)

    If I > 0 Then
      Return arrCollect(I).RecallKeepIndex()
    Else
      Return 0
    End If
  End Function

  Public Function Recall(ByVal varName As String, ByVal Index As Integer) As Double
    Dim I As Integer

    I = FindVar(varName.ToUpper)

    If I > 0 Then
      Return arrCollect(I).Recall(Index)
    Else
      Return 0
    End If
  End Function

  Public Function QRecall(ByVal varIndex As Integer, ByVal Index As Integer) As Double
    ' quick version of recall. Used in loops
    Return arrCollect(varIndex).Recall(Index)
  End Function

  Protected Function QuickAddVar(ByVal varName As String) As String
    AddVar(varName, 1)
    Return Me.count
  End Function

  Public Function Read1Var(ByVal sFilename As String, ByVal varName As String) As Integer
    Dim I, CountRead As Integer
    Dim TextLine As String
    Dim X As Double
    Dim fileReader As System.IO.StreamReader

    I = FindVar(varName)

    If I = 0 Then I = QuickAddVar(varName)

    If My.Computer.FileSystem.FileExists(sFilename) Then
      fileReader = New StreamReader(sFilename)
      arrCollect(I).Reset()
      CountRead = 0
      Do Until fileReader.EndOfStream
        TextLine = fileReader.ReadLine()
        X = ParseDbl(TextLine)
        arrCollect(I).Store(CountRead, X)
        CountRead += 1
      Loop
      fileReader.Close()
      Return CountRead
    End If
    Return 0
  End Function

  Public Function Read2Vars(ByVal sFilename As String, ByVal varName1 As String, ByVal varName2 As String) As Integer
    Dim I, J, CountRead As Integer
    Dim TextLine As String
    Dim X, Y As Double
    Dim s() As String

    I = FindVar(varName1)
    If I = 0 Then I = QuickAddVar(varName1)

    J = FindVar(varName2)
    If J = 0 Then J = QuickAddVar(varName2)

    If My.Computer.FileSystem.FileExists(sFilename) Then
      Dim fileReader As System.IO.StreamReader
      fileReader = New StreamReader(sFilename)
      arrCollect(I).Reset()
      arrCollect(J).Reset()
      CountRead = 0
      Do Until fileReader.EndOfStream
        TextLine = fileReader.ReadLine()
        s = TextLine.Split(",")

        X = ParseDbl(s(0))
        arrCollect(I).Store(CountRead, X)
        Y = ParseDbl(s(1))
        arrCollect(J).Store(CountRead, Y)
        CountRead += 1
      Loop
      fileReader.Close()
      Return CountRead
    End If
    Return 0
  End Function

  Public Function Read3Vars(ByVal sFilename As String, ByVal varName1 As String,
                            ByVal varName2 As String, ByVal varName3 As String) As Integer
    Dim I, J, K, CountRead As Integer
    Dim TextLine As String
    Dim X, Y, Z As Double
    Dim s() As String

    I = FindVar(varName1)
    If I = 0 Then I = QuickAddVar(varName1)

    J = FindVar(varName2)
    If J = 0 Then J = QuickAddVar(varName2)

    K = FindVar(varName3)
    If K = 0 Then K = QuickAddVar(varName3)

    If My.Computer.FileSystem.FileExists(sFilename) Then
      Dim fileReader As System.IO.StreamReader
      fileReader = New StreamReader(sFilename)
      arrCollect(I).Reset()
      arrCollect(J).Reset()
      arrCollect(K).Reset()
      CountRead = 0
      Do Until fileReader.EndOfStream
        TextLine = fileReader.ReadLine()
        s = TextLine.Split(",")

        X = ParseDbl(s(0))
        arrCollect(I).Store(CountRead, X)
        Y = ParseDbl(s(1))
        arrCollect(J).Store(CountRead, Y)
        Z = ParseDbl(s(2))
        arrCollect(K).Store(CountRead, Z)
        CountRead += 1
      Loop
      fileReader.Close()
      Return CountRead
    End If
    Return 0
  End Function

  Public Function Read4Vars(ByVal sFilename As String, ByVal varName1 As String,
                            ByVal varName2 As String, ByVal varName3 As String,
                            ByVal varName4 As String) As Boolean
    Dim I, J, K, L, CountRead As Integer
    Dim TextLine As String
    Dim X, Y, Z, T As Double
    Dim s() As String

    I = FindVar(varName1)
    If I = 0 Then I = QuickAddVar(varName1)

    J = FindVar(varName2)
    If J = 0 Then J = QuickAddVar(varName2)

    K = FindVar(varName3)
    If K = 0 Then K = QuickAddVar(varName3)


    L = FindVar(varName4)
    If L = 0 Then L = QuickAddVar(varName4)

    If My.Computer.FileSystem.FileExists(sFilename) Then
      Dim fileReader As System.IO.StreamReader
      fileReader = New StreamReader(sFilename)
      arrCollect(I).Reset()
      arrCollect(J).Reset()
      arrCollect(K).Reset()
      arrCollect(L).Reset()
      CountRead = 0
      Do Until fileReader.EndOfStream
        TextLine = fileReader.ReadLine()
        s = TextLine.Split(",")
        X = ParseDbl(s(0))
        arrCollect(I).Store(CountRead, X)
        Y = ParseDbl(s(1))
        arrCollect(J).Store(CountRead, Y)
        Z = ParseDbl(s(2))
        arrCollect(K).Store(CountRead, Z)
        T = ParseDbl(s(3))
        arrCollect(L).Store(CountRead, T)
        CountRead += 1
      Loop
      fileReader.Close()
      Return CountRead
    End If
    Return 0
  End Function

  Public Function ReadNVars(ByVal sFilename As String, ByVal N As Integer, ByVal varNames() As String) As Integer
    Dim I, VarIdx(), CountRead As Integer
    Dim TextLine As String
    Dim X As Double
    Dim s() As String

    Try
      ReDim VarIdx(N)
      For I = 1 To N
        VarIdx(I) = FindVar(varNames(I + 2))
        If VarIdx(I) = 0 Then VarIdx(I) = QuickAddVar(varNames(I + 2))
      Next

      If My.Computer.FileSystem.FileExists(sFilename) Then
        Dim fileReader As System.IO.StreamReader
        fileReader = New StreamReader(sFilename)
        For I = 1 To N
          arrCollect(VarIdx(I)).Reset()
        Next
        CountRead = 0
        Do Until fileReader.EndOfStream
          TextLine = fileReader.ReadLine()
          s = TextLine.Split(",")
          For I = 1 To N
            X = ParseDbl(s(I - 1))
            arrCollect(VarIdx(I)).Store(CountRead, X)
          Next
          CountRead += 1
        Loop
        fileReader.Close()
        Return CountRead
      End If
    Catch
    Catch
      sErrMessage = "Error in READNVARS command"
    End Try
    Return 0
  End Function

  Public Function Write1Var(ByVal sFilename As String, ByVal varName As String) As Integer
    Dim I, VarIdx As Integer

    Try
      VarIdx = FindVar(varName)

      If VarIdx > 0 Then
        Dim fileWriter As System.IO.StreamWriter
        fileWriter = New StreamWriter(sFilename)
        For I = 0 To arrCollect(VarIdx).arrSize
          fileWriter.WriteLine(arrCollect(VarIdx).Recall(I))
        Next
        fileWriter.Close()
        Return arrCollect(VarIdx).arrSize + 1
      Else
        WriteLine("Variable " & varName & " does not exist in command WRITE1VAR.")
      End If
    Catch
      WriteLine("File " & sFilename & " generated an error in command WRITE1VAR.")
    End Try
    Return 0
  End Function


  Public Function Write2Vars(ByVal sFilename As String, ByVal varName1 As String, ByVal varName2 As String) As Integer
    Dim I, J, N As Integer

    I = FindVar(varName1)
    J = FindVar(varName2)
    Try
      If I > 0 AndAlso J > 0 AndAlso arrCollect(I).arrSize = arrCollect(J).arrSize Then
        Dim fileWriter As System.IO.StreamWriter
        fileWriter = New StreamWriter(sFilename)
        For N = 0 To arrCollect(I).arrSize
          fileWriter.WriteLine("{0},{1}", arrCollect(I).Recall(N), arrCollect(J).Recall(N))
        Next
        fileWriter.Close()
        Return arrCollect(I).arrSize + 1
      Else
        WriteLine("Check variables " & varName1 & ", " & varName2 & " in command WRITE1VARS.")
      End If
    Catch
      WriteLine("File " & sFilename & " generated an error in command WRITE2VARS.")
    End Try
    Return 0
  End Function

  Public Function Write3Vars(ByVal sFilename As String, ByVal varName1 As String,
                            ByVal varName2 As String, ByVal varName3 As String) As Integer
    Dim I, J, K, N As Integer
    Dim bOK As Boolean

    I = FindVar(varName1)
    J = FindVar(varName2)
    K = FindVar(varName3)
    bOK = I > 0 AndAlso J > 0 AndAlso K > 0
    bOK = bOK AndAlso arrCollect(I).arrSize = arrCollect(J).arrSize
    bOK = bOK AndAlso arrCollect(I).arrSize = arrCollect(K).arrSize

    Try
      If bOK Then
        Dim fileWriter As System.IO.StreamWriter
        fileWriter = New StreamWriter(sFilename)
        For N = 0 To arrCollect(I).arrSize
          fileWriter.WriteLine("{0}, {1}, {2}", arrCollect(I).Recall(N), arrCollect(J).Recall(N),
                             arrCollect(K).Recall(N))
        Next
        fileWriter.Close()
        Return arrCollect(I).arrSize + 1
      Else
        WriteLine("Check variables " & varName1 & ", " & varName2 & ", " & varName3 & " in command WRITE3VARS.")
      End If
    Catch
      WriteLine("File " & sFilename & " generated an error in command WRITE3VARS.")
    End Try
    Return 0
  End Function

  Public Function Write4Vars(ByVal sFilename As String, ByVal varName1 As String,
                            ByVal varName2 As String, ByVal varName3 As String,
                            ByVal varName4 As String) As Integer
    Dim I, J, K, L, N As Integer
    Dim bOK As Boolean

    I = FindVar(varName1)
    J = FindVar(varName2)
    K = FindVar(varName3)
    L = FindVar(varName4)
    bOK = I > 0 AndAlso J > 0 AndAlso K > 0 AndAlso L > 0
    bOK = bOK AndAlso arrCollect(I).arrSize = arrCollect(J).arrSize
    bOK = bOK AndAlso arrCollect(I).arrSize = arrCollect(K).arrSize
    bOK = bOK AndAlso arrCollect(I).arrSize = arrCollect(L).arrSize

    Try
      If bOK Then
        Dim fileWriter As System.IO.StreamWriter
        fileWriter = New StreamWriter(sFilename)
        For N = 0 To arrCollect(I).arrSize
          fileWriter.WriteLine("{0}, {1}, {2}, {3}", arrCollect(I).Recall(N), arrCollect(J).Recall(N),
                             arrCollect(K).Recall(N), arrCollect(L).Recall(N))
        Next
        fileWriter.Close()
        Return arrCollect(I).arrSize + 1
      Else
        WriteLine("Check variables " & varName1 & ", " & varName2 & ", " & varName3 & ", " & varName4 & " in command WRITE4VARS.")
      End If
    Catch
      WriteLine("File " & sFilename & " generated an error in command WRITE4VARS.")
    End Try
    Return 0
  End Function


  Public Function WriteNVars(ByVal sFilename As String, ByVal N As Integer, ByVal varNames() As String) As Integer
    Dim I, J, VarIdx() As Integer

    Try
      ReDim VarIdx(N)
      For I = 1 To N
        VarIdx(I) = FindVar(varNames(I + 2))
        If VarIdx(I) = 0 Then
          WriteLine("One or more variables do not exist in the command WRITENVARS.")
          Return 0
        End If
      Next

      For I = 1 To N - 1
        If arrCollect(VarIdx(I)).arrSize <> arrCollect(VarIdx(I + 1)).arrSize Then
          WriteLine("The variables do not have the same number of elements in the command WRITENVARS.")
          Return 0
        End If
      Next

      Dim fileWriter As System.IO.StreamWriter
      fileWriter = New StreamWriter(sFilename)
      For I = 0 To arrCollect(VarIdx(1)).arrSize
        For J = 1 To N - 1
          fileWriter.Write("{0}, ", arrCollect(VarIdx(J)).Recall(I))
        Next
        fileWriter.WriteLine("{0}", arrCollect(VarIdx(N)).Recall(I))
      Next

      fileWriter.Close()
      Return arrCollect(VarIdx(1)).arrSize + 1
    Catch
      sErrMessage = "Error in WRITENVARS command"
    End Try
    Return 0
  End Function

  Public Sub Var2Array(ByVal varName As String, ByVal FirstInVar As Integer,
                      ByVal LastInVar As Integer, ByRef anArray() As Double)
    Dim I, J, N As Integer

    N = LastInVar - FirstInVar + 1
    ReDim anArray(N)

    J = 0
    For I = FirstInVar To LastInVar
      anArray(J) = Recall(varName, I)
      J += 1
    Next

  End Sub

  Public Function SortAscending(ByVal sVarName As String) As Boolean
    Dim I As Integer

    I = FindVar(sVarName)
    If I > 0 Then
      arrCollect(I).SortAscending()
      Return True
    Else
      Return False
    End If
  End Function

  Public Function SortDescending(ByVal sVarName As String) As Boolean
    Dim I As Integer

    I = FindVar(sVarName)
    If I > 0 Then
      arrCollect(I).SortAscending()
      Return True
    Else
      Return False
    End If
  End Function

  Public Function Search(ByVal sVarname As String, ByVal FindX As Double, ByVal Optional First As Integer = 0,
                         ByVal Optional Tolerance As Double = 0,
                                 ByVal Optional SearchMode As Integer = 0) As Integer
    Dim I As Integer

    I = FindVar(sVarname)
    If I > 0 Then
      Return arrCollect(I).Search(FindX, First, Tolerance, SearchMode)
    Else
      Return -2
    End If

  End Function

  Public Function MiddleFirstSearch(ByVal sVarname As String, ByVal FindX As Double,
                                    ByVal Optional Tolerance As Double = 0,
                                    ByVal Optional SearchMode As Integer = 0) As Integer
    Dim I As Integer

    I = FindVar(sVarname)
    If I > 0 Then
      Return arrCollect(I).MiddleFirstSearch(FindX, Tolerance, SearchMode)
    Else
      Return -2
    End If

  End Function

  Public Function CircularSearch(ByVal sVarname As String, ByVal FindX As Double, ByVal Optional First As Integer = 0,
                                 ByVal Optional Tolerance As Double = 0,
                                 ByVal Optional SearchMode As Integer = 0) As Integer
    Dim I As Integer

    I = FindVar(sVarname)
    If I > 0 Then
      Return arrCollect(I).CircularSearch(FindX, First, Tolerance, SearchMode)
    Else
      Return -2
    End If

  End Function

  Public Function HeuristicSearch(ByVal sVarname As String, ByRef NewPosition As Integer,
                                  ByVal FindX As Double,
                                  ByVal Optional Shift As Integer = 1,
                                  ByVal Optional First As Integer = 0,
                                  ByVal Optional Tolerance As Double = 0) As Integer
    Dim I As Integer

    I = FindVar(sVarname)
    If I > 0 Then
      Return arrCollect(I).HeuristicSearch(FindX, NewPosition, Shift, First, Tolerance)
    Else
      Return -2
    End If

  End Function

  Public Function BinarySearch(ByVal sVarname As String, ByVal FindX As Double,
                               ByVal Tolerance As Double,
                               ByRef CountAbove As Integer, ByRef CountBelow As Integer) As Integer
    Dim I As Integer

    I = FindVar(sVarname)
    If I > 0 Then
      Return arrCollect(I).BinarySearch(FindX, Tolerance, CountABove, CountBelow)
    Else
      Return -2
    End If

  End Function
End Class
