﻿Imports System.IO
Imports System.Threading
Imports System.Console
Imports System.Math
' The above two Imports elimitate the need to use Console. and Math. with console I/O and math function calls.

Public Class ProgRPNCalc
  Inherits RPNCalc

  Private Const DOUBLE_QUOTE As String = Chr(34)
  Private Const INIT_MAX_LOCAL_VARS As Integer = 10
  Private Const INIT_NUM_LOCAL_REGS As Integer = 10
  Private MAX_GOSUB = 100
  Private MAX_FLAGS = 100
  Private NumLocalVars As Integer
  Private sVarsNames() As String
  Private VarsValues() As Double
  Private VarsHashTable As New CHashTable
  Private VarsCount As Integer
  Private sAlphaVarsNames() As String
  Private AlphaVarsValues() As String
  Private AlphaVarsHashTable As New CHashTable
  Private AlphaVarsCount As Integer
  Private sLines() As String
  Private sLbls() As String
  Private LblsIndex() As Integer
  Private RtnPtrs(MAX_GOSUB) As Integer
  Private RtnCount As Integer
  Private Flags(MAX_FLAGS) As Boolean
  Private LblsCount As Integer
  Private PrgmPtr As Integer
  Private bUpdatePrgmPtr As Boolean
  Private Alpha As String
  Private bStop, bOK As Boolean
  Private bShowStack As Boolean = True ' is meant to be cleared and set ONLY by the methods fxSolve and fxIntegrate
  Private bOnErrReumeNext, bOnErrGoto As Boolean, ErrHandlingLine As Integer
  Private objLocalVars, objVarArrs As CDynArrCollection
  Private BestXPower, BestYPower, BestZPower As Double

  Public Function CanExecute() As Boolean
    Return bOK
  End Function

  Private Function EndsWith(ByVal tok As String, ByVal sFind As String, ByRef intVal As Integer) As Integer
    Dim L As Integer = tok.Length
    Dim c As String

    c = tok.Substring(L - 1, 1)
    If c = "?" And tok.Substring(L - 2, 2) = sFind Then
      intVal = 1
      Return True
    ElseIf c >= "0" And c <= "9" Then
      If tok.Substring(L - 3, 2) = sFind Then
        intVal = Max(1, ParseInt(c))
        Return True
      End If
    End If
    intVal = 0
    Return False
  End Function

  Private Function StrReverse(ByVal s As String) As String
    Dim i As Integer
    Dim Len As Integer = s.Length
    Dim sRes As String = ""

    If Len = 0 Then Return sRes

    For i = Len - 1 To 0 Step -1
      sRes &= s.Substring(i, 1)
    Next
    Return sRes
  End Function

  Private Function StrFilter(ByVal strMain As String, ByVal sFilterCharSet As String)
    Dim s As String = ""
    Dim I As Integer

    For I = 0 To strMain.Length - 1
      If sFilterCharSet.IndexOf(strMain.Substring(I, 1)) < 0 Then s &= strMain.Substring(I, 1)
    Next

    Return s
  End Function

  Public Sub New(ByVal sInputFile As String)
    Dim I As Integer

    ' clear all flags
    For I = 1 To MAX_FLAGS
      Flags(I) = False
    Next
    VarsCount = 0
    AlphaVarsCount = 0
    sLines = File.ReadAllLines(sInputFile)
    For I = 0 To sLines.Length - 1
      If Not (sLines(I).StartsWith("'") Or sLines(I).StartsWith(DOUBLE_QUOTE) Or sLines(I).StartsWith("|-")) Then
        sLines(I) = sLines(I).ToUpper
      End If
      sLines(I) = sLines(I).Replace("LEFT", "LEFT$")
      sLines(I) = sLines(I).Replace("RIGHT", "RIGHT$")
      sLines(I) = sLines(I).Replace("MID", "MID$")
      sLines(I) = sLines(I).Replace("TRIM", "TRIM$")
      sLines(I) = sLines(I).Replace("REV", "REV$")
      sLines(I) = sLines(I).Replace("UCASE", "UCASE$")
      sLines(I) = sLines(I).Replace("LCASE", "LCASE$")
      sLines(I) = sLines(I).Replace("SWAPCASE", "SWAPCASE$")
      sLines(I) = sLines(I).Replace("POS", "POS$") ' also works for LASTPOS
      sLines(I) = sLines(I).Replace("REPLACE", "REPLACE$")
      sLines(I) = sLines(I).Replace("FILTER", "FILTER$")
    Next
    ScanLabels()
    If bOK And LblsCount > 1 Then SortLabels()
    bOK = bOK AndAlso CheckGotos()
    objVarArrs = New CDynArrCollection()
    objLocalVars = New CDynArrCollection()

    ' add thw default (and empty) initial local variables
    For I = 0 To INIT_MAX_LOCAL_VARS
      objLocalVars.AddVar(CStr(I), INIT_NUM_LOCAL_REGS)
    Next

  End Sub

  Public Function GetErrorMessage() As String
    Return sErrMessage
  End Function

  Public Sub ScanLabels()
    Dim aLine, sLbL As String
    Dim LineNumber As Integer = 0
    Dim I As Integer

    LblsCount = 0
    bOK = True
    For Each aLine In sLines
      If aLine.StartsWith("LBL") Then
        sLbL = aLine.Substring(3).ToUpper.Trim
        If IsNumeric(sLbL) Then
          ' does it have extra leading zeros?
          Do While sLbL.Substring(0, 1) = "0" And sLbL.Length > 1
            sLbL = sLbL.Substring(1)
          Loop
        End If

        I = FindLbl(sLbL) ' search for a duplicate label
        If I = 0 Then
          LblsCount += 1
          ReDim Preserve sLbls(LblsCount)
          ReDim Preserve LblsIndex(LblsCount)
          sLbls(LblsCount) = sLbL
          LblsIndex(LblsCount) = LineNumber
        Else
          bOK = False
          WriteLine("Duplicate label {0} at line {1}", sLbL, LineNumber + 1)
          Exit Sub
        End If
      End If
      LineNumber += 1
    Next
  End Sub

  Public Sub SortLabels()
    Dim I, J, Offset, IntBuffer As Integer
    Dim bInOrder As Boolean
    Dim Buffer As String

    Offset = LblsCount
    Do While Offset > 1
      Offset = Fix(Offset * 5.0# / 11.0#)
      If Offset < 1 Then Offset = 1
      Do
        bInOrder = True
        For I = 1 To LblsCount - Offset
          J = I + Offset
          If sLbls(I) > sLbls(J) Then
            bInOrder = False
            ' swap label names
            Buffer = sLbls(I)
            sLbls(I) = sLbls(J)
            sLbls(J) = Buffer
            ' swap the line numbers associated with the labels
            IntBuffer = LblsIndex(I)
            LblsIndex(I) = LblsIndex(J)
            LblsIndex(J) = IntBuffer
          End If
        Next I
      Loop Until bInOrder
    Loop
  End Sub

  Protected Function FindLbl(ByVal sFind As String) As Integer
    Dim Hi, Low, Median As Integer
    Dim X As String

    Low = 1
    Hi = LblsCount
    sFind = sFind.ToUpper()
    ' is the label numeric?
    If IsNumeric(sFind) Then
      ' does it have extra leading zeros?
      Do While sFind.Substring(0, 1) = "0" And sFind.Length > 1
        sFind = sFind.Substring(1)
      Loop
    End If

    Do While Hi >= Low
      Median = (Low + Hi) / 2
      X = sLbls(Median).ToUpper
      If sFind > X Then
        Low = Median + 1
      ElseIf sFind < X Then
        Hi = Median - 1
      Else
        Return Median
      End If
    Loop
    Return 0
  End Function

  Private Sub ClearFlags()
    Dim I As Integer

    For I = 0 To MAX_FLAGS
      Flags(I) = False
    Next I
  End Sub

  Public Function ExecLblMain() As Boolean

    If FindLbl("MAIN") > 0 Then
      ExecProgram("MAIN")
      Return True
    End If
    Return False
  End Function

  Function ForLoop(ByVal LoopCtrlVar As String) As Boolean
    Dim CurrentVal, FinalVal, IncVal As Double
    Dim s, StrArr() As String
    Dim I As Integer
    Dim bResult As Boolean

    Try
      ' get the two of three numbers
      s = GetAlphaVarValue(LoopCtrlVar)
      StrArr = s.Split(",")
      CurrentVal = ParseDbl(StrArr(0))
      FinalVal = ParseDbl(StrArr(1))
      Try
        IncVal = ParseDbl(StrArr(2))
        If IncVal = 0 Then IncVal = 1
      Catch
        IncVal = 1
      End Try
      CurrentVal += IncVal
      If IncVal > 0 Then
        bResult = If(CurrentVal <= FinalVal, True, False)
      Else
        bResult = If(CurrentVal >= FinalVal, True, False)
      End If
      If bResult Then
        ' rebuild loop control values string
        s = CStr(CurrentVal) & "," & CStr(FinalVal) & "," & CStr(IncVal)
        I = FindAlphaVar(LoopCtrlVar)
        AlphaVarsValues(I) = s
      End If
      Return bResult
    Catch
      sErrMessage = "Error in FOR command"
    End Try
    Return False
  End Function

  Function ForRcl(ByVal incVarName As String) As Double
    Dim CurrentVal As Double
    Dim s, StrArr() As String

    Try
      ' get the two of three numbers
      s = GetAlphaVarValue(incVarName)
      StrArr = s.Split(",")
      CurrentVal = ParseDbl(StrArr(0))
      Return CurrentVal
    Catch
      sErrMessage = "Error in FORCL command"
    End Try
    Return 0
  End Function

  Private Sub ForSet(ByVal VarName As String, ByVal xFirst As Double, ByVal xLast As Double, ByVal xInc As Double)
    Dim I As Integer
    Dim s As String

    I = FindAlphaVar(VarName)
    If I < 1 Then AddAlphaVar(VarName)

    Try
      s = Alpha
      Alpha = CStr(xFirst) & "," & CStr(xLast) & "," & CStr(xInc)
      StoAlphaVarValue(VarName)
      Alpha = s
    Catch
      sErrMessage = "Error in FORSET command"
    End Try
  End Sub

  Function Dse(ByVal s As String) As Boolean
    Dim X As Decimal
    Dim I, aaa, bbb, cc As Integer

    If Integer.TryParse(s, I) Then
      I = Abs(I)
    Else
      sErrMessage = "Error in string to numeric conversion in the DSE command"
      Return True ' result is irrelevant but required
      Exit Function
    End If

    X = GetMem(I)
    If GetErrMessage() <> "" Then Return True
    aaa = Fix(X)
    X = X - aaa
    X = SCALE * X
    bbb = Fix(X)
    X = X - bbb
    cc = Fix(100 * X)
    If cc < 1 Then cc = 1

    aaa -= cc
    X = aaa + bbb / SCALE + cc / SCALE / 100
    SetMem(I, X)
    Return aaa = bbb
  End Function

  Function DseInd(ByVal s As String) As Boolean
    Dim X As Decimal
    Dim I, J, aaa, bbb, cc As Integer

    If Integer.TryParse(s, I) Then
      I = Abs(I)
    Else
      sErrMessage = "Error in string to numeric conversion in the DSEIND command"
      Return True ' result is irrelevant but required
      Exit Function
    End If

    J = GetMem(I)
    X = GetMem(J)
    If GetErrMessage() <> "" Then Return True
    aaa = Fix(X)
    X = X - aaa
    X = SCALE * X
    bbb = Fix(X)
    X = X - bbb
    cc = Fix(100 * X)
    If cc < 1 Then cc = 1

    aaa -= cc
    X = aaa + bbb / SCALE + cc / SCALE / 100
    SetMem(J, X)
    Return aaa = bbb
  End Function

  Function Dsl(ByVal s As String) As Boolean
    Dim X As Decimal
    Dim I, aaa, bbb, cc As Integer

    If Integer.TryParse(s, I) Then
      I = Abs(I)
    Else
      sErrMessage = "Error in string to numeric conversion in the DSL command"
      Return True ' result is irrelevant but required
      Exit Function
    End If

    X = GetMem(I)
    If GetErrMessage() <> "" Then Return True
    aaa = Fix(X)
    X = X - aaa
    X = SCALE * X
    bbb = Fix(X)
    X = X - bbb
    X = SCALE * X
    cc = Fix(100 * X)
    If cc < 1 Then cc = 1

    aaa -= cc
    If aaa >= bbb Then
      X = aaa + bbb / SCALE + cc / SCALE / 100
      SetMem(I, X)
    End If
    Return aaa < bbb
  End Function

  Function DslInd(ByVal s As String) As Boolean
    Dim X As Decimal
    Dim I, J, aaa, bbb, cc As Integer

    If Integer.TryParse(s, I) Then
      I = Abs(I)
    Else
      sErrMessage = "Error in string to numeric conversion in the DSLIND command"
      Return True ' result is irrelevant but required
      Exit Function
    End If

    J = GetMem(I)
    X = GetMem(J)
    If GetErrMessage() <> "" Then Return True
    aaa = Fix(X)
    X = X - aaa
    X = SCALE * X
    bbb = Fix(X)
    X = X - bbb
    cc = Fix(100 * X)
    If cc < 1 Then cc = 1

    aaa -= cc
    If aaa >= bbb Then
      X = aaa + bbb / SCALE + cc / SCALE / 100
      SetMem(J, X)
    End If
    Return aaa < bbb
  End Function

  Function Isg(ByVal s As String) As Boolean
    Dim X As Decimal
    Dim I, aaa, bbb, cc As Integer

    If Integer.TryParse(s, I) Then
      I = Abs(I)
    Else
      sErrMessage = "Error in string to numeric conversion in the ISG command"
      Return True ' result is irrelevant but required
      Exit Function
    End If

    X = GetMem(I)
    If GetErrMessage() <> "" Then Return True
    aaa = Fix(X)
    X = X - aaa
    X = SCALE * X
    bbb = Fix(X)
    X = X - bbb
    cc = Fix(100 * X)
    If cc < 1 Then cc = 1

    aaa += cc
    X = aaa + bbb / SCALE + cc / SCALE / 100
    SetMem(I, X)
    Return aaa > bbb
  End Function

  Function IsgInd(ByVal s As String) As Boolean
    Dim X As Decimal
    Dim I, J, aaa, bbb, cc As Integer

    If Integer.TryParse(s, I) Then
      I = Abs(I)
    Else
      sErrMessage = "Error in string to numeric conversion in the ISGIND command"
      Return True ' result is irrelevant but required
      Exit Function
    End If

    J = GetMem(I)
    X = GetMem(J)
    If GetErrMessage() <> "" Then Return True
    aaa = Fix(X)
    X = X - aaa
    X = SCALE * X
    bbb = Fix(X)
    X = X - bbb
    cc = Fix(100 * X)
    If cc < 1 Then cc = 1

    aaa += cc
    X = aaa + bbb / SCALE + cc / SCALE / 100
    SetMem(J, X)
    Return aaa > bbb
  End Function

  Function Dse41(ByVal s As String) As Boolean
    Dim X As Decimal
    Dim I, aaa, bbb, cc As Integer

    If Integer.TryParse(s, I) Then
      I = Abs(I)
    Else
      sErrMessage = "Error in string to numeric conversion in the DSE41 command"
      Return True ' result is irrelevant but required
      Exit Function
    End If

    X = GetMem(I)
    If GetErrMessage() <> "" Then Return True
    aaa = Fix(X)
    X = X - aaa
    X = SCALE41 * X
    bbb = Fix(X)
    X = X - bbb
    cc = Fix(100 * X)
    If cc < 1 Then cc = 1

    aaa -= cc
    X = aaa + bbb / SCALE41 + cc / SCALE41 / 100
    SetMem(I, X)
    Return aaa = bbb
  End Function

  Function DseInd41(ByVal s As String) As Boolean
    Dim X As Decimal
    Dim I, J, aaa, bbb, cc As Integer

    If Integer.TryParse(s, I) Then
      I = Abs(I)
    Else
      sErrMessage = "Error in string to numeric conversion in the DSEIND41 command"
      Return True ' result is irrelevant but required
      Exit Function
    End If

    J = GetMem(I)
    X = GetMem(J)
    If GetErrMessage() <> "" Then Return True
    aaa = Fix(X)
    X = X - aaa
    X = SCALE41 * X
    bbb = Fix(X)
    X = X - bbb
    cc = Fix(100 * X)
    If cc < 1 Then cc = 1

    aaa -= cc
    X = aaa + bbb / SCALE41 + cc / SCALE41 / 100
    SetMem(J, X)
    Return aaa = bbb
  End Function

  Function Dsl41(ByVal s As String) As Boolean
    Dim X As Decimal
    Dim I, aaa, bbb, cc As Integer

    If Integer.TryParse(s, I) Then
      I = Abs(I)
    Else
      sErrMessage = "Error in string to numeric conversion in the DSL41 command"
      Return True ' result is irrelevant but required
      Exit Function
    End If

    X = GetMem(I)
    If GetErrMessage() <> "" Then Return True
    aaa = Fix(X)
    X = X - aaa
    X = SCALE41 * X
    bbb = Fix(X)
    X = X - bbb
    X = SCALE41 * X
    cc = Fix(100 * X)
    If cc < 1 Then cc = 1

    aaa -= cc
    If aaa >= bbb Then
      X = aaa + bbb / SCALE41 + cc / SCALE41 / 100
      SetMem(I, X)
    End If
    Return aaa < bbb
  End Function

  Function DslInd41(ByVal s As String) As Boolean
    Dim X As Decimal
    Dim I, J, aaa, bbb, cc As Integer

    If Integer.TryParse(s, I) Then
      I = Abs(I)
    Else
      sErrMessage = "Error in string to numeric conversion in the DSLIND41 command"
      Return True ' result is irrelevant but required
      Exit Function
    End If

    J = GetMem(I)
    X = GetMem(J)
    If GetErrMessage() <> "" Then Return True
    aaa = Fix(X)
    X = X - aaa
    X = SCALE41 * X
    bbb = Fix(X)
    X = X - bbb
    cc = Fix(100 * X)
    If cc < 1 Then cc = 1

    aaa -= cc
    If aaa >= bbb Then
      X = aaa + bbb / SCALE41 + cc / SCALE41 / 100
      SetMem(J, X)
    End If
    Return aaa < bbb
  End Function

  Function Isg41(ByVal s As String) As Boolean
    Dim X As Decimal
    Dim I, aaa, bbb, cc As Integer

    If Integer.TryParse(s, I) Then
      I = Abs(I)
    Else
      sErrMessage = "Error in string to numeric conversion in the ISG41 command"
      Return True ' result is irrelevant but required
      Exit Function
    End If

    X = GetMem(I)
    If GetErrMessage() <> "" Then Return True
    aaa = Fix(X)
    X = X - aaa
    X = SCALE41 * X
    bbb = Fix(X)
    X = X - bbb
    cc = Fix(100 * X)
    If cc < 1 Then cc = 1

    aaa += cc
    X = aaa + bbb / SCALE41 + cc / SCALE41 / 100
    SetMem(I, X)
    Return aaa > bbb
  End Function

  Function IsgInd41(ByVal s As String) As Boolean
    Dim X As Decimal
    Dim I, J, aaa, bbb, cc As Integer

    If Integer.TryParse(s, I) Then
      I = Abs(I)
    Else
      sErrMessage = "Error in string to numeric conversion in the ISGIND41 command"
      Return True ' result is irrelevant but required
      Exit Function
    End If

    J = GetMem(I)
    X = GetMem(J)
    If GetErrMessage() <> "" Then Return True
    aaa = Fix(X)
    X = X - aaa
    X = SCALE41 * X
    bbb = Fix(X)
    X = X - bbb
    cc = Fix(100 * X)
    If cc < 1 Then cc = 1

    aaa += cc
    X = aaa + bbb / SCALE41 + cc / SCALE41 / 100
    SetMem(J, X)
    Return aaa > bbb
  End Function


  Private Function IsFlagSet(ByVal s As String, ByVal Optional bInd As Boolean = False) As Boolean
    Dim I As Integer

    I = Abs(ParseInt(s))
    If bInd Then I = Abs(GetMem(I))
    If I >= 0 And I <= MAX_FLAGS Then
      Return Flags(I) = True
    Else
      If sErrMessage.Length = 0 Then sErrMessage = "Index out Of range In IsFlagSet"
      Return False
    End If
  End Function

  Private Function IsFlagSetThenClear(ByVal s As String, ByVal Optional bInd As Boolean = False) As Boolean
    Dim I As Integer
    Dim bBool As Boolean

    I = Abs(ParseInt(s))
    If bInd Then I = Abs(GetMem(I))
    If I >= 0 And I <= MAX_FLAGS Then
      bBool = Flags(I)
      Flags(I) = False
      Return bBool
    Else
      If sErrMessage.Length = 0 Then sErrMessage = "Index out Of range In IsFlagSetThenClear"
      Return False
    End If
  End Function

  Private Function IsFlagSetThenSet(ByVal s As String, ByVal Optional bInd As Boolean = False) As Boolean
    Dim I As Integer
    Dim bBool As Boolean

    I = Abs(ParseInt(s))
    If bInd Then I = Abs(GetMem(I))
    If I >= 0 And I <= MAX_FLAGS Then
      bBool = Flags(I)
      Flags(I) = True
      Return bBool
    Else
      If sErrMessage.Length = 0 Then sErrMessage = "Index out Of range In IsFlagSetThenSet"
      Return False
    End If
  End Function

  Private Function IsFlagSetThenFlip(ByVal s As String, ByVal Optional bInd As Boolean = False) As Boolean
    Dim I As Integer
    Dim bBool As Boolean

    I = Abs(ParseInt(s))
    If bInd Then I = Abs(GetMem(I))
    If I >= 0 And I <= MAX_FLAGS Then
      bBool = Flags(I)
      Flags(I) = Not Flags(I)
      Return bBool
    Else
      If sErrMessage.Length = 0 Then sErrMessage = "Index out Of range In IsFlagSetThenFlip"
      Return False
    End If
  End Function

  Private Function IsFlagClear(ByVal s As String, ByVal Optional bInd As Boolean = False) As Boolean
    Dim I As Integer

    I = Abs(ParseInt(s))
    If bInd Then I = Abs(GetMem(I))
    If I >= 0 And I <= MAX_FLAGS Then
      Return Not Flags(I)
    Else
      If sErrMessage.Length = 0 Then sErrMessage = "Index out Of range In IsFlagClear"
      Return False
    End If
  End Function

  Private Function IsFlagClearThenClear(ByVal s As String, ByVal Optional bInd As Boolean = False) As Boolean
    Dim I As Integer
    Dim bBool As Boolean

    I = Abs(ParseInt(s))
    If bInd Then I = Abs(GetMem(I))
    If I >= 0 And I <= MAX_FLAGS Then
      bBool = Flags(I)
      Flags(I) = False
      Return Not bBool
    Else
      If sErrMessage.Length = 0 Then sErrMessage = "Index out Of range In IsFlagClearThenClear"
      Return False
    End If
  End Function

  Private Function IsFlagClearThenSet(ByVal s As String, ByVal Optional bInd As Boolean = False) As Boolean
    Dim I As Integer
    Dim bBool As Boolean

    I = Abs(ParseInt(s))
    If bInd Then I = Abs(GetMem(I))
    If I >= 0 And I <= MAX_FLAGS Then
      bBool = Flags(I)
      Flags(I) = True
      Return Not bBool
    Else
      If sErrMessage.Length = 0 Then sErrMessage = "Index out Of range In IsFlagClearThenSet"
      Return False
    End If
  End Function

  Private Function IsFlagClearThenFlip(ByVal s As String, ByVal Optional bInd As Boolean = False) As Boolean
    Dim I As Integer
    Dim bBool As Boolean

    I = Abs(ParseInt(s))
    If bInd Then I = Abs(GetMem(I))
    If I >= 0 And I <= MAX_FLAGS Then
      bBool = Flags(I)
      Flags(I) = Not Flags(I)
      Return Not bBool
    Else
      If sErrMessage.Length = 0 Then sErrMessage = "Index out Of range In IsFlagClearThenFlip"
      Return False
    End If
  End Function


  Private Function CheckGotos() As Boolean
    Dim aLine, aLbl As String

    For Each aLine In sLines
      If (aLine.StartsWith("GTO") Or aLine.StartsWith("GSB")) And aLine.IndexOf("IND") < 0 Then
        aLbl = aLine.Substring(4)
        If FindLbl(aLbl) < 1 Then
          Return False
        End If
      End If
    Next
    Return True
  End Function


  Private Function FindVar(ByVal VarName As String) As Integer

    Return VarsHashTable.Find(VarName)
  End Function

  Private Sub AddVar(ByVal VarName As String, ByVal VarValue As Double)
    VarsCount += 1
    ReDim Preserve sVarsNames(VarsCount), VarsValues(VarsCount)
    sVarsNames(VarsCount) = VarName.ToUpper
    VarsValues(VarsCount) = VarValue
    VarsHashTable.Insert(VarName, VarsCount)
  End Sub

  Private Function RclVarValue(ByVal VarName As String) As Double
    Dim I As Integer

    I = FindVar(VarName)
    If I > 0 Then
      Return VarsValues(I)
    Else
      sErrMessage = "Invalid named variable In RclVarValue"
      Return 0
    End If
  End Function

  Private Sub StoVarValue(ByVal VarName As String, ByVal VarValue As Double)
    Dim I As Integer

    I = FindVar(VarName)
    If I > 0 Then
      VarsValues(I) = VarValue
    Else
      AddVar(VarName, VarValue)
    End If
  End Sub

  Private Function FindAlphaVar(ByVal AlphaVarName As String) As Integer
    Return AlphaVarsHashTable.Find(AlphaVarName)
  End Function

  Private Sub AddAlphaVar(ByVal AlphaVarName As String)
    AlphaVarsCount += 1
    ReDim Preserve sAlphaVarsNames(AlphaVarsCount), AlphaVarsValues(AlphaVarsCount)
    sAlphaVarsNames(AlphaVarsCount) = AlphaVarName.ToUpper
    AlphaVarsValues(AlphaVarsCount) = Alpha
    AlphaVarsHashTable.Insert(AlphaVarName.ToUpper, AlphaVarsCount)
  End Sub

  Private Function DoesAlphaVarExist(ByVal AlphaVarName As String) As Boolean
    Return FindAlphaVar(AlphaVarName) > 0
  End Function

  Private Sub RclAlphaVarValue(ByVal AlphaVarName As String)
    Dim I As Integer

    I = FindAlphaVar(AlphaVarName)
    If I > 0 Then
      Alpha = AlphaVarsValues(I)
    Else
      sErrMessage = "Invalid named variable In RclAlphaVarValue"
    End If
  End Sub

  Private Function GetAlphaVarValue(ByVal AlphaVarName As String) As String
    Dim I As Integer

    I = FindAlphaVar(AlphaVarName)
    If I > 0 Then
      Return AlphaVarsValues(I)
    Else
      Return ""
    End If
  End Function

  Private Sub StoAlphaVarValue(ByVal AlphaVarName As String)
    Dim I As Integer

    I = FindAlphaVar(AlphaVarName)
    If I > 0 Then
      AlphaVarsValues(I) = Alpha
    Else
      AddAlphaVar(AlphaVarName)
    End If
  End Sub

  Private Sub fxVCAadd(ByVal tok As String)
    Dim I, I1, I2, VarI1, VarI2, N, J As Integer
    Dim Max1, Max2 As Integer
    Dim var1, var2 As String
    Dim X1, X2 As Double
    Dim StrArr() As String

    N = Fix(Stack(Z_STACK_REG))
    I2 = Fix(Stack(Y_STACK_REG))
    I2 = Max(0, I2)
    I1 = Fix(Stack(X_STACK_REG))
    I1 = Max(0, I1)
    StrArr = tok.Split(" ")
    If StrArr.GetUpperBound(0) > 1 Then
      var1 = StrArr(1)
      var2 = StrArr(2)
      VarI1 = objVarArrs.FindVar(var1)
      VarI2 = objVarArrs.FindVar(var2)
      If VarI1 > 0 And VarI2 > 0 Then
        Max1 = objVarArrs.GetArrSize(var1)
        Max2 = objVarArrs.GetArrSize(var2)
        I1 = Min(I1, Max1)
        I2 = Min(I2, Max2)
        J = 0
        For I = 0 To N - 1
          J += 1
          X1 = objVarArrs.QRecall(VarI1, I1 + I)
          X2 = objVarArrs.QRecall(VarI2, I2 + I)
          objVarArrs.QStore(VarI1, I1 + I, X1 + X2)
          I1 += 1
          I2 += 1
          If I1 > Max1 Or I2 > Max2 Then
            Enter(J)  ' return number of elements processed
            Exit Sub
          End If
        Next
        Enter(J) ' return number of elements processed
      ElseIf VarI1 = 0 And VarI2 = 0 Then
        sErrMessage = "Variables " & StrArr(0) & " and " & StrArr(1) & " do not exist in command VCA+"
      ElseIf VarI1 > 0 And VarI2 = 0 Then
        sErrMessage = "Variable " & StrArr(1) & " does not exist  in command VCA+"
      ElseIf VarI1 = 0 And VarI2 > 0 Then
        sErrMessage = "Variable " & StrArr(0) & " does not exist  in command VCA+"
      End If
    Else
      sErrMessage = "Variable(s) missing in command VCA+ "
    End If

  End Sub


  Private Sub fxVCAsubt(ByVal tok As String)
    Dim I, I1, I2, VarI1, VarI2, N, J As Integer
    Dim Max1, Max2 As Integer
    Dim var1, var2 As String
    Dim X1, X2 As Double
    Dim StrArr() As String

    N = Fix(Stack(Z_STACK_REG))
    I2 = Fix(Stack(Y_STACK_REG))
    I2 = Max(0, I2)
    I1 = Fix(Stack(X_STACK_REG))
    I1 = Max(0, I1)
    StrArr = tok.Split(" ")
    If StrArr.GetUpperBound(0) > 1 Then
      var1 = StrArr(1)
      var2 = StrArr(2)
      VarI1 = objVarArrs.FindVar(var1)
      VarI2 = objVarArrs.FindVar(var2)
      If VarI1 > 0 And VarI2 > 0 Then
        Max1 = objVarArrs.GetArrSize(var1)
        Max2 = objVarArrs.GetArrSize(var2)
        I1 = Min(I1, Max1)
        I2 = Min(I2, Max2)
        J = 0
        For I = 0 To N - 1
          J += 1
          X1 = objVarArrs.QRecall(VarI1, I1 + I)
          X2 = objVarArrs.QRecall(VarI2, I2 + I)
          objVarArrs.QStore(VarI1, I1 + I, X1 - X2)
          I1 += 1
          I2 += 1
          If I1 > Max1 Or I2 > Max2 Then
            Enter(J)
            Exit Sub
          End If
        Next
        Enter(J) ' return number of elements processed
      ElseIf VarI1 = 0 And VarI2 = 0 Then
        sErrMessage = "Variables " & StrArr(0) & " and " & StrArr(1) & " do not exist  in command VCA-"
      ElseIf VarI1 > 0 And VarI2 = 0 Then
        sErrMessage = "Variable " & StrArr(1) & " does not exist  in command VCA-"
      ElseIf VarI1 = 0 And VarI2 > 0 Then
        sErrMessage = "Variable " & StrArr(0) & " does not exist  in command VCA-"
      End If
    Else
      sErrMessage = "Variable(s) missing in command VCA-"
    End If

  End Sub

  Private Sub fxVCAmult(ByVal tok As String)
    Dim I, I1, I2, VarI1, VarI2, N, J As Integer
    Dim Max1, Max2 As Integer
    Dim var1, var2 As String
    Dim X1, X2 As Double
    Dim StrArr() As String

    N = Fix(Stack(Z_STACK_REG))
    I2 = Fix(Stack(Y_STACK_REG))
    I2 = Max(0, I2)
    I1 = Fix(Stack(X_STACK_REG))
    I1 = Max(0, I1)
    StrArr = tok.Split(" ")
    If StrArr.GetUpperBound(0) > 1 Then
      var1 = StrArr(1)
      var2 = StrArr(2)
      VarI1 = objVarArrs.FindVar(var1)
      VarI2 = objVarArrs.FindVar(var2)
      If VarI1 > 0 And VarI2 > 0 Then
        Max1 = objVarArrs.GetArrSize(var1)
        Max2 = objVarArrs.GetArrSize(var2)
        I1 = Min(I1, Max1)
        I2 = Min(I2, Max2)
        J = 0
        For I = 0 To N - 1
          J += 1
          X1 = objVarArrs.QRecall(VarI1, I1 + I)
          X2 = objVarArrs.QRecall(VarI2, I2 + I)
          objVarArrs.QStore(VarI1, I1 + I, X1 * X2)
          I1 += 1
          I2 += 1
          If I1 > Max1 Or I2 > Max2 Then
            Enter(J)  ' return number of elements processed
            Exit Sub
          End If
        Next
        Enter(J) ' return number of elements processed
      ElseIf VarI1 = 0 And VarI2 = 0 Then
        sErrMessage = "Variables " & StrArr(0) & " and " & StrArr(1) & " do not exist in command VCA*"
      ElseIf VarI1 > 0 And VarI2 = 0 Then
        sErrMessage = "Variable " & StrArr(1) & " does not exist  in command VCA*"
      ElseIf VarI1 = 0 And VarI2 > 0 Then
        sErrMessage = "Variable " & StrArr(0) & " does not exist  in command VCA*"
      End If
    Else
      sErrMessage = "Variable(s) missing  in command VCA*"
    End If

  End Sub

  Private Sub fxVCAdiv(ByVal tok As String)
    Dim I, I1, I2, VarI1, VarI2, N, J As Integer
    Dim Max1, Max2 As Integer
    Dim var1, var2 As String
    Dim X1, X2 As Double
    Dim StrArr() As String

    N = Fix(Stack(Z_STACK_REG))
    I2 = Fix(Stack(Y_STACK_REG))
    I2 = Max(0, I2)
    I1 = Fix(Stack(X_STACK_REG))
    I1 = Max(0, I1)
    StrArr = tok.Split(" ")
    If StrArr.GetUpperBound(0) > 1 Then
      var1 = StrArr(1)
      var2 = StrArr(2)
      VarI1 = objVarArrs.FindVar(var1)
      VarI2 = objVarArrs.FindVar(var2)
      If VarI1 > 0 And VarI2 > 0 Then
        Max1 = objVarArrs.GetArrSize(var1)
        Max2 = objVarArrs.GetArrSize(var2)
        I1 = Min(I1, Max1)
        I2 = Min(I2, Max2)
        J = 0
        For I = 0 To N - 1
          J += 1
          X1 = objVarArrs.QRecall(VarI1, I1 + I)
          X2 = objVarArrs.QRecall(VarI2, I2 + I)
          If X2 <> 0 Then
            objVarArrs.QStore(VarI1, I1 + I, X1 / X2)
          Else
            objVarArrs.QStore(VarI2, I2 + I, Double.MaxValue)
          End If
          I1 += 1
          I2 += 1
          If I1 > Max1 Or I2 > Max2 Then
            Enter(J)  ' return number of elements processed
            Exit Sub
          End If
        Next
        Enter(J) ' return number of elements processed
      ElseIf VarI1 = 0 And VarI2 = 0 Then
        sErrMessage = "Variables " & StrArr(0) & " and " & StrArr(1) & " do not exist in command VCA*"
      ElseIf VarI1 > 0 And VarI2 = 0 Then
        sErrMessage = "Variable " & StrArr(1) & " does not exist  in command VCA*"
      ElseIf VarI1 = 0 And VarI2 > 0 Then
        sErrMessage = "Variable " & StrArr(0) & " does not exist  in command VCA*"
      End If
    Else
      sErrMessage = "Variable(s) missing  in command VCA*"
    End If

  End Sub

  Private Sub fxVCASadd(ByVal tok As String)
    Dim I, I1, VarI1, N, J As Integer
    Dim Max1 As Integer
    Dim var1 As String
    Dim X1, X As Double
    Dim StrArr() As String

    N = Fix(Stack(Z_STACK_REG))
    I1 = Fix(Stack(Y_STACK_REG))
    X = Stack(X_STACK_REG)
    I1 = Max(0, I1)
    StrArr = tok.Split(" ")
    If StrArr.GetUpperBound(0) > 0 Then
      var1 = StrArr(1)
      VarI1 = objVarArrs.FindVar(var1)
      If VarI1 > 0 Then
        Max1 = objVarArrs.GetArrSize(var1)
        I1 = Min(I1, Max1)
        J = 0
        For I = 0 To N - 1
          J += 1
          X1 = objVarArrs.QRecall(VarI1, I1 + I)
          objVarArrs.QStore(VarI1, I1 + I, X1 + X)
          I1 += 1
          If I1 > Max1 Then
            Enter(J)  ' return number of elements processed
            Exit Sub
          End If
        Next
        Enter(J) ' return number of elements processed
      ElseIf VarI1 = 0 Then
        sErrMessage = "Variable " & StrArr(0) & " does not exist  in command VCAS+"
      End If
    Else
      sErrMessage = "Variable(s) missing in command VCAS+ "
    End If

  End Sub

  Private Sub fxVCASsubt(ByVal tok As String)
    Dim I, I1, VarI1, N, J As Integer
    Dim Max1 As Integer
    Dim var1 As String
    Dim X1, X As Double
    Dim StrArr() As String

    N = Fix(Stack(Z_STACK_REG))
    I1 = Fix(Stack(Y_STACK_REG))
    X = Stack(X_STACK_REG)
    I1 = Max(0, I1)
    StrArr = tok.Split(" ")
    If StrArr.GetUpperBound(0) > 0 Then
      var1 = StrArr(1)
      VarI1 = objVarArrs.FindVar(var1)
      If VarI1 > 0 Then
        Max1 = objVarArrs.GetArrSize(var1)
        I1 = Min(I1, Max1)
        J = 0
        For I = 0 To N - 1
          J += 1
          X1 = objVarArrs.QRecall(VarI1, I1 + I)
          objVarArrs.QStore(VarI1, I1 + I, X1 - X)
          I1 += 1
          If I1 > Max1 Then
            Enter(J)  ' return number of elements processed
            Exit Sub
          End If
        Next
        Enter(J) ' return number of elements processed
      ElseIf VarI1 = 0 Then
        sErrMessage = "Variable " & StrArr(0) & " does not exist  in command VCAS-"
      End If
    Else
      sErrMessage = "Variable(s) missing in command VCAS-"
    End If

  End Sub


  Private Sub fxVCASAsubt(ByVal tok As String)
    Dim I, I1, VarI1, N, J As Integer
    Dim Max1 As Integer
    Dim var1 As String
    Dim X1, X As Double
    Dim StrArr() As String

    N = Fix(Stack(Z_STACK_REG))
    I1 = Fix(Stack(Y_STACK_REG))
    X = Stack(X_STACK_REG)
    I1 = Max(0, I1)
    StrArr = tok.Split(" ")
    If StrArr.GetUpperBound(0) > 0 Then
      var1 = StrArr(1)
      VarI1 = objVarArrs.FindVar(var1)
      If VarI1 > 0 Then
        Max1 = objVarArrs.GetArrSize(var1)
        I1 = Min(I1, Max1)
        J = 0
        For I = 0 To N - 1
          J += 1
          X1 = objVarArrs.QRecall(VarI1, I1 + I)
          objVarArrs.QStore(VarI1, I1 + I, X - X1)
          I1 += 1
          If I1 > Max1 Then
            Enter(J)  ' return number of elements processed
            Exit Sub
          End If
        Next
        Enter(J) ' return number of elements processed
      ElseIf VarI1 = 0 Then
        sErrMessage = "Variable " & StrArr(0) & " does not exist  in command VCASA-"
      End If
    Else
      sErrMessage = "Variable(s) missing in command VCASA-"
    End If

  End Sub

  Private Sub fxVCASmult(ByVal tok As String)
    Dim I, I1, VarI1, N, J As Integer
    Dim Max1 As Integer
    Dim var1 As String
    Dim X1, X As Double
    Dim StrArr() As String

    N = Fix(Stack(Z_STACK_REG))
    I1 = Fix(Stack(Y_STACK_REG))
    X = Stack(X_STACK_REG)
    I1 = Max(0, I1)
    StrArr = tok.Split(" ")
    If StrArr.GetUpperBound(0) > 0 Then
      var1 = StrArr(1)
      VarI1 = objVarArrs.FindVar(var1)
      If VarI1 > 0 Then
        Max1 = objVarArrs.GetArrSize(var1)
        I1 = Min(I1, Max1)
        J = 0
        For I = 0 To N - 1
          J += 1
          X1 = objVarArrs.QRecall(VarI1, I1 + I)
          objVarArrs.QStore(VarI1, I1 + I, X1 * X)
          I1 += 1
          If I1 > Max1 Then
            Enter(J)  ' return number of elements processed
            Exit Sub
          End If
        Next
        Enter(J) ' return number of elements processed
      ElseIf VarI1 = 0 Then
        sErrMessage = "Variable " & StrArr(0) & " does not exist  in command VCAS*"
      End If
    Else
      sErrMessage = "Variable(s) missing in command VCAS*"
    End If

  End Sub

  Private Sub fxVCASdiv(ByVal tok As String)
    Dim I, I1, VarI1, N, J As Integer
    Dim Max1 As Integer
    Dim var1 As String
    Dim X1, X As Double
    Dim StrArr() As String

    N = Fix(Stack(Z_STACK_REG))
    I1 = Fix(Stack(Y_STACK_REG))
    X = Stack(X_STACK_REG)
    I1 = Max(0, I1)
    StrArr = tok.Split(" ")
    If StrArr.GetUpperBound(0) > 0 Then
      var1 = StrArr(1)
      VarI1 = objVarArrs.FindVar(var1)
      If VarI1 > 0 Then
        Max1 = objVarArrs.GetArrSize(var1)
        I1 = Min(I1, Max1)
        J = 0
        For I = 0 To N - 1
          J += 1
          X1 = objVarArrs.QRecall(VarI1, I1 + I)
          If X <> 0 Then
            objVarArrs.QStore(VarI1, I1 + I, X1 / X)
          Else
            objVarArrs.QStore(VarI1, I1 + I, Double.MaxValue)
          End If
          I1 += 1
          If I1 > Max1 Then
            Enter(J)  ' return number of elements processed
            Exit Sub
          End If
        Next
        Enter(J) ' return number of elements processed
      ElseIf VarI1 = 0 Then
        sErrMessage = "Variable " & StrArr(0) & " does not exist  in command VCAS/"
      End If
    Else
      sErrMessage = "Variable(s) missing in command VCAS/"
    End If

  End Sub


  Private Sub fxVCASAdiv(ByVal tok As String)
    Dim I, I1, VarI1, N, J As Integer
    Dim Max1 As Integer
    Dim var1 As String
    Dim X1, X As Double
    Dim StrArr() As String

    N = Fix(Stack(Z_STACK_REG))
    I1 = Fix(Stack(Y_STACK_REG))
    X = Stack(X_STACK_REG)
    I1 = Max(0, I1)
    StrArr = tok.Split(" ")
    If StrArr.GetUpperBound(0) > 0 Then
      var1 = StrArr(1)
      VarI1 = objVarArrs.FindVar(var1)
      If VarI1 > 0 Then
        Max1 = objVarArrs.GetArrSize(var1)
        I1 = Min(I1, Max1)
        J = 0
        For I = 0 To N - 1
          J += 1
          X1 = objVarArrs.QRecall(VarI1, I1 + I)
          If X1 <> 0 Then
            objVarArrs.QStore(VarI1, I1 + I, X / X1)
          Else
            objVarArrs.QStore(VarI1, I1 + I, Double.MaxValue)
          End If
          I1 += 1
          If I1 > Max1 Then
            Enter(J)  ' return number of elements processed
            Exit Sub
          End If
        Next
        Enter(J) ' return number of elements processed
      ElseIf VarI1 = 0 Then
        sErrMessage = "Variable " & StrArr(0) & " does not exist  in command VCASA/"
      End If
    Else
      sErrMessage = "Variable(s) missing in command VCASA/"
    End If

  End Sub

  Private Sub fxVCAfx(ByVal tok As String)
    Dim I, I1, VarI1, N, J As Integer
    Dim Max1 As Integer
    Dim var1 As String
    Dim X As Double
    Dim StrArr() As String

    N = Fix(Stack(Y_STACK_REG))
    I1 = Fix(Stack(X_STACK_REG))
    I1 = Max(0, I1)
    StrArr = tok.Split(" ")
    If StrArr.GetUpperBound(0) > 0 And Alpha.Trim.Length > 0 Then
      var1 = StrArr(1)
      VarI1 = objVarArrs.FindVar(var1)
      If VarI1 > 0 Then
        Max1 = objVarArrs.GetArrSize(var1)
        I1 = Min(I1, Max1)
        J = 0
        For I = 0 To N - 1
          J += 1
          X = objVarArrs.QRecall(var1, I1 + I)
          Enter(X)
          Try
            X = Calc(Alpha)
            If sErrMessage.Length = 0 Then
              objVarArrs.QStore(VarI1, I1 + I, X)
            Else
              sErrMessage = ""
              WriteLine("Runtime error in command VCAfx.")
              objVarArrs.Store(VarI1, I1 + I, Double.MaxValue)
            End If
          Catch
            WriteLine("Runtime error in command VCAfx.")
            objVarArrs.Store(VarI1, I1 + I, Double.MaxValue)
          End Try
          I1 += 1
          If I1 > Max1 Then
            Enter(J)  ' return number of elements processed
            Exit Sub
          End If
        Next
        Enter(J) ' return number of elements processed
      ElseIf VarI1 = 0 Then
        sErrMessage = "Variable " & StrArr(0) & " does not exist  in command VCAfx"
      End If
    Else
      If Alpha.Trim.Length = 0 Then
        sErrMessage = "Alpha register is empty in command VCAfx"
      Else
        sErrMessage = "Variable(s) missing in command VCAfx"
      End If
    End If

  End Sub

  Private Sub ViewMemRegisters()
    Dim I1, I2, I As Double
    Dim X As Decimal

    X = Stack(X_STACK_REG)
    I1 = Fix(X)
    X = X - I1
    I2 = Fix(SCALE * X)

    ' make sure that I1<=I2
    If I1 > I2 Then ' Swap I1 and I2
      I = I1
      I1 = I2
      I2 = I
    End If
    ' check range of indices
    If I1 < 0 Then I1 = 0
    If I2 > MAX_MEM Then I2 = MAX_MEM

    ' show memory registers
    For I = I1 To I2
      WriteLine("Mem({0}) = {1}", I, Output(GetMem(I)))
    Next
  End Sub

  Protected Sub Mem2ArrVar(ByVal varName As String)
    Dim I, J, First, Last, Start, Finish As Integer

    varName = varName.ToUpper
    J = objVarArrs.FindVar(varName)
    First = Fix(Stack(X_STACK_REG))
    Last = Fix(Stack(Y_STACK_REG))
    Start = Fix(Stack(Z_STACK_REG))

    If J < 1 Then
      sErrMessage = "Invalid named variable In Mem2ArrVar"
      Enter(0)
      Exit Sub
    End If

    If First < 0 Then First = 0
    If Last > MAX_MEM Then Last = MAX_MEM

    J = Start
    Finish = objVarArrs.GetSize(varName)
    For I = First To Last
      objVarArrs.Store(varName, J, GetMem(I))
      J += 1
      If J > Finish Then Exit For
    Next

    Enter(J - Start)  ' return number of elements processed
  End Sub

  Protected Sub ArrVar2Mem(ByVal varName As String)
    Dim I, J, First, Last, Start, Finish As Integer

    varName = varName.ToUpper
    J = objVarArrs.FindVar(varName)

    If J < 1 Then
      sErrMessage = "Invalid named variable In ArrVar2Mem"
      Enter(0)
      Exit Sub
    End If

    First = Fix(Stack(X_STACK_REG))
    Last = Fix(Stack(Y_STACK_REG))
    Start = Fix(Stack(Z_STACK_REG))

    If First > Last Then
      I = First
      First = Last
      Last = I
    End If

    If First < 0 Then First = 0
    ' cannot copy beyond the limit of the memory registers
    If Last > MAX_MEM Then Last = MAX_MEM

    J = Start
    Finish = objVarArrs.GetSize(varName)
    For I = First To Last
      SetMem(I, objVarArrs.Recall(varName, J))
      J += 1
      If J > Finish Then Exit For
    Next
    Enter(J - Start)  ' return number of elements processed
  End Sub

  Public Sub ReadMem(ByVal sFilename As String)
    Dim I, J, K, N, Max As Integer
    Dim TextLine, sNums() As String

    If My.Computer.FileSystem.FileExists(sFilename) Then
      Dim fileReader As System.IO.StreamReader
      fileReader = New StreamReader(sFilename)
      I = Fix(Stack(X_STACK_REG)) ' get the index of the data insertion point
      Max = MAX_MEM
      J = 0
      Try
        Do Until fileReader.EndOfStream
          TextLine = fileReader.ReadLine().Trim
          ' ignore blank lines
          If TextLine.Length > 0 Then
            ' Just one value in the linea read?
            If TextLine.IndexOf(",") < 0 Then
              If IsNumeric(TextLine) Then
                SetMem(I, ParseDbl(TextLine))
                I += 1
                J += 1
              End If
            Else
              sNums = TextLine.Split(",")
              ' getthe number of values
              N = sNums.GetUpperBound(0) + 1
              For K = 0 To N - 1
                ' Can we store the new value in the next memory register?
                If I <= Max Then
                  If IsNumeric(sNums(K)) Then
                    SetMem(I, ParseDbl(sNums(K).Trim))
                    I += 1
                    J += 1
                  End If
                Else
                  WriteLine("Extra input data In ReadMem is ignored.")
                  Exit Do
                End If
              Next
            End If
          End If
        Loop
        fileReader.Close()
        Enter(J)  ' return number of elements read
      Catch
        sErrMessage = "Error reading numbers from file " & sFilename & " In READMEM command"
      End Try
    Else
      sErrMessage = "Cannot open file " & sFilename & " In READMEM command"
    End If
  End Sub

  Public Sub WriteMem(ByVal sFilename As String)
    Dim I, J, N, NumsPerLine, Count As Integer

    Try
      Dim fileWriter As System.IO.StreamWriter
      fileWriter = New StreamWriter(sFilename)
      I = Abs(Fix(Stack(X_STACK_REG))) ' get the index of the first memory register written
      Count = Abs(Fix(Stack(Y_STACK_REG))) ' get the number of memory register written to the file
      NumsPerLine = Abs(Fix(Stack(Z_STACK_REG)))
      NumsPerLine = Max(1, NumsPerLine)
      J = 0
      N = 1
      Do While Count > 0
        If I <= MAX_MEM Then
          If N < NumsPerLine Then
            fileWriter.Write(CStr(GetMem(I)) & ",")
            N += 1
          Else
            fileWriter.WriteLine(CStr(GetMem(I)))
            N = 1
          End If
          I += 1
          J += 1
          Count -= 1
        Else
          WriteLine("Reached maximum memory register index in WriteMem.")
          Exit Do
        End If
      Loop
      fileWriter.Close()
      Enter(J) ' push in the stack the actual number of memory registers written
    Catch
      sErrMessage = "Cannot open file " & sFilename & " In WRITEMEM command"
    End Try
  End Sub

  Private Function SwapCase(ByRef s As String) As String
    Dim I As Integer
    Dim Len As Integer = s.Length
    Dim c As String
    Dim sRes As String = ""

    For I = 0 To Len - 1
      c = s.Substring(I, 1)
      If c >= "A" And c <= "Z" Then
        sRes &= c.ToLower
      ElseIf c >= "a" And c <= "z" Then
        sRes &= c.ToUpper
      Else
        ' just append the character
        sRes &= c
      End If
    Next
    Return sRes
  End Function

  Private Sub fxStoreFlags(ByVal sVarname As String)
    Dim I As Integer
    Dim s, s2 As String

    s2 = Alpha
    s = ""
    For I = 0 To MAX_FLAGS
      s &= If(Flags(I), "1", "0")
    Next
    Alpha = s
    StoAlphaVarValue(sVarname)
    Alpha = s2
  End Sub

  Private Sub fxRecallFlags(ByVal sVarname As String)
    Dim I As Integer
    Dim s, s2 As String

    s2 = Alpha
    RclAlphaVarValue(sVarname)
    s = Alpha
    For I = 0 To MAX_FLAGS
      Flags(I) = s.Substring(I, 1) = "1"
    Next
    Alpha = s2
  End Sub

  Public Sub ShowLabels()
    Dim I As Integer

    For I = 1 To LblsCount
      WriteLine(I & "  " & sLbls(I))
    Next
  End Sub

  Private Sub fxAddStatData(ByVal s As String)
    Dim sVals() As String
    Dim sVal As String
    Dim X, Y As Double
    Dim bGetX As Boolean

    Try
      sVals = s.Split(",")
      bGetX = True
      For Each sVal In sVals
        If bGetX Then
          X = ParseDbl(sVal)
        Else
          Y = ParseDbl(sVal)
          StatCount += 1
          SumX += X
          SumX2 += X * X
          SumY += Y
          SumY2 += Y * Y
          SumXY += X * Y
        End If
        bGetX = Not bGetX
      Next
    Catch
      sErrMessage = "Error In fxAddStatData"
    End Try

  End Sub

  Private Sub fxAddMemData(ByVal s As String)
    Dim sVals() As String
    Dim sVal As String
    Dim Idx As Double
    Dim bSkip As Boolean

    Try
      sVals = s.Split(",")
      Idx = ParseInt(sVals(0))
      bSkip = True
      For Each sVal In sVals
        If Not bSkip Then
          ' is idx value with memory register's limits
          If Idx <= MAX_MEM Then
            SetMem(Idx, ParseDbl(sVal))
            Idx += 1
          Else
            Exit For
          End If
        Else
          bSkip = False
        End If
      Next
    Catch
      sErrMessage = "Error In fxAddMemData"
    End Try

  End Sub

  Private Sub fxAddVarData(ByVal s As String)
    Dim sVals() As String
    Dim sVal, sVarName As String
    Dim I, Idx, N, ElemsToSkip As Double

    Try
      sVals = s.Split(",")
      sVarName = sVals(0)
      If IsNumeric(sVarName) Then
        sVarName = Alpha
        I = FindVar(sVarName)
        ElemsToSkip = 1 ' elements in sVals to skip
        Idx = ParseInt(sVals(0))
      Else
        I = FindVar(sVarName)
        ElemsToSkip = 2 ' elements in sVals to skip
        Idx = ParseInt(sVals(1))
      End If
      If Idx < 0 OrElse Idx > MAX_MEM Then
        sErrMessage = "Index out Of range In ElemsToSkip"
        Return
      End If
      ' Need to create a new variable?
      If I = 0 Then
        N = sVals.GetUpperBound(0) - sVals.GetLowerBound(0) + 1
        objVarArrs.AddVar(sVarName, N - ElemsToSkip)
      End If
      For Each sVal In sVals
        If ElemsToSkip = 0 Then
          objVarArrs.Store(sVarName, Idx, ParseDbl(sVal))
          Idx += 1
        Else
          ElemsToSkip -= 1
        End If
      Next
    Catch
      sErrMessage = "Error In fxAddMemData"
    End Try

  End Sub

  Private Sub JumpSteps(ByVal I As Integer)
    If I > 0 Then
      PrgmPtr += I
    ElseIf I < 0 Then
      PrgmPtr -= I
    End If
    bUpdatePrgmPtr = False
  End Sub

  Public Sub fxPrtStackReg(ByVal X As Double, ByVal Format As String, ByVal sRegName As String)
    Dim s As String
    Dim Eps As Double = 100 * Double.MinValue

    If sRegName.Trim() = "" Then
      ' search the stack registers to determine which one is close to the value of X
      If Abs(X - Stack(X_STACK_REG)) < Eps Then
        sRegName = "X"
        WriteLine("Invalid register name In subroutine fxPrtStackReg. Selecting the X stack register.")
      ElseIf Abs(X - Stack(Y_STACK_REG)) < Eps Then
        sRegName = "Y"
        WriteLine("Invalid register name In subroutine fxPrtStackReg. Selecting the Y stack register.")
      ElseIf Abs(X - Stack(Z_STACK_REG)) < Eps Then
        sRegName = "Z"
        WriteLine("Invalid register name In subroutine fxPrtStackReg. Selecting the Z stack register.")
      ElseIf Abs(X - Stack(T_STACK_REG)) < Eps Then
        sRegName = "T"
        WriteLine("Invalid register name In subroutine fxPrtStackReg. Selecting the T stack register.")
      Else
        sRegName = "X"
        WriteLine("Invalid register name In subroutine fxPrtStackReg. Default selection Of the X stack register.")
      End If
    End If
    ' pick the first letter (in case argument has more than one)
    sRegName = sRegName.Substring(0, 1).ToUpper

    ' do we need to use the ALpha register?
    If Format.Length <= 4 Then
      s = Alpha ' get format from Alpha register
    Else
      s = Format.Substring(4) ' get format string from command
    End If

    ' perform the formatting
    Try
      s = s.ToUpper
      WriteLine(Output(X, s))
    Catch
      WriteLine("Formatting failed In PRT" & sRegName &
                        "command. Displaying unformattted value Of stack register" & sRegName & ".")
      WriteLine(X)
    End Try

  End Sub

  Public Sub fxPrtxStack(ByVal sFormats As String)
    ' print the stack using formats for each register
    Dim s, sFrmts() As String
    Dim N As Integer
    Dim X As Double

    sFormats = sFormats.ToUpper
    If sFormats.Trim = "" Then
      ' default displayt of stack
      ShowStack()
      Return
    End If
    sFrmts = sFormats.Split(",")
    N = sFrmts.GetUpperBound(0)
    Select Case N
      Case 0
        s = sFrmts(0)
        X = Stack(T_STACK_REG)
        WriteLine("T: " & Output(X, s))
        X = Stack(Z_STACK_REG)
        WriteLine("Z: " & Output(X, s))
        X = Stack(Y_STACK_REG)
        WriteLine("Y: " & Output(X, s))
        X = Stack(X_STACK_REG)
        WriteLine("X: " & Output(X, s))

      Case 1
        s = sFrmts(1)
        X = Stack(T_STACK_REG)
        WriteLine("T: " & Output(X, s))
        X = Stack(Z_STACK_REG)
        WriteLine("Z: " & Output(X, s))
        X = Stack(Y_STACK_REG)
        WriteLine("Y: " & Output(X, s))

        s = sFrmts(0)
        X = Stack(X_STACK_REG)
        WriteLine("X:" & Output(X, s))

      Case 2
        s = sFrmts(2)
        X = Stack(T_STACK_REG)
        WriteLine("T: " & Output(X, s))

        s = sFrmts(1)
        X = Stack(Z_STACK_REG)
        WriteLine("Z: " & Output(X, s))

        X = Stack(Y_STACK_REG)
        WriteLine("Y: " & Output(X, s))

        s = sFrmts(0)
        X = Stack(X_STACK_REG)
        WriteLine("X: " & Output(X, s))

      Case 3
        s = sFrmts(3)
        X = Stack(T_STACK_REG)
        WriteLine("T: " & Output(X, s))

        s = sFrmts(2)
        X = Stack(Z_STACK_REG)
        WriteLine("Z: " & Output(X, s))

        s = sFrmts(1)
        X = Stack(Y_STACK_REG)
        WriteLine("Y: " & Output(X, s))

        s = sFrmts(0)
        X = Stack(X_STACK_REG)
        WriteLine("X: " & Output(X, s))
    End Select
  End Sub

  Private Function IsBetween() As Boolean
    Dim X, Y, Z As Double

    X = Stack(X_STACK_REG)
    If Stack(Y_STACK_REG) < Stack(Z_STACK_REG) Then
      Y = Stack(Y_STACK_REG)
      Z = Stack(Z_STACK_REG)
    Else
      Y = Stack(Z_STACK_REG)
      Z = Stack(Y_STACK_REG)
    End If

    Return X >= Y And X <= Z
  End Function

  Private Function IsWithin() As Boolean
    Dim X, Y, Z As Double

    X = Stack(X_STACK_REG)
    If Stack(Y_STACK_REG) < Stack(Z_STACK_REG) Then
      Y = Stack(Y_STACK_REG)
      Z = Stack(Z_STACK_REG)
    Else
      Y = Stack(Z_STACK_REG)
      Z = Stack(Y_STACK_REG)
    End If

    Return X > Y And X < Z
  End Function

  Private Function IsOutside() As Boolean
    Dim X, Y, Z As Double

    X = Stack(X_STACK_REG)
    If Stack(Y_STACK_REG) < Stack(Z_STACK_REG) Then
      Y = Stack(Y_STACK_REG)
      Z = Stack(Z_STACK_REG)
    Else
      Y = Stack(Z_STACK_REG)
      Z = Stack(Y_STACK_REG)
    End If

    Return X <= Y OrElse X >= Z
  End Function

  Private Function IsWithout() As Boolean
    Dim X, Y, Z As Double

    X = Stack(X_STACK_REG)
    If Stack(Y_STACK_REG) < Stack(Z_STACK_REG) Then
      Y = Stack(Y_STACK_REG)
      Z = Stack(Z_STACK_REG)
    Else
      Y = Stack(Z_STACK_REG)
      Z = Stack(Y_STACK_REG)
    End If

    Return X < Y OrElse X > Z
  End Function

  Private Function CheckFlagVars(ByVal s As String) As Boolean
    ' Check flag pattern in s for having only 1s and 0s.
    Dim I As Integer
    Dim L As Integer = s.Length

    For I = 0 To L - 1
      If Not (s.IndexOf("0") >= 0 OrElse s.IndexOf("1") >= 0) Then
        sErrMessage = "Invalid group flag test pattern of " & s
        Return False
      End If
    Next

    Return True
  End Function

  Private Function fxGroupFlagTest(ByVal Idx As Integer, ByVal sFlagVals As String) As Boolean
    Dim bFlag As Boolean
    Dim Len As Integer = sFlagVals.Length
    Dim I As Integer

    ' No 1s or 0s ONLY in the sFlagVar parameter?
    If Not CheckFlagVars(sFlagVals) Then Return False ' no correct pattern to test!!

    For I = 0 To Len - 1
      If Idx + I > MAX_FLAGS Then Return False
      bFlag = If(sFlagVals.Substring(I, 1) = "0", False, True)
      If bFlag Then
        If Not Flags(Idx + I) Then Return False
      Else
        If Flags(Idx + I) Then Return False
      End If
    Next
    Return True
  End Function

  Private Function fxGroupFlagTestSet(ByVal Idx As Integer, ByVal sFlagVals As String) As Boolean
    Dim bFlag As Boolean
    Dim Len As Integer = sFlagVals.Length
    Dim I As Integer

    ' No 1s or 0s ONLY in the sFlagVar parameter?
    If Not CheckFlagVars(sFlagVals) Then Return False ' no correct pattern to test!!

    For I = 0 To Len - 1
      If Idx + I > MAX_FLAGS Then Return False
      bFlag = If(sFlagVals.Substring(I, 1) = "0", False, True)
      If bFlag Then
        If Not Flags(Idx + I) Then Return False
      Else
        If Flags(Idx + I) Then Return False
      End If
    Next

    ' set all flags involved
    For I = 0 To Len - 1
      Flags(Idx + I) = True
    Next

    Return True
  End Function

  Private Function fxGroupFlagTestClear(ByVal Idx As Integer, ByVal sFlagVals As String) As Boolean
    Dim bFlag As Boolean
    Dim Len As Integer = sFlagVals.Length
    Dim I As Integer

    ' No 1s or 0s ONLY in the sFlagVar parameter?
    If Not CheckFlagVars(sFlagVals) Then Return False ' no correct pattern to test!!

    For I = 0 To Len - 1
      If Idx + I > MAX_FLAGS Then Return False
      bFlag = If(sFlagVals.Substring(I, 1) = "0", False, True)
      If bFlag Then
        If Not Flags(Idx + I) Then Return False
      Else
        If Flags(Idx + I) Then Return False
      End If
    Next

    ' clear all flags involved
    For I = 0 To Len - 1
      Flags(Idx + I) = False
    Next

    Return True
  End Function

  Private Function fxGroupFlagTestFlip(ByVal Idx As Integer, ByVal sFlagVals As String) As Boolean
    Dim bFlag As Boolean
    Dim Len As Integer = sFlagVals.Length
    Dim I As Integer

    ' No 1s or 0s ONLY in the sFlagVar parameter?
    If Not CheckFlagVars(sFlagVals) Then Return False ' no correct pattern to test!!

    For I = 0 To Len - 1
      bFlag = If(sFlagVals.Substring(I, 1) = "0", False, True)
      If bFlag Then
        If Not Flags(Idx + I) Then Return False
      Else
        If Flags(Idx + I) Then Return False
      End If
    Next

    ' flip all flags involved
    For I = 0 To Len - 1
      Flags(Idx + I) = Not Flags(Idx + I)
    Next

    Return True
  End Function

  '----DateFx
  Private Function StrToDate(ByVal s As String) As Date
    Try
      Dim dt As Date = s

      Return dt
    Catch
      ' Return the last New year!!!
      Return CDate("#1/1/" & CStr(Year(Now()) & " 00:00:00 AM#"))
    End Try
  End Function

  Private Function DateToStr(ByVal dt As Date) As String
    Return "#" & dt.ToString() & "#"
  End Function

  Private Function DecDateTimeToStr(ByVal xDate As Decimal, ByVal xTime As Decimal) As String
    Dim sAmPm As String
    Dim iYear, iMonth, iDay, iHr, iMin, iSec As Integer
    Dim bAm As Boolean

    iYear = Fix(xDate)
    xDate = 100 * (xDate - iYear)
    iMonth = Fix(xDate)
    xDate = 100 * (xDate - iMonth)
    iDay = xDate

    iHr = Fix(xTime)
    xTime = 100 * (xTime - iHr)
    iMin = Fix(xTime)
    xDate = 100 * (xTime - iMin)
    iSec = xTime
    If iHr > 12 Then
      sAmPm = "PM"
      iHr -= 12
    Else
      sAmPm = "AM"
    End If

    Return "#" & CStr(iMonth) & "/" & CStr(iDay) & "/" & CStr(iYear) & " " &
           CStr(iHr) & ":" & CStr(iMin) & ":" & CStr(iSec) & " " & sAmPm & "#"
  End Function

  Protected Sub StrToDecDateTime(ByVal s As String, ByRef xDate As Double, ByRef xTime As Double)
    Dim dt As Date = s
    Dim iHour, iMinute, iSecond As Integer
    Dim iYear, iMonth, iDay As Integer

    iHour = Hour(dt)
    iMinute = Minute(dt)
    iSecond = Second(dt)
    xTime = iHour + iMinute / 100 + iSecond / 10000

    iYear = Year(dt)
    iMonth = Month(dt)
    iDay = Day(dt)
    xDate = iYear + iMonth / 100 + iDay / 10000
  End Sub

  Private Sub ViewDate(ByVal s As String)
    Dim I As Integer
    Dim s2 As String

    I = FindAlphaVar(s)
    If I > 0 Then
      s2 = GetAlphaVarValue(s)
      If IsDate(s2) Then
        I = s2.Length
        WriteLine(s2.Substring(1, I - 2))
      Else
        WriteLine("Variable " & s & " does not contain a string in VIEWDT/DVIEW command.")
      End If
    Else
      sErrMessage = "Variable " & s & " Is missing for VIEWDT/DVIEW command"
    End If
  End Sub

  Private Sub StoreDate(ByVal s As String, ByVal xDate As Double, ByVal xTime As Double)
    Dim s2 As String

    s2 = Alpha
    Alpha = DecDateTimeToStr(xDate, xTime)
    StoAlphaVarValue(s)
    Alpha = s2
  End Sub

  Private Sub RecallDate(ByVal s As String, ByRef xDate As Double, ByRef xTime As Double)
    Dim I As Integer
    Dim s2 As String

    I = FindAlphaVar(s)
    If I > 0 Then
      s2 = Alpha
      RclAlphaVarValue(s)
      StrToDecDateTime(Alpha, xDate, xTime)
      Alpha = s2
    Else
      xDate = 0
      xTime = 0
    End If
  End Sub

  Private Sub AddDt(ByVal sVar As String, ByVal IncVal As Integer, ByVal sUnitToAdd As String)
    Dim dt As Date
    Dim I As Integer
    Dim s As String

    I = FindAlphaVar(sVar)
    If I > 0 Then
      s = Alpha
      RclAlphaVarValue(sVar)
      dt = StrToDate(Alpha)
      sUnitToAdd = sUnitToAdd.Trim.ToUpper
      If sUnitToAdd = "DAY" Then
        dt = DateAdd(DateInterval.Day, IncVal, dt)
      ElseIf sUnitToAdd = "MONTH" Then
        dt = DateAdd(DateInterval.Month, IncVal, dt)
      ElseIf sUnitToAdd = "YEAR" Then
        dt = DateAdd(DateInterval.Year, IncVal, dt)
      ElseIf sUnitToAdd = "HOUR" Then
        dt = DateAdd(DateInterval.Hour, IncVal, dt)
      ElseIf sUnitToAdd = "MINUTE" Then
        dt = DateAdd(DateInterval.Minute, IncVal, dt)
      ElseIf sUnitToAdd = "SECOND" Then
        dt = DateAdd(DateInterval.Second, IncVal, dt)
      ElseIf sUnitToAdd = "WEEK" Then
        dt = DateAdd(DateInterval.Weekday, IncVal, dt)
      ElseIf sUnitToAdd = "WEEKOFYEAR" Then
        dt = DateAdd(DateInterval.WeekOfYear, IncVal, dt)
      End If

      Alpha = DateToStr(dt)
      StoAlphaVarValue(sVar)
      Alpha = s
    Else
      sErrMessage = "Variable " & sVar & " is undefined in ADDDT command"
    End If
  End Sub

  Private Function DiffDt(ByVal sVar1 As String, ByVal sVar2 As String, ByVal sUnitToAdd As String) As Integer
    Dim dt1, dt2 As Date
    Dim I, J As Integer
    Dim s, s2 As String

    I = FindAlphaVar(sVar1)
    J = FindAlphaVar(sVar2)

    If I > 0 And J > 0 Then
      s = Alpha
      RclAlphaVarValue(sVar1)
      dt1 = StrToDate(Alpha)
      RclAlphaVarValue(sVar2)
      dt2 = StrToDate(Alpha)
      sUnitToAdd = sUnitToAdd.Trim.ToUpper
      Alpha = s
      If sUnitToAdd = "DAY" Then
        Return DateDiff(DateInterval.Day, dt1, dt2)
      ElseIf sUnitToAdd = "MONTH" Then
        Return DateDiff(DateInterval.Month, dt1, dt2)
      ElseIf sUnitToAdd = "YEAR" Then
        Return DateDiff(DateInterval.Year, dt1, dt2)
      ElseIf sUnitToAdd = "HOUR" Then
        Return DateDiff(DateInterval.Hour, dt1, dt2)
      ElseIf sUnitToAdd = "MINUTE" Then
        Return DateDiff(DateInterval.Minute, dt1, dt2)
      ElseIf sUnitToAdd = "SECOND" Then
        Return DateDiff(DateInterval.Second, dt1, dt2)
      ElseIf sUnitToAdd = "WEEK" Then
        Return DateDiff(DateInterval.Weekday, dt1, dt2)
      ElseIf sUnitToAdd = "WEEKOFYEAR" Then
        Return DateDiff(DateInterval.WeekOfYear, dt1, dt2)
      End If
    Else
      If I < 1 And J > 0 Then
        sErrMessage = "Variable " & sVar1 & " is undefined in DIFFDT command"
      ElseIf J < 1 And I > 0 Then
        sErrMessage = "Variable " & sVar2 & " is undefined in DIFFDT command"
      Else
        sErrMessage = "Variables " & sVar1 & " and " & sVar2 & " are undefined in DIFFDT command"
      End If
    End If
    Return 0
  End Function

  '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  Public Function IsPrgmOK() As Boolean
    Return bOK
  End Function

  Private Sub ExecPrgmLine(ByVal tok As String)
    Dim alfa, s, s2, s3, StrArr() As String
    Dim I, J, K, N As Integer
    Dim X, Y As Double

    bUpdatePrgmPtr = True
    If tok.Trim = "" Then
      ' empty
    ElseIf tok.StartsWith("GTOIND") Then
      s = tok.Substring(6).Trim
      Try
        I = ParseInt(s) ' get index of indrection register
        I = GetMem(I)
        If I >= 0 And I <= MAX_MEM Then
          I = FindLbl(CStr(I))
          If I > 0 Then
            PrgmPtr = LblsIndex(I)
            bUpdatePrgmPtr = False
          Else
            sErrMessage = "Label Not found In GTOIND command"
          End If
        Else
          sErrMessage = "Invalid indirection register In GTOIND command"
        End If
      Catch
        sErrMessage = "Error In GTOIND command"
      End Try
    ElseIf tok.StartsWith("GTO") Then
      s = tok.Substring(3).Trim
      I = FindLbl(s)
      PrgmPtr = LblsIndex(I)
      bUpdatePrgmPtr = False
    ElseIf tok.StartsWith("GSBIND") Then
      s = tok.Substring(6).Trim
      Try
        I = ParseInt(s) ' get index of indrection register
        I = GetMem(I)
        If I >= 0 And I <= MAX_MEM Then
          I = FindLbl(CStr(I))
          If I > 0 Then
            RtnCount += 1
            If RtnCount > NumLocalVars Then
              objLocalVars.AddVar(CStr(RtnCount), INIT_NUM_LOCAL_REGS)
              NumLocalVars += 1
            Else
              ' cleasr the values in an existing local variable object
              objLocalVars.Reset(CStr(RtnCount))
              objLocalVars.Resize(CStr(RtnCount), INIT_NUM_LOCAL_REGS)
            End If
            RtnPtrs(RtnCount) = PrgmPtr + 1 ' set to resume after RTN to the next instruction
            PrgmPtr = LblsIndex(I) + 1 ' resume execution after the GOSUB's label
            bUpdatePrgmPtr = False
          Else
            sErrMessage = "Label Not found In GSBIND command"
          End If
        Else
          sErrMessage = "Invalid indirection register In GSBIND command"
        End If
      Catch
        sErrMessage = "Error In GSBIND command"
      End Try
    ElseIf tok.StartsWith("GSB") Then
      s = tok.Substring(3).Trim
      I = FindLbl(s)
      RtnCount += 1
      RtnPtrs(RtnCount) = PrgmPtr + 1 ' set to resume after RTN to the next instruction
      PrgmPtr = LblsIndex(I) + 1 ' resume execution after the GOSUB's label
      bUpdatePrgmPtr = False
    ElseIf tok = "RTN" Then
      If RtnCount > 0 Then
        PrgmPtr = RtnPtrs(RtnCount)
        RtnCount -= 1
        bUpdatePrgmPtr = False
      Else
        bStop = True
      End If
    ElseIf tok.StartsWith("XEQ") Then
      ' execute runtime simple function
      s = tok.Substring(3).Trim
      s2 = Alpha
      If s.Length > 0 Then
        RclAlphaVarValue(s)
      End If
      ExecAlphaSub()
      Alpha = s2 ' restore original contents of the ALpha register
    ElseIf tok = "CLA" Then
      Alpha = ""
    ElseIf "!@#$%".IndexOf(tok(0)) > -1 Or tok.StartsWith("C ") Or
           tok.StartsWith("REM ") Then ' comment line
      ' do nothing
    ElseIf tok.StartsWith("'") Or tok.StartsWith(DOUBLE_QUOTE) Then
      If tok.StartsWith(DOUBLE_QUOTE) Then tok = "'" & tok.Substring(1) ' replace leading double
      Alpha = tok.Substring(1) ' copy 2nd character and on in Alpha register
      I = Alpha.LastIndexOf("'")
      J = Alpha.LastIndexOf(DOUBLE_QUOTE)
      N = Alpha.Length
      If I = N - 1 Or J = N - 1 Then
        Alpha = Alpha.Substring(0, N - 1)
      End If
    ElseIf tok.StartsWith("|-")
      Alpha = Alpha & tok.Substring(2) ' append 2nd character and on in Alpha register
      I = Alpha.LastIndexOf("'")
      J = Alpha.LastIndexOf(DOUBLE_QUOTE)
      N = Alpha.Length
      If I = N - 1 Or J = N - 1 Then
        Alpha = Alpha.Substring(0, N - 1)
      End If
    ElseIf tok.StartsWith("VIEW") Then
      If tok = "VIEWX" Or tok = "VIEW" Then
        WriteLine("X: " & Output(Stack(X_STACK_REG)))
      ElseIf tok = "VIEWY" Then
        WriteLine("Y: " & Output(Stack(Y_STACK_REG)))
      ElseIf tok = "VIEWZ" Then
        WriteLine("Z: " & Output(Stack(Z_STACK_REG)))
      ElseIf tok = "VIEWT" Then
        WriteLine("T: " & Output(Stack(T_STACK_REG)))
      ElseIf tok.StartsWith("VIEWMEM") Then
        s = tok.Substring(7).Trim
        I = ParseInt(s)
        WriteLine("Mem({0}) = {1}", I, Output(GetMem(I)))
      ElseIf tok = "VIEWREGS" Then
        ViewMemRegisters()
        WriteLine("Mem({0}) = {1}", I, Output(GetMem(I)))
      ElseIf tok.StartsWith("VIEWVAR") Then
        s = tok.Substring(7).Trim
        If s.Length = 0 Then s = Alpha
        WriteLine("Var['{0}'] = {1}", s, Output(RclVarValue(s)))
      ElseIf tok.StartsWith("VIEWDT") Then
        s = tok.Substring(6).Trim
        ViewDate(s)
      End If
    ElseIf tok.StartsWith("AVIEW") Then
      If tok.StartsWith("AVIEW2") Then
        s = tok.Substring(6).Trim
        If s.Length = 0 Then
          WriteLine(Alpha)
        Else
          I = FindAlphaVar(s)
          If I > 0 Then
            s = AlphaVarsValues(I)
            WriteLine(s)
          Else
            WriteLine("Missing text variable " & s)
          End If
        End If
        WriteLine("Press Enter to resume ...")
        s = ReadLine()
      Else ' command is AVIEW
        s = tok.Substring(5).Trim
        If s.Length = 0 Then
          WriteLine(Alpha)
        Else
          I = FindAlphaVar(s)
          If I > 0 Then
            s = AlphaVarsValues(I)
            WriteLine(s)
          Else
            WriteLine("Missing text variable " & s)
          End If
        End If
      End If
    ElseIf tok.StartsWith("VIEWD") Then
      s = tok.Substring(5).Trim
      ViewDate(s)
    ElseIf tok.StartsWith("X") And EndsWith(tok, "Y?", I) Then
      If tok.StartsWith("X=Y?") Then
        If Stack(X_STACK_REG) <> Stack(Y_STACK_REG) Then
          PrgmPtr += I
        End If
      ElseIf tok.StartsWith("X!=Y?") OrElse tok.StartsWith("X<>Y?") Then
        If Stack(X_STACK_REG) = Stack(Y_STACK_REG) Then
          PrgmPtr += I
        End If
      ElseIf tok.StartsWith("X>Y?") Then
        If Stack(X_STACK_REG) <= Stack(Y_STACK_REG) Then
          PrgmPtr += I
        End If
      ElseIf tok.StartsWith("X>=Y?") Then
        If Stack(X_STACK_REG) < Stack(Y_STACK_REG) Then
          PrgmPtr += I
        End If
      ElseIf tok.StartsWith("X<Y?") Then
        If Stack(X_STACK_REG) >= Stack(Y_STACK_REG) Then
          PrgmPtr += I
        End If
      ElseIf tok.StartsWith("X<=Y?") Then
        If Stack(X_STACK_REG) > Stack(Y_STACK_REG) Then
          PrgmPtr += I
        End If
      Else
        sErrMessage = "Invalid X_X?n test"
      End If
    ElseIf tok.StartsWith("X") And EndsWith(tok, "0?", I) Then
      If tok.StartsWith("X=0?") Then
        If Stack(X_STACK_REG) <> 0 Then
          PrgmPtr += I
        End If
      ElseIf tok.StartsWith("X!=0?") OrElse tok.StartsWith("X<>0?") Then
        If Stack(X_STACK_REG) = 0 Then
          PrgmPtr += I
        End If
      ElseIf tok.StartsWith("X>0?") Then
        If Stack(X_STACK_REG) <= 0 Then
          PrgmPtr += I
        End If
      ElseIf tok.StartsWith("X>=0?") Then
        If Stack(X_STACK_REG) < 0 Then
          PrgmPtr += I
        End If
      ElseIf tok.StartsWith("X<0?") Then
        If Stack(X_STACK_REG) >= 0 Then
          PrgmPtr += I
        End If
      ElseIf tok.StartsWith("X<=0?") Then
        If Stack(X_STACK_REG) > 0 Then
          PrgmPtr += I
        End If
      Else
        sErrMessage = "Invalid X_0?n test"
      End If

    ElseIf tok.StartsWith("X?") Then
      If tok.StartsWith("X?=") Then
        s = tok.Substring(3).Trim
        I = ParseInt(s)
        If Stack(X_STACK_REG) <> GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("X?<>") OrElse tok.StartsWith("X?!=") Then
        s = tok.Substring(4).Trim
        I = ParseInt(s)
        If Stack(X_STACK_REG) = GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("X?>") Then
        s = tok.Substring(3).Trim
        I = ParseInt(s)
        If Stack(X_STACK_REG) <= GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("X?>=") Then
        s = tok.Substring(4).Trim
        I = ParseInt(s)
        If Stack(X_STACK_REG) < GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("X?<") Then
        s = tok.Substring(3).Trim
        I = ParseInt(s)
        If Stack(X_STACK_REG) >= GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("X?<=") Then
        s = tok.Substring(4).Trim
        I = ParseInt(s)
        If Stack(X_STACK_REG) > GetMem(I) Then
          PrgmPtr += 1
        End If
      Else
        sErrMessage = "Invalid X? test"
      End If

    ElseIf tok.StartsWith("Y?") Then
      If tok.StartsWith("Y?=") Then
        s = tok.Substring(3).Trim
        I = ParseInt(s)
        If Stack(Y_STACK_REG) <> GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("Y?<>") OrElse tok.StartsWith("Y?!=") Then
        s = tok.Substring(4).Trim
        I = ParseInt(s)
        If Stack(Y_STACK_REG) = GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("Y?>") Then
        s = tok.Substring(3).Trim
        I = ParseInt(s)
        If Stack(Y_STACK_REG) <= GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("Y?>=") Then
        s = tok.Substring(4).Trim
        I = ParseInt(s)
        If Stack(Y_STACK_REG) < GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("Y?<") Then
        s = tok.Substring(3).Trim
        I = ParseInt(s)
        If Stack(Y_STACK_REG) >= GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("Y?<=") Then
        s = tok.Substring(4).Trim
        I = ParseInt(s)
        If Stack(Y_STACK_REG) > GetMem(I) Then
          PrgmPtr += 1
        End If
      Else
        sErrMessage = "Invalid Y? test"
      End If

    ElseIf tok.StartsWith("Z?") Then
      If tok.StartsWith("Z?=") Then
        s = tok.Substring(3).Trim
        I = ParseInt(s)
        If Stack(Z_STACK_REG) <> GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("Z?<>") OrElse tok.StartsWith("Z?!=") Then
        s = tok.Substring(4).Trim
        I = ParseInt(s)
        If Stack(Z_STACK_REG) = GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("Z?>") Then
        s = tok.Substring(3).Trim
        I = ParseInt(s)
        If Stack(Z_STACK_REG) <= GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("Z?>=") Then
        s = tok.Substring(4).Trim
        I = ParseInt(s)
        If Stack(Z_STACK_REG) < GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("Z?<") Then
        s = tok.Substring(3).Trim
        I = ParseInt(s)
        If Stack(Z_STACK_REG) >= GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("Z?<=") Then
        s = tok.Substring(4).Trim
        I = ParseInt(s)
        If Stack(Z_STACK_REG) > GetMem(I) Then
          PrgmPtr += 1
        End If
      Else
        sErrMessage = "Invalid Z? test"
      End If

    ElseIf tok.StartsWith("T?") Then
      If tok.StartsWith("T?=") Then
        s = tok.Substring(3).Trim
        I = ParseInt(s)
        If Stack(T_STACK_REG) <> GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("T?<>") OrElse tok.StartsWith("T?!=") Then
        s = tok.Substring(4).Trim
        I = ParseInt(s)
        If Stack(T_STACK_REG) = GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("T?>") Then
        s = tok.Substring(3).Trim
        I = ParseInt(s)
        If Stack(T_STACK_REG) <= GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("T?>=") Then
        s = tok.Substring(4).Trim
        I = ParseInt(s)
        If Stack(T_STACK_REG) < GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("T?<") Then
        s = tok.Substring(3).Trim
        I = ParseInt(s)
        If Stack(T_STACK_REG) >= GetMem(I) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("T?<=") Then
        s = tok.Substring(4).Trim
        I = ParseInt(s)
        If Stack(T_STACK_REG) > GetMem(I) Then
          PrgmPtr += 1
        End If
      Else
        sErrMessage = "Invalid T? test"
      End If

    ElseIf tok.StartsWith("IS_") Then
      If tok = "IS_BETWEEN" Then
        If Not IsBetween() Then
          PrgmPtr += 1
        End If
      ElseIf tok = "IS_WITHIN" Then
        If Not IsWithin() Then
          PrgmPtr += 1
        End If
      ElseIf tok = "IS_OUTSIDE" Then
        If Not IsOutside() Then
          PrgmPtr += 1
        End If
      ElseIf tok = "IS_WITHOUT" Then
        If Not IsWithout() Then
          PrgmPtr += 1
        End If
      Else
        sErrMessage = "Invalid IS_ test command"
      End If

    ElseIf tok.StartsWith("DSE") Then
      If tok.StartsWith("DSEIND41") Then
        s = tok.Substring(8).Trim
        If DseInd41(s) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("DSE41") Then
        s = tok.Substring(5).Trim
        If Dse41(s) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("DSEIND") Then
        s = tok.Substring(6).Trim
        If DseInd(s) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("DSE") Then
        s = tok.Substring(3).Trim
        If Dse(s) Then
          PrgmPtr += 1
        End If
      Else
        sErrMessage = "Invalid type Of DSE command"
      End If
    ElseIf tok.StartsWith("DSL")
      If tok.StartsWith("DSLIND41") Then
        s = tok.Substring(8).Trim
        If DslInd41(s) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("DSLIND") Then
        s = tok.Substring(6).Trim
        If DslInd(s) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("DSL41") Then
        s = tok.Substring(5).Trim
        If Dsl41(s) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("DSL") Then
        s = tok.Substring(3).Trim
        If Dsl(s) Then
          PrgmPtr += 1
        End If
      Else
        sErrMessage = "Invalid type Of DSL command"
      End If
    ElseIf tok.StartsWith("FOR") Then
      If tok.StartsWith("FORCL") Then
        s = tok.Substring(5).Trim
        If s.Length = 0 Then s = Alpha
        Enter(ForRcl(s))
      ElseIf tok.StartsWith("FORSET") Then
        s = tok.Substring(6).Trim
        If s.Length = 0 Then s = Alpha
        ForSet(s, Stack(X_STACK_REG), Stack(Y_STACK_REG), Stack(Z_STACK_REG))
      ElseIf tok.StartsWith("FOR") Then
        s = tok.Substring(3).Trim
        If s.Length = 0 Then s = Alpha
        If Not ForLoop(s) Then
          PrgmPtr += 1
        End If
      Else
        sErrMessage = "Invalid type Of FORxx command"
      End If
    ElseIf tok.StartsWith("ISG") Then
      If tok.StartsWith("ISGIND41") Then
        s = tok.Substring(8).Trim
        If IsgInd41(s) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("ISG41") Then
        s = tok.Substring(5).Trim
        If Isg41(s) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("ISGIND") Then
        s = tok.Substring(6).Trim
        If IsgInd(s) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("ISG") Then
        s = tok.Substring(3).Trim
        If Isg(s) Then
          PrgmPtr += 1
        End If
      Else
        sErrMessage = "Invalid type Of ISG command"
      End If

    ElseIf tok.StartsWith("FLIP") Then
      s = tok.Substring(4).Trim
      I = ParseInt(s)
      If I >= 0 And I <= MAX_FLAGS Then
        Flags(I) = Not Flags(I)
      Else
        sErrMessage = "Invalid index In the FLIP command"
      End If
    ElseIf tok.StartsWith("JUMP") Then
      If tok = "JUMPX" Then
        I = Fix(Stack(X_STACK_REG))
        JumpSteps(I)
      ElseIf tok = "JUMPY" Then
        I = Fix(Stack(Y_STACK_REG))
        JumpSteps(I)
      ElseIf tok = "JUMPZ" Then
        I = Fix(Stack(Z_STACK_REG))
        JumpSteps(I)
      ElseIf tok = "JUMPT" Then
        I = Fix(Stack(T_STACK_REG))
        JumpSteps(I)
      ElseIf tok.StartsWith("JUMPIND") Then
        Try
          s = tok.Substring(7).Trim
          I = ParseInt(s)
          JumpSteps(GetMem(I))
        Catch
          sErrMessage = "Error In the JUMPIND command"
        End Try
      Else ' jump forward
        Try
          s = tok.Substring(4).Trim
          I = ParseInt(s)
          JumpSteps(I)
        Catch
          sErrMessage = "Error In the JUMP command"
        End Try
      End If
    ElseIf tok.StartsWith("FS?") And tok.IndexOf("IND") > 2 Then
      If tok.StartsWith("FS?C IND") Then
        s = tok.Substring(8).Trim
        If Not IsFlagSetThenClear(s, True) Then
          PrgmPtr += 2
        End If
      ElseIf tok.StartsWith("FS?CIND") Then
        s = tok.Substring(7).Trim
        If Not IsFlagSetThenClear(s, True) Then
          PrgmPtr += 2
        End If
      ElseIf tok.StartsWith("FS?S IND") Then
        s = tok.Substring(8).Trim
        If Not IsFlagSetThenSet(s, True) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FS?SIND") Then
        s = tok.Substring(7).Trim
        If Not IsFlagSetThenSet(s, True) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FS?FLIP IND") Then
        s = tok.Substring(11).Trim
        If Not IsFlagSetThenFlip(s, True) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FS?FLIPIND") Then
        s = tok.Substring(10).Trim
        If Not IsFlagSetThenFlip(s, True) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FS? IND") Then
        s = tok.Substring(7).Trim
        If Not IsFlagSet(s, True) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FS?IND") Then
        s = tok.Substring(6).Trim
        If Not IsFlagSet(s, True) Then
          PrgmPtr += 1
        End If
      Else
        sErrMessage = "Invalid indirect flag Set test"
      End If
    ElseIf tok.StartsWith("FC?") And tok.IndexOf(" IND") > 2 Then
      If tok.StartsWith("FC?C IND") Then
        s = tok.Substring(8).Trim
        If Not IsFlagClearThenClear(s, True) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FC?CIND") Then
        s = tok.Substring(7).Trim
        If Not IsFlagClearThenClear(s, True) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FC?S IND") Then
        s = tok.Substring(8).Trim
        If Not IsFlagClearThenSet(s, True) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FC?SIND") Then
        s = tok.Substring(7).Trim
        If Not IsFlagClearThenSet(s, True) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FC?FLIP IND") Then
        s = tok.Substring(11).Trim
        If Not IsFlagClearThenFlip(s, True) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FC?FLIPIND") Then
        s = tok.Substring(10).Trim
        If Not IsFlagClearThenFlip(s, True) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FC? IND") Then
        s = tok.Substring(7).Trim
        If Not IsFlagClear(s, True) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FC?IND") Then
        s = tok.Substring(6).Trim
        If Not IsFlagClear(s, True) Then
          PrgmPtr += 1
        End If
      Else
        sErrMessage = "Invalid indirect flag clear test"
      End If
    ElseIf tok.StartsWith("FS?") Then
      If tok.StartsWith("FS?C") Then
        s = tok.Substring(4).Trim
        If Not IsFlagSetThenClear(s) Then
          PrgmPtr += 2
        End If
      ElseIf tok.StartsWith("FS?S") Then
        s = tok.Substring(4).Trim
        If Not IsFlagSetThenSet(s) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FS?FLIP") Then
        s = tok.Substring(7).Trim
        If Not IsFlagSetThenFlip(s) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FS?S") Then
        s = tok.Substring(4).Trim
        If Not IsFlagSet(s) Then
          PrgmPtr += 1
        End If
      Else
        sErrMessage = "Invalid Flag Set test"
      End If
    ElseIf tok.StartsWith("FC?") Then
      If tok.StartsWith("FC?C") Then
        s = tok.Substring(4).Trim
        If Not IsFlagClearThenClear(s) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FC?S") Then
        s = tok.Substring(4).Trim
        If Not IsFlagClearThenSet(s) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FC?FLIP") Then
        s = tok.Substring(7).Trim
        If Not IsFlagClearThenFlip(s) Then
          PrgmPtr += 1
        End If
      ElseIf tok.StartsWith("FC?") Then
        s = tok.Substring(3).Trim
        If Not IsFlagClear(s) Then
          PrgmPtr += 1
        End If
      Else
        sErrMessage = "Invalid flag clear test"
      End If
    ElseIf tok.StartsWith("GFTEST?") Then
      If tok.StartsWith("GFTEST?S") Then
        s = tok.Substring(8).Trim
        If s.Length = 0 Then Alpha = s
        If s.Length > 0 Then
          I = Abs(Fix(Stack(X_STACK_REG)))
          If Not fxGroupFlagTestSet(I, s) Then
            PrgmPtr += 1
          End If
        Else
          sErrMessage = "Missing argument incommand  GFTEST?S"
        End If
      ElseIf tok.StartsWith("GFTEST?C") Then
        s = tok.Substring(8).Trim
        If s.Length = 0 Then Alpha = s
        If s.Length > 0 Then
          I = Abs(Fix(Stack(X_STACK_REG)))
          If Not fxGroupFlagTestClear(I, s) Then
            PrgmPtr += 1
          End If
        Else
          sErrMessage = "Missing argument incommand  GFTEST?C"
        End If
      ElseIf tok.StartsWith("GFTEST?FLIP") Then
        s = tok.Substring("GFTEST?FLIP".Length).Trim
        If s.Length = 0 Then Alpha = s
        If s.Length > 0 Then
          I = Abs(Fix(Stack(X_STACK_REG)))
          If Not fxGroupFlagTestFlip(I, s) Then
            PrgmPtr += 1
          End If
        Else
          sErrMessage = "Missing argument incommand  GFTEST?FLIP"
        End If
      Else
        s = tok.Substring(7).Trim
        If s.Length = 0 Then Alpha = s
        If s.Length > 0 Then
          I = Abs(Fix(Stack(X_STACK_REG)))
          If Not fxGroupFlagTest(I, s) Then
            PrgmPtr += 1
          End If
        Else
          sErrMessage = "Missing argument incommand  GFTEST?"
        End If
      End If
    ElseIf tok.StartsWith("LBL") Then
      ' do nothing
    ElseIf tok = "PROMPT" Then
      s = Alpha
      If Not s.EndsWith(" ") Then s &= " "
      Write(s)
      s = ReadLine()

      If IsNumeric(s) Then
        Enter(ParseDbl(s))
      Else
        Alpha = s
      End If
    ElseIf tok = "-X-" Then
      WriteLine(Output(Stack(X_STACK_REG)))
      Thread.Sleep(2000)
    ElseIf tok.StartsWith("PSE") Then
      Try
        If tok.Length = 3 Then
          Thread.Sleep(CLng(Fix(Stack(X_STACK_REG))))
        Else
          Thread.Sleep(Long.Parse(tok.Substring(3).Trim))
        End If
      Catch
        WriteLine("Warning! Problem With pause duration value. Pausing For 2 seconds.")
        Thread.Sleep(2000)
      End Try
    ElseIf tok = "STOP" Or tok = "R/S" Then
      ShowStack()
      WriteLine("Press Enter To Resume program execution ..")
      s = ReadLine()
      s = s.Replace(" ", "")
      If IsNumeric(s) Then
        X = CDbl(s)
        Enter(X)
      End If
    ElseIf tok.StartsWith("VSTO") Then
      If tok.StartsWith("VSTO+") Then
        s = tok.Substring(5).Trim
        If s.Length = 0 Then s = Alpha
        StoVarValue(s, RclVarValue(s) + Stack(X_STACK_REG))
      ElseIf tok.StartsWith("VSTO-") Then
        s = tok.Substring(5).Trim
        If s.Length = 0 Then s = Alpha
        StoVarValue(s, RclVarValue(s) - Stack(X_STACK_REG))
      ElseIf tok.StartsWith("VSTO*") Then
        s = tok.Substring(5).Trim
        If s.Length = 0 Then s = Alpha
        StoVarValue(s, RclVarValue(s) * Stack(X_STACK_REG))
      ElseIf tok.StartsWith("VSTO-") Then
        s = tok.Substring(5).Trim
        If s.Length = 0 Then s = Alpha
        StoVarValue(s, RclVarValue(s) / Stack(X_STACK_REG))
      ElseIf tok.StartsWith("VSTO/") Then
        s = tok.Substring(5).Trim
        If s.Length = 0 Then s = Alpha
        X = Stack(X_STACK_REG)
        If X <> 0 Then
          StoVarValue(s, RclVarValue(s) / Stack(X_STACK_REG))
        Else
          sErrMessage = "division by zero In VSTO/ command"
        End If
      ElseIf tok.StartsWith("VSTO") Then
        s = tok.Substring(4).Trim
        If s.Length = 0 Then s = Alpha
        StoVarValue(s, Stack(X_STACK_REG))
      Else
        sErrMessage = "Invalid VSTO command"
      End If
    ElseIf tok.StartsWith("VRCL") Then
      If tok.StartsWith("VRCL+") Then
        s = tok.Substring(5).Trim
        If s.Length = 0 Then s = Alpha
        X = RclVarValue(s)
        Stack(X_STACK_REG) += X
      ElseIf tok.StartsWith("VRCL-") Then
        s = tok.Substring(5).Trim
        If s.Length = 0 Then s = Alpha
        X = RclVarValue(s)
        Stack(X_STACK_REG) -= X
      ElseIf tok.StartsWith("VRCL*") Then
        s = tok.Substring(5).Trim
        If s.Length = 0 Then s = Alpha
        X = RclVarValue(s)
        Stack(X_STACK_REG) *= X
      ElseIf tok.StartsWith("VRCL/") Then
        s = tok.Substring(5).Trim
        If s.Length = 0 Then s = Alpha
        X = RclVarValue(s)
        If X <> 0 Then
          Stack(X_STACK_REG) /= X
        Else
          sErrMessage = "Divide by zero In VRCL/ command"
        End If
      ElseIf tok.StartsWith("VRCL") Then
        s = tok.Substring(4).Trim
        If s.Length = 0 Then s = Alpha
        Enter(RclVarValue(s))
      Else
        sErrMessage = "Invalid VRCL command"
      End If
    ElseIf tok = "ONERROFF" Then
      bOnErrReumeNext = False
      bOnErrGoto = False
    ElseIf tok = "ONERRRESUME" Then
      bOnErrReumeNext = True
    ElseIf tok.StartsWith("ONERRGOTO") Then
      s = tok.Substring(10).Trim
      I = FindLbl(s)
      If I > 0 Then
        ErrHandlingLine = LblsIndex(I)
        bOnErrGoto = True
      Else
        sErrMessage = "Invalid label For the Error handler"
      End If
    ElseIf tok = "ATOX"
      If IsNumeric(Alpha) Then
        Enter(ParseDbl(Alpha))
      Else
        sErrMessage = "Cannot convert contents Of the Alpha register To a number"
      End If
    ElseIf tok.StartsWith("VCA") Then
      If tok.IndexOf("VCA+") >= 0 Then
        fxVCAadd(tok)
      ElseIf tok.IndexOf("VCA-") >= 0 Then
        fxVCAsubt(tok)
      ElseIf tok.IndexOf("VCA*") >= 0 Then
        fxVCAmult(tok)
      ElseIf tok.IndexOf("VCA/") >= 0 Then
        fxVCAdiv(tok)
      ElseIf tok.IndexOf("VCAS+") >= 0 Then
        fxVCASadd(tok)
      ElseIf tok.IndexOf("VCAS-") >= 0 Then
        fxVCASsubt(tok)
      ElseIf tok.IndexOf("VCASA-") >= 0 Then
        fxVCASAsubt(tok)
      ElseIf tok.IndexOf("VCAS*") >= 0 Then
        fxVCASmult(tok)
      ElseIf tok.IndexOf("VCAS/") >= 0 Then
        fxVCASdiv(tok)
      ElseIf tok.IndexOf("VCASA/") >= 0 Then
        fxVCASAdiv(tok)
      ElseIf tok.IndexOf("VCAFX") >= 0 Then
        fxVCAfx(tok)
      Else
        sErrMessage = "Invalid VCAxxx command"
      End If
    ElseIf tok.StartsWith("ASTO") Then
      s = tok.Substring(4).Trim
      StoAlphaVarValue(s)
    ElseIf tok.StartsWith("ARCL") Then
      If tok.StartsWith("ARCLS+") Then
        s = tok.Substring(4).Trim
        Alpha &= " " & GetAlphaVarValue(s)
      ElseIf tok.StartsWith("ARCLS-") Then
        s = tok.Substring(4).Trim
        Alpha = GetAlphaVarValue(s) & " " & Alpha
      ElseIf tok.StartsWith("ARCL+") Then
        s = tok.Substring(4).Trim
        Alpha &= GetAlphaVarValue(s)
      ElseIf tok.StartsWith("ARCL-") Then
        s = tok.Substring(4).Trim
        Alpha = GetAlphaVarValue(s) & Alpha
      ElseIf tok = "ARCLX" Then
        Alpha &= CStr(Stack(X_STACK_REG))
      ElseIf tok = "ARCLY" Then
        Alpha &= CStr(Stack(Y_STACK_REG))
      ElseIf tok = "ARCLZ" Then
        Alpha &= CStr(Stack(Z_STACK_REG))
      ElseIf tok = "ARCLT" Then
        Alpha &= CStr(Stack(T_STACK_REG))
      ElseIf tok = "ARCLIX" Then
        Alpha &= CStr(Fix(Stack(X_STACK_REG)))
      ElseIf tok = "ARCLIY" Then
        Alpha &= CStr(Fix(Stack(Y_STACK_REG)))
      ElseIf tok = "ARCLIZ" Then
        Alpha &= CStr(Fix(Stack(Z_STACK_REG)))
      ElseIf tok = "ARCLIT" Then
        Alpha &= CStr(Fix(Stack(T_STACK_REG)))
      ElseIf tok.StartsWith("ARCL") Then
        s = tok.Substring(4).Trim
        RclAlphaVarValue(s)
      Else
        sErrMessage = "Invalid ARCL comand"
      End If
    ElseIf tok.IndexOf("->") > -1 Then
      If tok = "X->A" Then
        Alpha &= CStr(Stack(X_STACK_REG))
      ElseIf tok = "Y->A" Then
        Alpha &= CStr(Stack(Y_STACK_REG))
      ElseIf tok = "Z->A" Then
        Alpha &= CStr(Stack(Z_STACK_REG))
      ElseIf tok = "T->A" Then
        Alpha &= CStr(Stack(T_STACK_REG))
      ElseIf tok = "IX->A" Then
        Alpha &= CStr(Fix(Stack(X_STACK_REG)))
      ElseIf tok = "IY->A" Then
        Alpha &= CStr(Fix(Stack(Y_STACK_REG)))
      ElseIf tok = "IZ->A" Then
        Alpha &= CStr(Fix(Stack(Z_STACK_REG)))
      ElseIf tok = "IT->A" Then
        Alpha &= CStr(Fix(Stack(T_STACK_REG)))
      Else
        sErrMessage = "Error In appending To Alpha register"
      End If
    ElseIf tok = "ISPRIME" Then
      If Not IsPrime(Fix(Stack(X_STACK_REG))) Then
        PrgmPtr += 1
      End If
    ElseIf tok = "ISODD" Then
      I = Fix(Stack(X_STACK_REG))
      If I Mod 2 > 1 Then
        PrgmPtr += 1
      End If
    ElseIf tok = "ISEVEN" Then
      I = Fix(Stack(X_STACK_REG))
      If I Mod 2 = 0 Then
        PrgmPtr += 1
      End If
    ElseIf tok = "READDATA2" Then
      ReadData2()
    ElseIf tok = "READDATA3" Then
      ReadData3()
    ElseIf tok = "NORMDATA2" Then
      NormalizeData2(Fix(Stack(Z_STACK_REG)), Stack(X_STACK_REG) > 0, Stack(Y_STACK_REG) > 0)
    ElseIf tok = "NORMDATA3" Then
      NormalizeData3(Fix(Stack(T_STACK_REG)), Stack(X_STACK_REG) > 0, Stack(Y_STACK_REG) > 0, Stack(Z_STACK_REG) > 0)
    ElseIf tok = "SIGMA+T" Or tok = "S+T" Then
      fxSigmaAddFromMem(Alpha)
    ElseIf tok = "LR" Then
      ExecLR()
    ElseIf tok = "BESTLR" Then
      ExecBestLR()
    ElseIf tok = "BESTLRPWRS"
      BackupStack()
      Stack(Y_STACK_REG) = BestYPower
      Stack(X_STACK_REG) = BestXPower
    ElseIf tok = "MLR" Then
      ExecMLR()
    ElseIf tok = "BESTMLR" Then
      ExecBestMLR()
    ElseIf tok = "BESTMLRPWRS"
      BackupStack()
      Stack(Z_STACK_REG) = BestZPower
      Stack(Y_STACK_REG) = BestYPower
      Stack(X_STACK_REG) = BestXPower
    ElseIf tok = "POWERLR" Then
      ExecPowerLR()
    ElseIf tok = "LOGXLR" Then
      ExecLogxLR()
      PrgmPtr += 1
    ElseIf tok = "LOGYLR" Then
      ExecLogyLR()
    ElseIf tok = "POWERMLR" Then
      ExecPowerMLR()
    ElseIf tok = "QUADFIT" Then
      ExecQuadFit()
    ElseIf tok.StartsWith("READMEM") Then
      s = tok.Substring(7).Trim
      If s.Length = 0 Then s = Alpha
      ReadMem(s)
    ElseIf tok.StartsWith("WRITEMEM") Then
      s = tok.Substring(8).Trim
      If s.Length = 0 Then s = Alpha
      WriteMem(s)
    ElseIf tok = "END" Or tok = "QUIT" Or tok = "EXIT" Then
      bStop = True
    ElseIf tok = "AINPUT" Then
      WriteLine(Alpha)
      Alpha = ReadLine()
    ElseIf tok.StartsWith("ARCOPY") Then
      s = tok.Substring(6).Trim
      alfa = Alpha.Trim.ToUpper
      If s.Length = 0 Then
        ' check if the Alpha register has the array names we need
        If alfa.Length = 0 Then
          sErrMessage = "Missing two variable names In ARCOPY command"
        End If
        s = alfa
        StrArr = s.Split(", ")
        I = StrArr.GetUpperBound(0)
        ' must have both array names
        If I = 1 Then
          objVarArrs.CopyVar(StrArr(0).Trim, StrArr(1).Trim)
        Else
          sErrMessage = "Missing variable name In ARCOPY command"
        End If
      Else
        ' Is the target mentioned?
        If s.Substring(0, 1) = ", " Then
          s = s.Substring(1) ' remove the leading comma
          objVarArrs.CopyVar(alfa, s)
        ElseIf s.IndexOf(", ") < 0 Then ' command has the source array
          objVarArrs.CopyVar(s, alfa)
        ElseIf s.IndexOf(", ") > 0 Then ' comma is INSIDE string s
          StrArr = s.Split(", ")
          ' copy from one array to another
          objVarArrs.CopyVar(StrArr(0).Trim, StrArr(1).Trim)
        Else
          sErrMessage = "Missing variable name In ARCOPY command"
        End If
      End If
    ElseIf tok.StartsWith("ARNEW") Then
      If tok.StartsWith("ARNEWSEQ") Then
        s = tok.Substring(8).Trim
        If s.Length = 0 Then s = Alpha
        objVarArrs.AddVar(s, Fix(Stack(X_STACK_REG)), Stack(Y_STACK_REG), Stack(Z_STACK_REG))
      ElseIf tok.StartsWith("ARNEWRND") Then
        s = tok.Substring(8).Trim
        If s.Length = 0 Then s = Alpha
        objVarArrs.AddVar(s, Fix(Stack(X_STACK_REG)), True, False, Stack(Y_STACK_REG), Stack(Z_STACK_REG))
      ElseIf tok.StartsWith("ARNEWINTRND") Then
        s = tok.Substring(11).Trim
        If s.Length = 0 Then s = Alpha
        objVarArrs.AddVar(s, Fix(Stack(X_STACK_REG)), True, True, CLng(Stack(Y_STACK_REG)), CLng(Stack(Z_STACK_REG) + 1))
      ElseIf tok.StartsWith("ARNEWNORMRND") Then
        s = tok.Substring(11).Trim
        If s.Length = 0 Then s = Alpha
        objVarArrs.AddVar(s, Fix(Stack(X_STACK_REG)), False, False, Stack(Y_STACK_REG), Stack(Z_STACK_REG))
      ElseIf tok.StartsWith("ARNEWINTNORMRND") Then
        s = tok.Substring(14).Trim
        If s.Length = 0 Then s = Alpha
        objVarArrs.AddVar(s, Fix(Stack(X_STACK_REG)), False, True, Stack(Y_STACK_REG), Stack(Z_STACK_REG))
      ElseIf tok.StartsWith("ARNEW") Then
        s = tok.Substring(5).Trim
        If s.Length = 0 Then s = Alpha
        objVarArrs.AddVar(s, Fix(Stack(X_STACK_REG)))
      ElseIf tok.StartsWith("ARGETSIZE") Then
        s = tok.Substring(5).Trim
        If s.Length = 0 Then s = Alpha
        Enter(objVarArrs.GetSize(s))
      End If
    ElseIf tok.StartsWith("ARFILL") Then
      If tok.StartsWith("ARFILLSEQ") Then
        s = tok.Substring(9).Trim
        If s.Length = 0 Then s = Alpha
        objVarArrs.Fill(s, Fix(Stack(X_STACK_REG)), Stack(Y_STACK_REG))
      ElseIf tok.StartsWith("ARFIllRND") Then
        s = tok.Substring(9).Trim
        If s.Length = 0 Then s = Alpha
        objVarArrs.Fill(s, True, False, Stack(X_STACK_REG), Stack(Y_STACK_REG))
      ElseIf tok.StartsWith("ARFILLINTRND") Then
        s = tok.Substring(12).Trim
        If s.Length = 0 Then s = Alpha
        objVarArrs.Fill(s, True, True, CLng(Stack(X_STACK_REG)), CLng(Stack(Y_STACK_REG)))
      ElseIf tok.StartsWith("ARFILLNORMRND") Then
        s = tok.Substring(13).Trim
        If s.Length = 0 Then s = Alpha
        objVarArrs.Fill(s, False, False, Stack(X_STACK_REG), Stack(Y_STACK_REG))
      End If
    ElseIf tok.StartsWith("ARDEL") Then
      s = tok.Substring(5).Trim
      If s.Length = 0 Then s = Alpha
      objVarArrs.RemoveVar(s)
    ElseIf tok.StartsWith("ARSWAP") Then
      s = tok.Substring(5).Trim
      If s.Length = 0 Then s = Alpha
      StrArr = s.Split(" ")
      If StrArr.GetUpperBound(0) > 0 Then
        objVarArrs.SwapVars(StrArr(0), StrArr(1))
        s = objVarArrs.GetErrorMessage
        If s.Length > 0 Then sErrMessage = s
      Else
        sErrMessage = "Missing array name(s) In command ARSWAP"
      End If
    ElseIf tok.EndsWith("IDX") Then
      If tok = "RIDX" Then
        s = tok.Substring(4).Trim()
        If s.Length = 0 Then s = Alpha
        I = Fix(Stack(X_STACK_REG))
        objVarArrs.SetRecallIndex(s, I)
      ElseIf tok = "SIDX" Then
        s = tok.Substring(4).Trim()
        If s.Length = 0 Then s = Alpha
        I = Fix(Stack(X_STACK_REG))
        objVarArrs.SetStoreIndex(s, I)
      ElseIf tok = "+RIDX" Then
        s = tok.Substring(4).Trim()
        If s.Length = 0 Then s = Alpha
        I = Fix(Stack(X_STACK_REG))
        objVarArrs.IncRecallIndex(s, I)
      ElseIf tok = "+SIDX" Then
        s = tok.Substring(4).Trim()
        If s.Length = 0 Then s = Alpha
        I = Fix(Stack(X_STACK_REG))
        objVarArrs.IncStoreIndex(s, I)
      ElseIf tok = "GETRDIX" Then
        s = tok.Substring(4).Trim()
        If s.Length = 0 Then s = Alpha
        I = Fix(Stack(X_STACK_REG))
        Enter(objVarArrs.GetRecallIndex(s))
      ElseIf tok = "GETSDIX" Then
        s = tok.Substring(4).Trim()
        If s.Length = 0 Then s = Alpha
        I = Fix(Stack(X_STACK_REG))
        Enter(objVarArrs.GetStoreIndex(s))
      Else
        sErrMessage = "Error In IDX command"
      End If
    ElseIf tok.StartsWith("RST") Then
      If tok.StartsWith("RSTO ") Then
        fxRSTOcom(tok, "RSTO")
      ElseIf tok.StartsWith("RST+") Then
        fxRSTOUpdateIndex(tok, "RST+")
      ElseIf tok.StartsWith("RST-") Then
        fxRSTOUpdateIndex(tok, "RST-")
      ElseIf tok.StartsWith("RST$") Then
        fxRSTOUpdateIndex(tok, "RST$")
      ElseIf tok.StartsWith("RSTO+") Then
        fxRSTOcom(tok, "RSTO+")
      ElseIf tok.StartsWith("RSTO-") Then
        fxRSTOcom(tok, "RSTO-")
      ElseIf tok.StartsWith("RSTO*") Then
        fxRSTOcom(tok, "RSTO*")
      ElseIf tok.StartsWith("RSTO/") Then
        fxRSTOcom(tok, "RSTO/")
      Else
        sErrMessage = "Invalid RSTO command"
      End If
    ElseIf tok.StartsWith("RRC") Then
      If tok.StartsWith("RRCL+ ") Then
        fxRRCLcom(tok, "RRCL+")
      ElseIf tok.StartsWith("RRCL- ") Then
        fxRRCLcom(tok, "RRCL-")
      ElseIf tok.StartsWith("RRCL* ") Then
        fxRRCLcom(tok, "RRCL*")
      ElseIf tok.StartsWith("RRCL/ ") Then
        fxRRCLcom(tok, "RRCL/")
      ElseIf tok.StartsWith("RRCL") Then
        fxRRCLcom(tok, "RRCL")
      ElseIf tok.StartsWith("RRC+") Then
        fxRRCLUpdateIndex(tok, "RRC+")
      ElseIf tok.StartsWith("RRC-") Then
        fxRRCLUpdateIndex(tok, "RRC-")
      ElseIf tok.StartsWith("RRC$") Then
        fxRRCLUpdateIndex(tok, "RRC$")
      Else
        sErrMessage = "Invalid RRCL command"
      End If
    ElseIf tok.StartsWith("LSTO") Then
      If tok.StartsWith("LSTO ") Then
        s = tok.Substring(5).Trim
        I = ParseInt(s)
        objLocalVars.Store(CStr(RtnCount), I, Stack(X_STACK_REG))
      ElseIf tok.StartsWith("LSTO+") Then
        s = tok.Substring(5).Trim
        I = ParseInt(s)
        objLocalVars.StorePlus(CStr(RtnCount), I, Stack(X_STACK_REG))
      ElseIf tok.StartsWith("LSTO-") Then
        s = tok.Substring(5).Trim
        I = ParseInt(s)
        objLocalVars.StoreMinus(CStr(RtnCount), I, Stack(X_STACK_REG))
      ElseIf tok.StartsWith("LSTO*") Then
        s = tok.Substring(5).Trim
        I = ParseInt(s)
        objLocalVars.StoreMult(CStr(RtnCount), I, Stack(X_STACK_REG))
      ElseIf tok.StartsWith("LSTO/") Then
        s = tok.Substring(5).Trim
        I = ParseInt(s)
        objLocalVars.StoreDiv(CStr(RtnCount), I, Stack(X_STACK_REG))
      Else
        sErrMessage = "Invalid LSTO command"
      End If
    ElseIf tok.StartsWith("LRCL") Then
      If tok.StartsWith("LRCL+ ") Then
        s = tok.Substring(6).Trim
        I = ParseInt(s)
        Stack(X_STACK_REG) += objLocalVars.Recall(CStr(RtnCount), I)
      ElseIf tok.StartsWith("LRCL- ") Then
        s = tok.Substring(6).Trim
        I = ParseInt(s)
        Stack(X_STACK_REG) -= objLocalVars.Recall(CStr(RtnCount), I)
      ElseIf tok.StartsWith("LRCL* ") Then
        s = tok.Substring(6).Trim
        I = ParseInt(s)
        Stack(X_STACK_REG) *= objLocalVars.Recall(CStr(RtnCount), I)
      ElseIf tok.StartsWith("LRCL/ ") Then
        s = tok.Substring(6).Trim
        I = ParseInt(s)
        X = objLocalVars.Recall(CStr(RtnCount), I)
        If X <> 0 Then
          Stack(X_STACK_REG) *= X
        Else
          sErrMessage = "Division by zero In the LRCL command"
        End If
      ElseIf tok.StartsWith("LRCL ") Then
        s = tok.Substring(5).Trim
        I = ParseInt(s)
        Enter(objLocalVars.Recall(CStr(RtnCount), I))
      Else
        sErrMessage = "Invalid LRCL command"
      End If
    ElseIf tok.StartsWith("ARRESET") Then
      s = tok.Substring("ARRESET".Length).Trim
      If s.Length = 0 Then
        s = Alpha
      Else
        StrArr = tok.Split(" ")
        s = StrArr(1)
      End If
      objVarArrs.Reset(s)
    ElseIf tok.StartsWith("ARVAR2MEM") Then
      s = tok.Substring("ARVAR2MEM".Length).Trim
      s2 = Alpha
      If s.Length = 0 Then s = Alpha
      Try
        ArrVar2Mem(s)
        Alpha = s2 ' restore Alpha
      Catch
        sErrMessage = "Error In executing the command ARVAR2MEM"
      End Try
    ElseIf tok.StartsWith("MEM2ARVAR") Then
      s = tok.Substring("MEM2ARVAR".Length).Trim
      If s.Length = 0 Then s = Alpha
      Try
        Mem2ArrVar(s)
      Catch
        sErrMessage = "Error In executing the command MEM2ARVAR"
      End Try
    ElseIf tok.StartsWith("READ1VAR ") Then
      StrArr = tok.Split(" ")
      If StrArr.GetUpperBound(0) > 1 Then
        Enter(objVarArrs.Read1Var(StrArr(1), StrArr(2)))
      Else
        sErrMessage = "Missing argument In command READ1VAR"
      End If
    ElseIf tok.StartsWith("READ2VARS ") Then
      StrArr = tok.Split(" ")
      If StrArr.GetUpperBound(0) > 2 Then
        Enter(objVarArrs.Read2Vars(StrArr(1), StrArr(2), StrArr(3)))
      Else
        sErrMessage = "Missing argument In command READ2ARS"
      End If
    ElseIf tok.StartsWith("READ3VARS ") Then
      StrArr = tok.Split(" ")
      If StrArr.GetUpperBound(0) > 3 Then
        Enter(objVarArrs.Read3Vars(StrArr(1), StrArr(2), StrArr(3), StrArr(4)))
      Else
        sErrMessage = "Missing argument In command READ3ARS"
      End If
    ElseIf tok.StartsWith("READ4VARS ") Then
      StrArr = tok.Split(" ")
      If StrArr.GetUpperBound(0) > 4 Then
        Enter(objVarArrs.Read4Vars(StrArr(1), StrArr(2), StrArr(3), StrArr(4), StrArr(5)))
      Else
        sErrMessage = "Missing argument In command READ4ARS"
      End If
    ElseIf tok.StartsWith("READNVARS ") Then
      StrArr = tok.Split(" ")
      N = -2
      For Each s In StrArr
        N += 1
      Next
      N -= 1
      If N > 0 Then
        Enter(objVarArrs.ReadNVars(StrArr(1), N, StrArr))
      Else
        sErrMessage = "Missing variable names In command READNVARS"
      End If
    ElseIf tok.StartsWith("WRITE1VAR ") Then
      StrArr = tok.Split(" ")
      If StrArr.GetUpperBound(0) > 1 Then
        Enter(objVarArrs.Write1Var(StrArr(1), StrArr(2)))
      Else
        sErrMessage = "Missing argument In command WRITE1VAR"
      End If
    ElseIf tok.StartsWith("WRITE2VARS ") Then
      StrArr = tok.Split(" ")
      If StrArr.GetUpperBound(0) > 2 Then
        Enter(objVarArrs.Write2Vars(StrArr(1), StrArr(2), StrArr(3)))
      Else
        sErrMessage = "Missing argument In command WRITE2VARS"
      End If
    ElseIf tok.StartsWith("WRITE3VARS ") Then
      StrArr = tok.Split(" ")
      If StrArr.GetUpperBound(0) > 3 Then
        Enter(objVarArrs.Write3Vars(StrArr(1), StrArr(2), StrArr(3), StrArr(4)))
      Else
        sErrMessage = "Missing argument In command WRITE3VARS"
      End If
    ElseIf tok.StartsWith("WRITE4VARS ") Then
      StrArr = tok.Split(" ")
      If StrArr.GetUpperBound(0) > 4 Then
        Enter(objVarArrs.Write4Vars(StrArr(1), StrArr(2), StrArr(3), StrArr(4), StrArr(5)))
      Else
        sErrMessage = "Missing argument In command WRITE4VARS"
      End If
    ElseIf tok.StartsWith("WRITENVARS ") Then
      StrArr = tok.Split(" ")
      N = -2
      For Each s In StrArr
        N += 1
      Next
      N -= 1
      If N > 0 Then
        Enter(objVarArrs.WriteNVars(StrArr(1), N, StrArr))
      Else
        sErrMessage = "Missing varaible names In command WRITENVARS"
      End If
    ElseIf tok.StartsWith("SAVESTAT") Then
      s = tok.Substring(8)
      If s.Length = 0 Then s = Alpha
      SaveStatRegisters(s)
    ElseIf tok.StartsWith("GETSTAT") Then
      s = tok.Substring(7)
      If s.Length = 0 Then s = Alpha
      GetStatRegisters(s)
    ElseIf tok.StartsWith("MERGESTAT") Then
      s = tok.Substring(9)
      If s.Length = 0 Then s = Alpha
      MergeStatRegisters(s)
    ElseIf tok = "INTEG" Then
      fxIntegrate()
    ElseIf tok.EndsWith("QUAD") Then
      If tok = "GAUSSLEGQUAD" Then
        fxGaussLegendreQuadrature()
      ElseIf tok = "GAUSSLAGQUAD" Then
        fxGaussLaguerreQuadrature()
      ElseIf tok = "GAUSSHERQUAD" Then
        fxGaussHermiteQuadrature()
      ElseIf tok = "GAUSSCHEBQUAD" Then
        fxGaussChebyshevQuadrature()
      ElseIf tok = "GAUSSKRONQUAD" Then
        fxGaussKronrodQuadrature()
      ElseIf tok = "GAUSSKRONQUAD2" Then
        fxGaussKronrodQuadrature2()
      Else
        sErrMessage = "Error in a QUAD type of command"
      End If
    ElseIf tok.StartsWith("SOLVE") Then
      If tok = "SOLVEBIN" Then
        fxSolveBisection()
      ElseIf tok = "SOLVEHAL" Then
        fxSolveHalley()
      ElseIf tok = "SOLVEOST" Then
        fxSolveOstrowski()
      ElseIf tok = "SOLVE" Then
        fxSolve()
      Else
        sErrMessage = "Invalid SOLVE command"
      End If
    ElseIf tok.StartsWith("SCAN") Then
      s = tok.Substring(4)
      If s.Length > 0 Then Alpha = GetAlphaVarValue(s)
      If Alpha.Length = 0 Then
        sErrMessage = "Missing expression for f(x) in SCAN command"
      End If
      fxScan()
    ElseIf tok.StartsWith("SF") Then
      If tok.StartsWith("SFIND ") Then
        s = tok.Substring(2).Trim
        I = Abs(ParseInt(s))
        I = Abs(GetMem(I))
        If I <= MAX_FLAGS Then
          Flags(I) = True
        Else
          sErrMessage = "Invalid indrect address In command SFIND"
        End If
      ElseIf tok.StartsWith("SF ") Then
        s = tok.Substring(2).Trim
        I = Abs(ParseInt(s))
        If I <= MAX_FLAGS Then
          Flags(I) = True
        Else
          sErrMessage = "Invalid flag index In command SF"
        End If
      Else
        sErrMessage = "Invalid type Of SF command"
      End If
    ElseIf tok.StartsWith("CF") Then
      If tok.StartsWith("CFIND ") Then
        s = tok.Substring(2).Trim
        I = Abs(ParseInt(s))
        I = Abs(GetMem(I))
        If I <= MAX_FLAGS Then
          Flags(I) = False
        Else
          sErrMessage = "Invalid indrect address In command CFIND"
        End If
      ElseIf tok.StartsWith("CF ") Then
        s = tok.Substring(2).Trim
        I = Abs(ParseInt(s))
        If I <= MAX_FLAGS Then
          Flags(I) = False
        Else
          sErrMessage = "Invalid flag index In command CF"
        End If
      Else
        sErrMessage = "Invalid type Of CF command"
      End If
    ElseIf tok = "CLRFLGS" Then
      ClearFlags()
    ElseIf tok.StartsWith("STOFLGS") Then
      s = tok.Substring(7).Trim
      If s.Length = 0 Then s = Alpha
      fxStoreFlags(s)
    ElseIf tok.StartsWith("RCLFLGS") Then
      s = tok.Substring(7).Trim
      If s.Length = 0 Then s = Alpha
      fxRecallFlags(s)
    ElseIf tok.StartsWith("DATA") Then
      If tok.StartsWith("DATAS") Then
        s = tok.Substring(5).Trim
        fxAddStatData(s)
      ElseIf tok.StartsWith("DATAM") Then
        s = tok.Substring(5)
        fxAddMemData(s)
      ElseIf tok.StartsWith("DATAR") Then
        s = tok.Substring(5).Trim
        fxAddVarData(s)
      Else
        sErrMessage = "Invalid DATA statement"
      End If
    ElseIf tok.StartsWith("SORT") Then
      If tok.StartsWith("SORTARA") Then
        s = tok.Substring(7).Trim
        If s.Length = 0 Then

          If Not objVarArrs.SortAscending(Alpha) Then
            sErrMessage = "Invalid name Of array In SORTARA command"
          End If

        Else

          If Not objVarArrs.SortAscending(s) Then
            sErrMessage = "Invalid name Of array In SORTARA command"
          End If
        End If

      ElseIf tok.StartsWith("SORTARD") Then
        s = tok.Substring(7).Trim
        If s.Length = 0 Then

          If Not objVarArrs.SortDescending(Alpha) Then
            sErrMessage = "Invalid name Of array In SORTARD command"
          End If

        Else

          If Not objVarArrs.SortDescending(s) Then
            sErrMessage = "Invalid name Of array In SORTARD command"
          End If

        End If

      ElseIf tok = "SORTA" Then
        fxSortMemAscending()
      ElseIf tok = "SORTD" Then
        fxSortMemDescending()
      Else
        sErrMessage = "Undefined type Of SORT command"
      End If
    ElseIf tok.IndexOf("SEARCH") > -1 Then
      If tok.StartsWith("SEARCHA") Then
        BackupStack()
        s = tok.Substring("SEARCHA".Length).Trim
        If s.Length = 0 Then s = Alpha
        Enter(objVarArrs.Search(s, Stack(X_STACK_REG), Fix(Stack(Y_STACK_REG)), Stack(Z_STACK_REG), Fix(Stack(T_STACK_REG))))
      ElseIf tok.StartsWith("CSEARCHA") Then
        BackupStack()
        s = tok.Substring("CSEARCHA".Length).Trim
        If s.Length = 0 Then s = Alpha
        Enter(objVarArrs.CircularSearch(s, Stack(X_STACK_REG), Fix(Stack(Y_STACK_REG)), Stack(Z_STACK_REG), Fix(Stack(T_STACK_REG))))
        Stack(Y_STACK_REG) = I
      ElseIf tok.StartsWith("MSEARCHA") Then
        BackupStack()
        s = tok.Substring("MSEARCHA".Length).Trim
        If s.Length = 0 Then s = Alpha
        Enter(objVarArrs.MiddleFirstSearch(s, Stack(X_STACK_REG), Stack(Y_STACK_REG), Fix(Stack(Z_STACK_REG))))
        Stack(Y_STACK_REG) = I
      ElseIf tok.StartsWith("HSEARCHA") Then
        BackupStack()
        s = tok.Substring("HSEARCHA".Length).Trim
        If s.Length = 0 Then s = Alpha
        J = objVarArrs.HeuristicSearch(s, I, Stack(X_STACK_REG), Fix(Stack(Y_STACK_REG)), Fix(Stack(Z_STACK_REG)), Stack(T_STACK_REG))
        Enter(I)
        Enter(J)
      ElseIf tok.StartsWith("BINSEARCHA") Then
        BackupStack()
        s = tok.Substring("BINSEARCHA".Length).Trim
        If s.Length = 0 Then s = Alpha
        K = objVarArrs.BinarySearch(s, Stack(X_STACK_REG), Stack(Y_STACK_REG), I, J)
        Enter(J)
        Enter(I)
        Enter(K)
      ElseIf tok = "SEARCH" Then
        Enter(Search(Stack(X_STACK_REG), Fix(Stack(Y_STACK_REG)), Stack(Z_STACK_REG), Fix(Stack(T_STACK_REG))))
      ElseIf tok = "CSEARCH" Then
        Enter(CircularSearch(Stack(X_STACK_REG), Fix(Stack(Y_STACK_REG)), Stack(Z_STACK_REG), Fix(Stack(T_STACK_REG))))
      ElseIf tok = "MSEARCH" Then
        Enter(MiddleFirstSearch(Stack(X_STACK_REG), Stack(Y_STACK_REG), Fix(Stack(Z_STACK_REG))))
      ElseIf tok = "HSEARCH" Then
        I = HeuristicSearch(Stack(X_STACK_REG), I, Fix(Stack(Y_STACK_REG)), Fix(Stack(Z_STACK_REG)), Stack(T_STACK_REG))
        Enter(J)
        Enter(I)
      End If
    ElseIf tok = "RESIZE" Then
      s = tok.Substring(5)
      If s.Length = 0 Then s = Alpha
      objVarArrs.Resize(s, Fix(Stack(X_STACK_REG)))
    ElseIf tok = "NOSTACK" Then
      bShowStack = False
    ElseIf tok.StartsWith("FRMT") Then
      s = tok.Substring(4).ToUpper.Trim
      If s.Length = 0 Then s = DisplayFormat
      Try
        Alpha &= Output(Stack(X_STACK_REG), s)
      Catch
        Alpha &= Stack(X_STACK_REG)
        WriteLine("Formatting failed In FRMT command. Appended unformatted X register To ALpha register.")
      End Try
    ElseIf tok.StartsWith("PRT") Then
      If tok.StartsWith("PRTX") Then
        fxPrtStackReg(Stack(X_STACK_REG), tok, "X")
      ElseIf tok.StartsWith("PRTY") Then
        fxPrtStackReg(Stack(Y_STACK_REG), tok, "Y")
      ElseIf tok.StartsWith("PRTZ") Then
        fxPrtStackReg(Stack(Z_STACK_REG), tok, "Z")
      ElseIf tok.StartsWith("PRTT") Then
        fxPrtStackReg(Stack(T_STACK_REG), tok, "T")
      ElseIf tok.StartsWith("PRTSTK") Then
        If tok.Length = 6 Then
          fxPrtxStack(Alpha)
        Else
          fxPrtxStack(tok.Substring(6).Trim)
        End If
      End If
    ElseIf tok.StartsWith("EXIST") Then
      If tok = "EXISTARRNAME" Then
        I = Fix(Stack(X_STACK_REG))
        Flags(I) = objVarArrs.DoesArrNameExist(Alpha)
      ElseIf tok = "EXISTALPHAVAR" Then
        I = Fix(Stack(X_STACK_REG))
        Flags(I) = DoesAlphaVarExist(Alpha)
      Else
        sErrMessage = "Ivalid type Of EXIST command"
      End If
    ElseIf tok.IndexOf("$") >= 0 Then
      If tok = "LEFT$" Then
        If Alpha.Length > 0 Then Alpha = Alpha.Substring(0, Fix(Abs(Stack(X_STACK_REG))))
      ElseIf tok = "RIGHT$" Then
        I = Fix(Abs(Stack(X_STACK_REG)))
        If Alpha.Length > 0 Then Alpha = Alpha.Substring(Alpha.Length - I, I)
      ElseIf tok = "MID$" Then
        If Alpha.Length > 0 Then Alpha = Alpha.Substring(Fix(Abs(Stack(Y_STACK_REG))) - 1, Fix(Abs(Stack(X_STACK_REG))))
      ElseIf tok = "TRIM$" Then
        If Alpha.Length > 0 Then Alpha = Alpha.Trim
      ElseIf tok = "UCASE$" Then
        If Alpha.Length > 0 Then Alpha = Alpha.ToUpper
      ElseIf tok = "LCASE$" Then
        If Alpha.Length > 0 Then Alpha = Alpha.ToLower
      ElseIf tok = "SWAPCASE$" Then
        If Alpha.Length > 0 Then Alpha = SwapCase(Alpha)
      ElseIf tok = "REV$" Then
        If Alpha.Length > 0 Then Alpha = StrReverse(Alpha)
      ElseIf tok.StartsWith("POS$") Then
        Try
          s = tok.Substring(4).Trim
          s2 = GetAlphaVarValue(s)
          If s2.Length > 0 AndAlso Alpha.Length > 0 Then
            Enter(Alpha.IndexOf(s2))
          Else
            Enter(-1)
          End If
        Catch
          sErrMessage = "Error in command POS"
        End Try
      ElseIf tok.StartsWith("LASTPOS$") Then
        Try
          s = tok.Substring("LASTPOS$".Length + 1).Trim
          s2 = GetAlphaVarValue(s)
          If s2.Length > 0 AndAlso Alpha.Length > 0 Then
            Enter(Alpha.LastIndexOf(s2))
          Else
            Enter(-1)
          End If
        Catch
          sErrMessage = "Error in command LASTPOS"
        End Try
      ElseIf tok.StartsWith("REPLACE$") Then
        BackupStack()
        Try
          s = tok.Substring("REPLACE$".Length).Trim
          StrArr = s.Split(",")
          s2 = GetAlphaVarValue(StrArr(0))
          s3 = GetAlphaVarValue(StrArr(1))
          If s2.Length > 0 Then
            If Alpha.Length > 0 Then Alpha = Alpha.Replace(s2, s3)
          Else
            sErrMessage = "First argument in command REPLACE cannot be an empty string"
          End If
        Catch
          sErrMessage = "Error in command REPLACE"
        End Try
      ElseIf tok.StartsWith("FILTER$") Then
        s = tok.Substring("FILTER$".Length).Trim
        s = GetAlphaVarValue(s)
        If s.Length > 0 Then
          Alpha = StrFilter(Alpha, s)
        Else
          sErrMessage = "Missing argument in command FILTER"
        End If
      End If
    ElseIf tok = "DSP?" Then
      s = GetFormat()
      If s.Length > 0 Then
        WriteLine("Current display format is {0}", GetFormat())
        Alpha = DisplayFormat
      Else
        WriteLine("The current display format is not set.")
      End If
    ElseIf tok.StartsWith("DSTO") Then
      s = tok.Substring(4).Trim
      StoreDate(s, Stack(Y_STACK_REG), Stack(X_STACK_REG))
    ElseIf tok.StartsWith("DRCL") Then
      s = tok.Substring(4).Trim
      RecallDate(s, Y, X)
      Enter(Y)
      Enter(X)
    ElseIf tok.StartsWith("ADDDT") Then
      s = tok.Substring(5).Trim
      AddDt(s, Fix(Stack(X_STACK_REG)), Alpha)
    ElseIf tok.StartsWith("DIFFDT") Then
      s = tok.Substring(6).Trim
      StrArr = s.Split(" ")
      Try
        Enter(DiffDt(StrArr(0), StrArr(1), Alpha))
      Catch
        sErrMessage = "Missing variables in command DIFFDT"
      End Try
    Else
      sErrMessage = "Undefined command line '" & tok & "'"
    End If
    If bUpdatePrgmPtr Then PrgmPtr += 1
  End Sub

  Private Sub ReadData2()
    Dim I, J, Count As Integer
    Dim TextLine As String
    Dim s() As String

    If My.Computer.FileSystem.FileExists(Alpha) Then
      Dim fileReader As System.IO.StreamReader
      fileReader = New StreamReader(Alpha)
      I = Fix(Stack(X_STACK_REG)) ' get the index of the data insertion point
      J = I
      Count = 0
      I = I + 2 ' adjust index for first storage location
      Do Until fileReader.EndOfStream
        TextLine = fileReader.ReadLine()
        s = TextLine.Split(",")
        If IsIndexValid(I) And IsIndexValid(I + 1) Then
          SetMem(I, ParseDbl(s(0))) ' store X(I)
          SetMem(I + 1, ParseDbl(s(1))) ' store Y(I)
          I += 2
          Count += 1
        Else
          WriteLine("Extra input data in ReadData2 are ignored.")
          Exit Do
        End If
      Loop
      SetMem(J, 2) ' store the number of variables of data block
      SetMem(J + 1, Count) ' store the number of data points
      fileReader.Close()
      Enter(Count)
    Else
      sErrMessage = "Cannot open file " & Alpha & " in READDATA2 command"
    End If
  End Sub

  Private Sub NormalizeData2(ByVal StartIdx As Integer, ByVal Optional bLogX As Boolean = False, ByVal Optional bLogY As Boolean = False)
    Const NUM_VARS As Integer = 2
    Dim I, J, K, NumRows As Integer
    Dim VarMin(NUM_VARS) As Double, VarMax(NUM_VARS) As Double, VarDiff(NUM_VARS) As Double
    Dim Mat(,) As Double
    Dim bXhasZeroOrNegVals As Boolean = False
    Dim bYhasZeroOrNegVals As Boolean = False
    Dim bAbort As Boolean = False
    Dim Result As Integer = 2

    NumRows = GetMem(StartIdx)
    ReDim Mat(NumRows, NUM_VARS)
    K = StartIdx
    For I = 1 To NumRows
      For J = 1 To NUM_VARS
        K += 1
        Mat(I, J) = GetMem(K)
      Next
      If Mat(I, 1) < 0 Then bXhasZeroOrNegVals = True
      If Mat(I, 2) < 0 Then bYhasZeroOrNegVals = True
    Next

    ' transform x values into ln(x) if x has no zeros or negative values
    If bLogX And (Not bXhasZeroOrNegVals) Then
      For I = 1 To NumRows
        Mat(I, 1) = Log(Mat(I, 1))
      Next
    ElseIf bLogX And bXhasZeroOrNegVals Then
      WriteLine("Cannot transform X data into logarithm values!")
      Result = 1
    End If

    ' transform y values into ln(y) if y has no zeros or negative values
    If bLogY And (Not bYhasZeroOrNegVals) Then
      For I = 1 To NumRows
        Mat(I, 2) = Log(Mat(I, 2))
      Next
    ElseIf bLogY And bYhasZeroOrNegVals Then
      WriteLine("Cannot transform Y data into logarithm values!")
      Result = 1
    End If

    For I = 1 To NUM_VARS
      VarMin(I) = Double.MaxValue
      VarMax(I) = Double.MinValue
    Next

    For I = 1 To NumRows
      For J = 1 To NUM_VARS
        If Mat(I, J) < VarMin(J) Then VarMin(J) = Mat(I, J)
        If Mat(I, J) > VarMax(J) Then VarMax(J) = Mat(I, J)
      Next
    Next

    For J = 1 To NUM_VARS
      VarDiff(J) = VarMax(J) - VarMin(J)
      If VarDiff(J) = 0 Then
        Select Case J
          Case 1
            bAbort = True
            WriteLine("Variable X has the same value in each element.")
          Case 2
            bAbort = True
            WriteLine("Variable Y has the same value in each element.")
        End Select
      End If
    Next

    If bAbort Then
      sErrMessage = "Error in transforming data using command NORMDATA2"
      BackupStack()
      Enter(0)
      Exit Sub
    End If

    For I = 1 To NumRows
      For J = 1 To NUM_VARS
        Mat(I, J) = 1 + (Mat(I, J) - VarMin(J)) / VarDiff(J)
      Next
    Next

    ' write data back into the memory registers
    K = StartIdx
    For I = 1 To NumRows
      For J = 1 To NUM_VARS
        K += 1
        SetMem(K, Mat(I, J))
      Next
    Next
    BackupStack()
    Enter(Result)
  End Sub

  Private Sub ReadData3()
    Dim I, J, Count As Integer
    Dim TextLine As String
    Dim s() As String

    If My.Computer.FileSystem.FileExists(Alpha) Then
      Dim fileReader As System.IO.StreamReader
      fileReader = New StreamReader(Alpha)
      I = Fix(Stack(X_STACK_REG)) ' get the index of the data insertion point
      J = I
      Count = 0
      I = I + 2 ' adjust index for first storage location
      Do Until fileReader.EndOfStream
        TextLine = fileReader.ReadLine()
        s = TextLine.Split(",")
        If IsIndexValid(I) AndAlso IsIndexValid(I + 1) AndAlso IsIndexValid(I + 2) Then
          SetMem(I, ParseDbl(s(0))) ' store X(I)
          SetMem(I + 1, ParseDbl(s(1))) ' store Y(I)
          SetMem(I + 2, ParseDbl(s(2))) ' store Z(I)
          I += 3
          Count += 1
        Else
          WriteLine("Extra input data in ReadData3 are ignored.")
          Exit Do
        End If
      Loop
      SetMem(J, 3) ' store the number of variables of data block
      SetMem(J + 1, Count) ' store the number of data points
      fileReader.Close()
      Enter(Count)
    Else
      sErrMessage = "Cannot open file " & Alpha & " in READDATA3 command"
    End If
  End Sub

  Private Sub NormalizeData3(ByVal StartIdx As Integer, ByVal Optional bLogX As Boolean = False,
                             ByVal Optional bLogY As Boolean = False, ByVal Optional bLogZ As Boolean = False)
    Const NUM_VARS As Integer = 3
    Dim I, J, K, NumRows As Integer
    Dim VarMin(NUM_VARS) As Double, VarMax(NUM_VARS) As Double, VarDiff(NUM_VARS) As Double
    Dim Mat(,) As Double
    Dim bXhasZeroOrNegVals As Boolean = False
    Dim bYhasZeroOrNegVals As Boolean = False
    Dim bZhasZeroOrNegVals As Boolean = False
    Dim bAbort As Boolean = False
    Dim Result As Integer = 2

    NumRows = GetMem(StartIdx)
    ReDim Mat(NumRows, NUM_VARS)
    K = StartIdx
    For I = 1 To NumRows
      For J = 1 To NUM_VARS
        K += 1
        Mat(I, J) = GetMem(K)
      Next
      If Mat(I, 1) < 0 Then bXhasZeroOrNegVals = True
      If Mat(I, 2) < 0 Then bYhasZeroOrNegVals = True
      If Mat(I, 3) < 0 Then bZhasZeroOrNegVals = True
    Next

    ' transform x values into ln(x) if x has no zeros or negative values
    If bLogX And (Not bXhasZeroOrNegVals) Then
      For I = 1 To NumRows
        Mat(I, 1) = Log(Mat(I, 1))
      Next
    ElseIf bLogX And bXhasZeroOrNegVals Then
      WriteLine("Cannot transform X data into logarithm values!")
      Result = 1
    End If

    ' transform y values into ln(y) if y has no zeros or negative values
    If bLogY And (Not bYhasZeroOrNegVals) Then
      For I = 1 To NumRows
        Mat(I, 2) = Log(Mat(I, 2))
      Next
    ElseIf bLogY And bYhasZeroOrNegVals Then
      WriteLine("Cannot transform Y data into logarithm values!")
      Result = 1
    End If

    ' transform z values into ln(z) if z has no zeros or negative values
    If bLogZ And (Not bZhasZeroOrNegVals) Then
      For I = 1 To NumRows
        Mat(I, 3) = Log(Mat(I, 3))
      Next
    ElseIf bLogZ And bZhasZeroOrNegVals Then
      WriteLine("Cannot transform Z data into logarithm values!")
      Result = 1
    End If

    For I = 1 To NUM_VARS
      VarMin(I) = Double.MaxValue
      VarMax(I) = Double.MinValue
    Next

    For I = 1 To NumRows
      For J = 1 To NUM_VARS
        If Mat(I, J) < VarMin(J) Then VarMin(J) = Mat(I, J)
        If Mat(I, J) > VarMax(J) Then VarMax(J) = Mat(I, J)
      Next
    Next

    For J = 1 To NUM_VARS
      VarDiff(J) = VarMax(J) - VarMin(J)
      If VarDiff(J) = 0 Then
        Select Case J
          Case 1
            bAbort = True
            WriteLine("Variable X has the same value in each element.")
          Case 2
            bAbort = True
            WriteLine("Variable Y has the same value in each element.")
          Case 3
            bAbort = True
            WriteLine("Variable Z has the same value in each element.")
        End Select
      End If
    Next

    If bAbort Then
      sErrMessage = "Error in transforming data using command NORMDATA3"
      BackupStack()
      Enter(0)
      Exit Sub
    End If

    For I = 1 To NumRows
      For J = 1 To NUM_VARS
        Mat(I, J) = 1 + (Mat(I, J) - VarMin(J)) / VarDiff(J)
      Next
    Next

    ' write data back into the memory registers
    K = StartIdx
    For I = 1 To NumRows
      For J = 1 To NUM_VARS
        K += 1
        SetMem(K, Mat(I, J))
      Next
    Next

    BackupStack()
    Enter(Result)
  End Sub

  Private Sub ExecLR()
    ' Data members for linear(ized) regression
    Dim X, Y As Double
    Dim SumX, SumX2, SumY, SumY2, SumXY As Double
    Dim Sxx, Syy, Sxy As Double
    Dim MeanX, MeanY, MeanX2, MeanY2 As Double
    Dim Slope, Intercept, R2 As Double
    Dim Idx, N, I, J As Integer
    Dim Xpower As Double, Ypower As Double

    Ypower = Stack(Y_STACK_REG)
    Xpower = Stack(X_STACK_REG)
    I = Fix(Stack(Z_STACK_REG)) ' get the index where the stat data block starts
    J = Fix(GetMem(I)) ' get the content of Mem(I) which selects the regression type
    If J <> 2 Then
      sErrMessage = "Invalid data block for linearized regression"
      Exit Sub
    End If
    Idx = I + 2
    N = Fix(GetMem(I + 1))

    SumX = 0
    SumX2 = 0
    SumY = 0
    SumY2 = 0
    SumXY = 0

    Try
      For I = 1 To N
        X = GetMem(Idx)
        Y = GetMem(Idx + 1)
        Idx += 2
        If Xpower = 1 Then
          ' do nothing
        ElseIf Xpower <> 0 Then
          X = Pow(X, Xpower)
        Else
          X = Log(X)
        End If

        If Ypower = 1 Then
          ' do nothing
        ElseIf Ypower <> 0 Then
          Y = Pow(Y, Ypower)
        Else
          Y = Log(Y)
        End If

        SumX += X
        SumX2 += X * X
        SumY += Y
        SumY2 += Y * Y
        SumXY += X * Y
      Next

      Sxx = SumX2 - SumX * SumX / N
      Syy = SumY2 - SumY * SumY / N
      Sxy = SumXY - SumX * SumY / N
      MeanX = SumX / N
      MeanX2 = SumX2 / N
      MeanY = SumY / N
      MeanY2 = SumY2 / 2
      Slope = Sxy / Sxx
      Intercept = (SumY - Slope * SumX) / N
      R2 = Sxy * Sxy / Sxx / Syy

      BackupStack()
      Stack(T_STACK_REG) = Stack(X_STACK_REG)
      Stack(Z_STACK_REG) = R2
      Stack(Y_STACK_REG) = Intercept
      Stack(X_STACK_REG) = Slope
    Catch
      sErrMessage = "Math error in subroutine ExecLR"
    End Try
  End Sub

  Private Sub ExecBestLR()
    ' Data members for linear(ized) regression
    Dim X, Y As Double
    Dim SumX, SumX2, SumY, SumY2, SumXY As Double
    Dim Sxx, Syy, Sxy As Double
    Dim MeanX, MeanY, MeanX2, MeanY2 As Double
    Dim Slope, Intercept, R2, BestSlope, BestIntercept As Double
    Dim Idx, Idx0, N, I, J As Integer
    Dim Xpower As Double, Ypower As Double
    Dim BestR2 As Double = -1
    Dim Xpowers() As Double = {-4, -3.5, -3, -2.5, -2, -1.5, -1, -0.5, 0, 0.5, 1, 1.5, 2, 2.5, 3, 3.5, 4}
    Dim Ypowers() As Double = {-4, -3.5, -3, -2.5, -2, -1.5, -1, -0.5, 0, 0.5, 1, 1.5, 2, 2.5, 3, 3.5, 4}
    Dim sModel, sFilename As String
    Dim fileWriter As System.IO.StreamWriter

    sFilename = GetNextStreamFile("BestLR")
    fileWriter = New StreamWriter(sFilename)

    I = Fix(Stack(X_STACK_REG)) ' get the index where the stat data block starts
    J = Fix(GetMem(I)) ' get the content of Mem(I) which selects the regression type
    If J <> 2 Then
      sErrMessage = "Invalid data block for linearized regression"
      Exit Sub
    End If
    Idx0 = I + 2
    N = Fix(GetMem(I + 1))
    ' Write columns header
    fileWriter.WriteLine("R2,Ypower,Xpower,Intercept,Slope")
    For Each Ypower In Ypowers
      For Each Xpower In Xpowers

        SumX = 0
        SumX2 = 0
        SumY = 0
        SumY2 = 0
        SumXY = 0
        Idx = Idx0

        Try
          For I = 1 To N
            X = GetMem(Idx)
            Y = GetMem(Idx + 1)
            Idx += 2
            If Xpower = 1 Then
              ' do nothing
            ElseIf Xpower <> 0 Then
              X = Pow(X, Xpower)
            Else
              X = Log(X)
            End If

            If Ypower = 1 Then
              ' do nothing
            ElseIf Ypower <> 0 Then
              Y = Pow(Y, Ypower)
            Else
              Y = Log(Y)
            End If

            SumX += X
            SumX2 += X * X
            SumY += Y
            SumY2 += Y * Y
            SumXY += X * Y
          Next

          Sxx = SumX2 - SumX * SumX / N
          Syy = SumY2 - SumY * SumY / N
          Sxy = SumXY - SumX * SumY / N
          MeanX = SumX / N
          MeanX2 = SumX2 / N
          MeanY = SumY / N
          MeanY2 = SumY2 / 2
          Slope = Sxy / Sxx
          Intercept = (SumY - Slope * SumX) / N
          R2 = Sxy * Sxy / Sxx / Syy
          ' write current results to output file
          fileWriter.WriteLine(CStr(R2) & "," & CStr(Ypower) & "," & CStr(Xpower) & "," & CStr(Intercept) & "," & CStr(Slope))


          If R2 > BestR2 Then
            BestR2 = R2
            BestXPower = Xpower
            BestYPower = Ypower
            BestSlope = Slope
            BestIntercept = Intercept
          End If
        Catch
          Exit For
        End Try
      Next
    Next
    fileWriter.Close()
    WriteLine("Data for best linearized regression models was written to file " & sFilename)

    ' build model as a string
    If BestYPower = 1 Then
      sModel = "Y = a + c * "
    ElseIf BestYPower = 0
      sModel = "ln(Y) = a + c * "
    ElseIf BestYPower > 0 Then
      sModel = "Y^" & BestYPower & " = a + c * "
    Else
      sModel = "1/Y^" & Abs(BestYPower) & " = a + c * "
    End If

    If BestXPower = 1 Then
      sModel &= "X"
    ElseIf BestXPower = 0
      sModel &= "ln(X)"
    ElseIf BestXPower > 0 Then
      sModel &= "X^" & BestXPower
    Else
      sModel = "1/X^" & Abs(BestXPower)
    End If

    BackupStack()
    Alpha = sModel
    Stack(T_STACK_REG) = Stack(X_STACK_REG)
    Stack(Z_STACK_REG) = BestR2
    Stack(Y_STACK_REG) = BestIntercept
    Stack(X_STACK_REG) = BestSlope

  End Sub

  Private Sub ExecPowerLR()
    ' Perform a power fit
    Dim X, Y As Double
    Dim SumX, SumX2, SumY, SumY2, SumXY As Double
    Dim Sxx, Syy, Sxy As Double
    Dim MeanX, MeanY, MeanX2, MeanY2 As Double
    Dim Slope, Intercept, R2 As Double
    Dim I, J, Idx, N As Integer

    I = Fix(Stack(X_STACK_REG)) ' get the index where the stat data block starts
    J = GetMem(I) ' get the content of Mem(I) which selects the regression type
    If J <> 2 Then
      sErrMessage = "Wrong data block type in POWERLR command"
      Exit Sub
    End If
    N = Fix(GetMem(I + 1))
    Idx = I + 2

    SumX = 0
    SumX2 = 0
    SumY = 0
    SumY2 = 0
    SumXY = 0

    Try
      For I = 1 To N
        X = Log(GetMem(Idx))
        Y = Log(GetMem(Idx + 1))
        Idx += 2

        SumX += X
        SumX2 += X * X
        SumY += Y
        SumY2 += Y * Y
        SumXY += X * Y
      Next

      Sxx = SumX2 - SumX * SumX / N
      Syy = SumY2 - SumY * SumY / N
      Sxy = SumXY - SumX * SumY / N
      MeanX = SumX / N
      MeanX2 = SumX2 / N
      MeanY = SumY / N
      MeanY2 = SumY2 / 2
      Slope = Sxy / Sxx
      Intercept = (SumY - Slope * SumX) / N
      R2 = Sxy * Sxy / Sxx / Syy

      BackupStack()
      Stack(T_STACK_REG) = Stack(X_STACK_REG)
      Stack(Z_STACK_REG) = R2
      Stack(Y_STACK_REG) = Intercept
      Stack(X_STACK_REG) = Slope
    Catch
      sErrMessage = "Math error in POWERLR command"
    End Try
  End Sub

  Private Sub ExecLogxLR()
    ' Perform a Y = a + b*ln(X) fit
    Dim X, Y As Double
    Dim SumX, SumX2, SumY, SumY2, SumXY As Double
    Dim Sxx, Syy, Sxy As Double
    Dim MeanX, MeanY, MeanX2, MeanY2 As Double
    Dim Slope, Intercept, R2 As Double
    Dim I, J, Idx, N As Integer

    I = Fix(Stack(X_STACK_REG)) ' get the index where the stat data block starts
    J = GetMem(I) ' get the content of Mem(I) which selects the regression type
    If J <> 2 Then
      sErrMessage = "Wrong data block type in LOGXLR command"
      Exit Sub
    End If
    N = Fix(GetMem(I + 1))
    Idx = I + 2

    SumX = 0
    SumX2 = 0
    SumY = 0
    SumY2 = 0
    SumXY = 0

    Try
      For I = 1 To N
        X = Log(GetMem(Idx))
        Y = GetMem(Idx + 1)
        Idx += 2

        SumX += X
        SumX2 += X * X
        SumY += Y
        SumY2 += Y * Y
        SumXY += X * Y
      Next

      Sxx = SumX2 - SumX * SumX / N
      Syy = SumY2 - SumY * SumY / N
      Sxy = SumXY - SumX * SumY / N
      MeanX = SumX / N
      MeanX2 = SumX2 / N
      MeanY = SumY / N
      MeanY2 = SumY2 / 2
      Slope = Sxy / Sxx
      Intercept = (SumY - Slope * SumX) / N
      R2 = Sxy * Sxy / Sxx / Syy

      BackupStack()
      Stack(T_STACK_REG) = Stack(X_STACK_REG)
      Stack(Z_STACK_REG) = R2
      Stack(Y_STACK_REG) = Intercept
      Stack(X_STACK_REG) = Slope
    Catch
      sErrMessage = "Math error in LOGXLR command"
    End Try
  End Sub

  Private Sub ExecLogyLR()
    ' Perform a ln(Y) = a + b*X fit
    Dim X, Y As Double
    Dim SumX, SumX2, SumY, SumY2, SumXY As Double
    Dim Sxx, Syy, Sxy As Double
    Dim MeanX, MeanY, MeanX2, MeanY2 As Double
    Dim Slope, Intercept, R2 As Double
    Dim I, J, Idx, N As Integer

    I = Fix(Stack(X_STACK_REG)) ' get the index where the stat data block starts
    J = GetMem(I) ' get the content of Mem(I) which selects the regression type
    If J <> 2 Then
      sErrMessage = "Wrong data block type in LOGYLR command"
      Exit Sub
    End If
    N = Fix(GetMem(I + 1))
    Idx = I + 2

    SumX = 0
    SumX2 = 0
    SumY = 0
    SumY2 = 0
    SumXY = 0

    Try
      For I = 1 To N
        X = GetMem(Idx)
        Y = Log(GetMem(Idx + 1))
        Idx += 2

        SumX += X
        SumX2 += X * X
        SumY += Y
        SumY2 += Y * Y
        SumXY += X * Y
      Next

      Sxx = SumX2 - SumX * SumX / N
      Syy = SumY2 - SumY * SumY / N
      Sxy = SumXY - SumX * SumY / N
      MeanX = SumX / N
      MeanX2 = SumX2 / N
      MeanY = SumY / N
      MeanY2 = SumY2 / 2
      Slope = Sxy / Sxx
      Intercept = (SumY - Slope * SumX) / N
      R2 = Sxy * Sxy / Sxx / Syy

      BackupStack()
      Stack(T_STACK_REG) = Stack(X_STACK_REG)
      Stack(Z_STACK_REG) = R2
      Stack(Y_STACK_REG) = Intercept
      Stack(X_STACK_REG) = Slope
    Catch
      sErrMessage = "Math error in the command LOGYLR"
    End Try
  End Sub

  Private Sub ExecMLR()
    ' Data members for linear(ized) regression
    Dim X, Y, Z As Double
    Dim A, B, C As Double
    Dim SumX, SumX2, SumY, SumY2, SumXY As Double
    Dim SumZ, SumZ2, SumXZ, SumYZ As Double
    Dim Sxx, Syy, Szz, Sxy, Syz, Sxz As Double
    Dim SlopeX, SlopeY, Intercept, R2 As Double
    Dim Idx, N, I, J As Integer
    Dim Xpower, Ypower, Zpower As Double

    Zpower = Stack(Z_STACK_REG)
    Ypower = Stack(Y_STACK_REG)
    Xpower = Stack(X_STACK_REG)
    I = Fix(Stack(T_STACK_REG)) ' get the index where the stat data block starts
    J = Fix(GetMem(I)) ' get the content of Mem(I) which selects the regression type
    If J <> 3 Then
      sErrMessage = "Invalid data block for linearized regression in MLR command"
      Exit Sub
    End If
    Idx = I + 2
    N = Fix(GetMem(I + 1))

    SumY2 = 0
    SumXY = 0
    SumZ = 0
    SumZ2 = 0
    SumXZ = 0
    SumYZ = 0

    Try
      For I = 1 To N
        X = GetMem(Idx)
        Y = GetMem(Idx + 1)
        Z = GetMem(Idx + 2)
        Idx += 3
        If Xpower = 1 Then
          ' do nothing
        ElseIf Xpower <> 0 Then
          X = Pow(X, Xpower)
        Else
          X = Log(X)
        End If

        If Ypower = 1 Then
          ' do nothing
        ElseIf Ypower <> 0 Then
          Y = Pow(Y, Ypower)
        Else
          Y = Log(Y)
        End If
        If Zpower = 1 Then
          ' do nothing
        ElseIf Zpower <> 0 Then
          Z = Pow(Z, Zpower)
        Else
          Z = Log(Z)
        End If

        SumX += X
        SumX2 += X * X
        SumY += Y
        SumY2 += Y * Y
        SumXY += X * Y
        SumZ += Z
        SumZ2 += Z * Z
        SumXZ += X * Z
        SumYZ += Y * Z
      Next

      Sxx = N * SumX2 - SumX * SumX
      Syy = N * SumY2 - SumY * SumY
      Szz = N * SumZ2 - SumZ * SumZ
      Sxy = N * SumXY - SumX * SumY
      Sxz = N * SumXZ - SumX * SumZ
      Syz = N * SumYZ - SumY * SumZ

      A = Sxx * Syz
      B = Sxy * Sxz
      SlopeY = (A - B) / (Sxx * Syy - Sqr(Sxy))
      SlopeX = (Sxz - SlopeY * Sxy) / Sxx
      Intercept = (SumZ - SlopeX * SumX - SlopeY * SumY) / N
      C = Sqr(SumZ) / N
      R2 = (Intercept * SumZ + SlopeX * SumXZ + SlopeY * SumYZ - C) / (SumZ2 - C)

      BackupStack()
      Stack(T_STACK_REG) = R2
      Stack(Z_STACK_REG) = Intercept
      Stack(Y_STACK_REG) = SlopeY
      Stack(X_STACK_REG) = SlopeX
    Catch
      sErrMessage = "Math error in subroutine ExecMLR"
    End Try
  End Sub

  Private Function GetNextStreamFile(ByVal sFilename As String) As String
    Dim sStreamName As String
    Dim iMonth, iYear, iDay, iHour, iMinute, iSecond As Integer
    Dim dt As Date

    dt = Now()
    iYear = Year(dt)
    iMonth = Month(dt)
    iDay = Day(dt)
    iHour = Hour(dt)
    iMinute = Minute(dt)
    iSecond = Second(dt)

    sStreamName = sFilename & "_" & CStr(iYear) & "." &
                                    iMonth.ToString("D2") & "." &
                                    iDay.ToString("D2") & "_" &
                                    iHour.ToString("D2") & "." &
                                    iMonth.ToString("D2") & "." &
                                    iSecond.ToString("D2") & ".CSV"
    Return sStreamName
  End Function

  Private Sub ExecBestMLR()
    ' Data members for linear(ized) regression
    Dim X, Y, Z As Double
    Dim A, B, C As Double
    Dim SumX, SumX2, SumY, SumY2, SumXY As Double
    Dim SumZ, SumZ2, SumXZ, SumYZ As Double
    Dim Sxx, Syy, Szz, Sxy, Syz, Sxz As Double
    Dim SlopeX, SlopeY, Intercept, R2 As Double
    Dim BestSlopeX, BestSlopeY, BestIntercept, BestR2 As Double
    Dim Idx0, Idx, N, I, J As Integer
    Dim Xpower, Ypower, Zpower As Double
    Dim Xpowers() As Double = {-4, -3.5, -3, -2.5, -2, -1.5, -1, -0.5, 0, 0.5, 1, 1.5, 2, 2.5, 3, 3.5, 4}
    Dim Ypowers() As Double = {-4, -3.5, -3, -2.5, -2, -1.5, -1, -0.5, 0, 0.5, 1, 1.5, 2, 2.5, 3, 3.5, 4}
    Dim Zpowers() As Double = {-4, -3.5, -3, -2.5, -2, -1.5, -1, -0.5, 0, 0.5, 1, 1.5, 2, 2.5, 3, 3.5, 4}
    Dim sModel, sFilename As String
    Dim fileWriter As System.IO.StreamWriter

    sFilename = GetNextStreamFile("BestMLR")
    fileWriter = New StreamWriter(sFilename)

    BestR2 = -1
    I = Fix(Stack(X_STACK_REG)) ' get the index where the stat data block starts
    J = Fix(GetMem(I)) ' get the content of Mem(I) which selects the regression type
    If J <> 3 Then
      sErrMessage = "Invalid data block for linearized regression in MLR command"
      Exit Sub
    End If
    Idx0 = I + 2
    N = Fix(GetMem(I + 1))
    ' Write columns header
    fileWriter.WriteLine("R2,Zpower,Ypower,Xpower,Intercept,SlopeY,SlopeX")
    For Each Zpower In Zpowers
      For Each Ypower In Ypowers
        For Each Xpower In Xpowers

          SumX = 0
          SumX2 = 0
          SumY = 0
          SumY2 = 0
          SumXY = 0
          SumZ = 0
          SumZ2 = 0
          SumXZ = 0
          SumYZ = 0
          Idx = Idx0

          Try
            For I = 1 To N
              X = GetMem(Idx)
              Y = GetMem(Idx + 1)
              Z = GetMem(Idx + 2)
              Idx += 3
              If Xpower = 1 Then
                ' do nothing
              ElseIf Xpower <> 0 Then
                X = Pow(X, Xpower)
              Else
                X = Log(X)
              End If

              If Ypower = 1 Then
                ' do nothing
              ElseIf Ypower <> 0 Then
                Y = Pow(Y, Ypower)
              Else
                Y = Log(Y)
              End If
              If Zpower = 1 Then
                ' do nothing
              ElseIf Zpower <> 0 Then
                Z = Pow(Z, Zpower)
              Else
                Z = Log(Z)
              End If

              SumX += X
              SumX2 += X * X
              SumY += Y
              SumY2 += Y * Y
              SumXY += X * Y
              SumZ += Z
              SumZ2 += Z * Z
              SumXZ += X * Z
              SumYZ += Y * Z
            Next

            Sxx = N * SumX2 - SumX * SumX
            Syy = N * SumY2 - SumY * SumY
            Szz = N * SumZ2 - SumZ * SumZ
            Sxy = N * SumXY - SumX * SumY
            Sxz = N * SumXZ - SumX * SumZ
            Syz = N * SumYZ - SumY * SumZ

            A = Sxx * Syz
            B = Sxy * Sxz
            SlopeY = (A - B) / (Sxx * Syy - Sqr(Sxy))
            SlopeX = (Sxz - SlopeY * Sxy) / Sxx
            Intercept = (SumZ - SlopeX * SumX - SlopeY * SumY) / N
            C = Sqr(SumZ) / N
            R2 = (Intercept * SumZ + SlopeX * SumXZ + SlopeY * SumYZ - C) / (SumZ2 - C)
            ' write current results to output file
            fileWriter.WriteLine(CStr(R2) & "," & CStr(Xpower) & "," & CStr(Ypower) & "," & CStr(Xpower) &
                                 "," & CStr(Intercept) & "," & CStr(SlopeY) & "," & CStr(SlopeX))

            If R2 > BestR2 Then
              BestR2 = R2
              BestXPower = Xpower
              BestYPower = Ypower
              BestZPower = Zpower
              BestSlopeX = SlopeX
              BestSlopeY = SlopeY
              BestIntercept = Intercept
            End If
          Catch
            Exit For
          End Try
        Next
      Next
    Next
    fileWriter.Close()
    WriteLine("Data for best multi-linearized regression models was written to file " & sFilename)
    ' build model as a string
    If BestZPower = 1 Then
      sModel = "Z = a + b * "
    ElseIf BestZPower = 0
      sModel = "ln(Z) = a + b * "
    ElseIf BestZPower > 0 Then
      sModel = "Z^" & BestZPower & " = a + b * "
    Else
      sModel = "1/Z^" & Abs(BestZPower) & " = a + b * "
    End If

    If BestXPower = 1 Then
      sModel &= "X + c * "
    ElseIf BestYPower = 0
      sModel &= "ln(X) + c *"
    ElseIf BestXPower > 0 Then
      sModel &= "X^" & BestXPower & " + c * "
    Else
      sModel = "1/X^" & Abs(BestXPower) & " + c * "
    End If

    If BestYPower = 1 Then
      sModel &= "Y"
    ElseIf BestYPower = 0
      sModel &= "ln(Y)"
    ElseIf BestYPower > 0 Then
      sModel &= "Y^" & BestYPower
    Else
      sModel = "1/Y^" & Abs(BestYPower)
    End If

    BackupStack()
    Alpha = sModel
    Stack(T_STACK_REG) = BestR2
    Stack(Z_STACK_REG) = BestIntercept
    Stack(Y_STACK_REG) = BestSlopeY
    Stack(X_STACK_REG) = BestSlopeX

  End Sub

  Private Sub ExecPowerMLR()
    ' Data members for linear(ized) regression
    Dim X, Y, Z As Double
    Dim A, B, C As Double
    Dim SumX, SumX2, SumY, SumY2, SumXY As Double
    Dim SumZ, SumZ2, SumXZ, SumYZ As Double
    Dim Sxx, Syy, Szz, Sxy, Syz, Sxz As Double
    Dim SlopeX, SlopeY, Intercept, R2 As Double
    Dim I, J, Idx, N As Integer

    I = Fix(Stack(X_STACK_REG)) ' get the index where the stat data block starts
    J = GetMem(I) ' get the content of Mem(I) which selects the regression type
    If J <> 3 Then
      sErrMessage = "Wrong data block type in POWERMLR command"
      Exit Sub
    End If
    N = Fix(GetMem(I + 1))
    Idx = I + 2

    SumX = 0
    SumX2 = 0
    SumY = 0
    SumY2 = 0
    SumXY = 0
    SumZ = 0
    SumZ2 = 0
    SumXZ = 0
    SumYZ = 0

    Try
      For I = 1 To N
        X = Log(GetMem(Idx))
        Y = Log(GetMem(Idx + 1))
        Z = Log(GetMem(Idx + 2))
        Idx += 3

        SumX += X
        SumX2 += X * X
        SumY += Y
        SumY2 += Y * Y
        SumXY += X * Y
        SumZ += Z
        SumZ2 += Z * Z
        SumXZ += X * Z
        SumYZ += Y * Z
      Next

      Sxx = N * SumX2 - SumX * SumX
      Syy = N * SumY2 - SumY * SumY
      Szz = N * SumZ2 - SumZ * SumZ
      Sxy = N * SumXY - SumX * SumY
      Sxz = N * SumXZ - SumX * SumZ
      Syz = N * SumYZ - SumY * SumZ

      A = Sxx * Syz
      B = Sxy * Sxz
      SlopeY = (A - B) / (Sxx * Syy - Sqr(Sxy))
      SlopeX = (Sxz - SlopeY * Sxy) / Sxx
      Intercept = (SumZ - SlopeX * SumX - SlopeY * SumY) / N
      C = Sqr(SumZ) / N
      R2 = (Intercept * SumZ + SlopeX * SumXZ + SlopeY * SumYZ - C) / (SumZ2 - C)

      BackupStack()
      Stack(T_STACK_REG) = R2
      Stack(Z_STACK_REG) = Intercept
      Stack(Y_STACK_REG) = SlopeY
      Stack(X_STACK_REG) = SlopeX
    Catch
      sErrMessage = "Math error in POWERMLR command"
    End Try
  End Sub

  Private Sub ExecQuadFit()
    ' Data members for linear(ized) regression
    Dim X, Y, Z As Double
    Dim A, B, C As Double
    Dim SumX, SumX2, SumY, SumY2, SumXY As Double
    Dim SumZ, SumZ2, SumXZ, SumYZ As Double
    Dim Sxx, Syy, Szz, Sxy, Syz, Sxz As Double
    Dim SlopeX, SlopeY, Intercept, R2 As Double
    Dim I, J, Idx, N As Integer

    I = Fix(Stack(X_STACK_REG)) ' get the index where the stat data block starts
    J = GetMem(I) ' get the content of Mem(I) which selects the regression type
    If J <> 2 Then
      sErrMessage = "Wrong data block type in QUADFIT command"
      Exit Sub
    End If
    N = Fix(GetMem(I + 1))
    Idx = I + 2

    SumX = 0
    SumX2 = 0
    SumY = 0
    SumY2 = 0
    SumXY = 0
    SumZ = 0
    SumZ2 = 0
    SumXZ = 0
    SumYZ = 0

    Try
      For I = 1 To N
        X = GetMem(Idx)
        Y = X * X
        Z = GetMem(Idx + 1)
        Idx += 2

        SumX += X
        SumX2 += X * X
        SumY += Y
        SumY2 += Y * Y
        SumXY += X * Y
        SumZ += Z
        SumZ2 += Z * Z
        SumXZ += X * Z
        SumYZ += Y * Z
      Next

      Sxx = N * SumX2 - SumX * SumX
      Syy = N * SumY2 - SumY * SumY
      Szz = N * SumZ2 - SumZ * SumZ
      Sxy = N * SumXY - SumX * SumY
      Sxz = N * SumXZ - SumX * SumZ
      Syz = N * SumYZ - SumY * SumZ

      A = Sxx * Syz
      B = Sxy * Sxz
      SlopeY = (A - B) / (Sxx * Syy - Sqr(Sxy))
      SlopeX = (Sxz - SlopeY * Sxy) / Sxx
      Intercept = (SumZ - SlopeX * SumX - SlopeY * SumY) / N
      C = Sqr(SumZ) / N
      R2 = (Intercept * SumZ + SlopeX * SumXZ + SlopeY * SumYZ - C) / (SumZ2 - C)

      BackupStack()
      Stack(T_STACK_REG) = R2
      Stack(Z_STACK_REG) = Intercept
      Stack(Y_STACK_REG) = SlopeY
      Stack(X_STACK_REG) = SlopeX
    Catch
      sErrMessage = "Math error in QUADFIT command"
    End Try
  End Sub

  Private Sub ExecAlphaSub()
    Dim X As Double

    X = MyBase.Calc(Alpha)
    If GetErrMessage() = "" Then
      BackupStack()
      Enter(X)
    End If
  End Sub

  Private Overloads Function Calc(ByVal S As String) As Double
    Dim tokens() As String
    Dim I, ThisLbl As Integer

    S = S.ToUpper
    tokens = S.Split(" ")
    If tokens(0) = "GSB" Then
      I = FindLbl(tokens(1))
      If I > 0 Then
        ThisLbl = PrgmPtr ' resume execution at the GOSUB's label
        ExecProgram(sLbls(I))
        PrgmPtr = ThisLbl
        Return Stack(X_STACK_REG)
      Else
        sErrMessage = "Label " & tokens(1) & " in Alpha register was not found by method Calc()"
        Return 0
      End If
    Else
      Return MyBase.Calc(S)
    End If

  End Function

  Private Function LegendrePoly(ByVal X As Double, ByVal Order As Integer) As Double
    ' Implementation of nonrecursive relation for Legendre polynomials.
    Dim L0, L1, L2 As Double
    Dim I, K As Integer

    L0 = 1
    If Order = 0 Then Return L0

    L1 = X
    If Order = 1 Then Return L1

    K = 1
    For I = 2 To Order
      L2 = ((2 * K + 1) * X * L1 - K * L0) / (K + 1)
      K = K + 1
      L0 = L1
      L1 = L2
    Next I
    Return L2
  End Function

  Function LegendrePolyDeriv(ByVal X As Double, ByVal Order As Integer) As Double
    ' First derivative of Legendre polynomials.
    Dim h As Double

    h = 0.001 * (1 + Abs(X))
    Return (LegendrePoly(X + h, Order) - LegendrePoly(X - h, Order)) / 2 / h
  End Function


  Private Function NormalizeFloor(ByVal X As Double) As Double
    Dim bLTone As Boolean = X < 1

    X = Log(X) / Log(10)
    X = Fix(X)
    If bLTone Then X -= 1
    Return 10 ^ X
  End Function

  Private Sub fxGaussLegendreQuadrature()
    ' Implement Integrate GAUSSQUAD using Gauss-Legendre quadrature
    Const MAX_ITER = 1000
    Dim a, b, toler As Double
    Dim J, K, Order As Integer
    Dim MaxX, X As Double
    Dim Sum, DeltaX As Double
    Dim Xa, Fa, Xb, Fb, Fx As Double
    Dim Xrt, Wt, Diff As Double
    Dim N, Iter As Integer

    BackupStack()
    bShowStack = False
    ' get input from the stack
    toler = Stack(T_STACK_REG)
    Order = Fix(Stack(Z_STACK_REG))
    a = Stack(X_STACK_REG)
    b = Stack(Y_STACK_REG)
    DeltaX = NormalizeFloor((b - a) / 100 / Order)
    MaxX = 1000

    ' if a>b then swap a and b
    If a > b Then
      X = a
      a = b
      b = X
    End If

    Try
      N = Order
      Sum = 0
      Xb = 0
      Fb = LegendrePoly(Xb, Order)

      ' found root at x=0 (for odd polynomial orders)?
      If Abs(Fb) < 0.000001 Then
        N = N - 1
        Xrt = Xb
        Wt = 2 / (1 - Xrt * Xrt) / (LegendrePolyDeriv(Xrt, Order)) ^ 2
        X = (b - a) * Xrt / 2 + (a + b) / 2
        Stack(X_STACK_REG) = X
        Fx = Calc(Alpha)
        Sum += Wt * Fx
      End If

      ' find roots of the Legendre polynomial using the scan range method
      Do
        Xa = Xb
        Fa = Fb
        Xb = Xa + DeltaX
        Fb = LegendrePoly(Xb, Order)
        ' root in interval [Xa, Xb]?
        If Fa * Fb < 0 Then
          ' start Newton;s method
          Xrt = (Xa + Xb) / 2
          Iter = 0
          Do
            Iter += 1
            Diff = LegendrePoly(Xrt, Order) / LegendrePolyDeriv(Xrt, Order)
            Xrt -= Diff
          Loop Until Abs(Diff) <= toler Or Iter > MAX_ITER
          N -= 2 ' decrement 2 ... one for positive-value root and one for it's negative twin

          ' calculate weight for postive and negative roots
          Wt = 2 / (1 - Xrt * Xrt) / (LegendrePolyDeriv(Xrt, Order)) ^ 2
          ' calculate transformed x for positive root
          X = (b - a) * Xrt / 2 + (a + b) / 2
          Stack(X_STACK_REG) = X
          Fx = Calc(Alpha)
          ' update the value of the integral
          Sum += Wt * Fx
          ' calculate transformed x for negative root
          X = (a - b) * Xrt / 2 + (a + b) / 2
          Stack(X_STACK_REG) = X
          Fx = Calc(Alpha)
          ' update the value of the integral
          Sum += Wt * Fx
        End If
      Loop Until N = 0 Or Xa > MaxX

      ' found all roots?
      If N = 0 Then
        Stack(X_STACK_REG) = (b - a) / 2 * Sum
      Else
        WriteLine("Error in Gauss-Legendre quadrature calcations.")
        Stack(X_STACK_REG) = 0
      End If
    Catch
      sErrMessage = "Runtime math error in the GAUSSLEGQUAD command"
    End Try
    bShowStack = True
  End Sub

  Private Function LaguerrePoly(ByVal X As Double, ByVal Order As Integer) As Double
    ' Implementation of recursive relation for Laguerre polynomials.
    Dim L0, L1, L2 As Double
    Dim I, K As Integer

    L0 = 1
    If Order = 0 Then Return L0

    L1 = 1 - X
    If Order = 1 Then Return L1

    K = 1
    For I = 2 To Order
      L2 = ((2 * K + 1 - X) * L1 - K * L0) / (K + 1)
      K = K + 1
      L0 = L1
      L1 = L2
    Next I
    Return L2
  End Function

  Private Sub fxGaussLaguerreQuadrature()
    ' Gauss-Laguerre Quadrature
    Const MAX_ITER = 1000

    Dim Sum, DeltaX, Toler As Double
    Dim Xa, Fa, Xb, Fb, Fx As Double
    Dim Xrt, Wt, h, Diff As Double
    Dim N, Order, Iter As Integer
    Dim Maxx As Double = 1000

    Toler = Stack(Z_STACK_REG)
    DeltaX = Stack(Y_STACK_REG)
    Order = Fix(Stack(X_STACK_REG))

    Try
      N = Order
      Sum = 0
      Xb = 0
      Fb = LaguerrePoly(Xb, Order)

      ' find the roots of the Laguerre polynomials using the scan range method
      Do
        Xa = Xb
        Fa = Fb
        Xb = Xa + DeltaX
        Fb = LaguerrePoly(Xb, Order)
        ' root in interval [Xa, Xb]?
        If Fa * Fb < 0 Then
          ' start Newton's method
          Xrt = (Xa + Xb) / 2
          Iter = 0
          Do
            Iter += 1
            h = 0.001 * (1 + Abs(Xrt))
            Diff = 2 * h * LaguerrePoly(Xrt, Order) / (LaguerrePoly(Xrt + h, Order) - LaguerrePoly(Xrt - h, Order))
            Xrt -= Diff
          Loop Until Abs(Diff) <= Toler Or Iter > MAX_ITER
          N -= 1
          ' calculaet weight at x root
          Wt = Xrt / (Order + 1) ^ 2 / LaguerrePoly(Xrt, Order + 1) ^ 2
          ' calculate function value
          Stack(X_STACK_REG) = Xrt
          Fx = Calc(Alpha)
          ' update area value
          Sum += Wt * Fx
        End If
      Loop Until N = 0 Or Xa > Maxx

      ' found all roots?
      If N = 0 Then
        Stack(X_STACK_REG) = Sum
      Else
        WriteLine("Error in Gauss-Laguerre quadrature calcations.")
        Stack(X_STACK_REG) = 0
      End If
    Catch
      sErrMessage = "Runtime math error in the GAUSSLAGQUAD command"
    End Try
    bShowStack = True
  End Sub

  Private Function Factorial(ByVal N As Integer) As Double
    Dim Prod As Double = 1

    Do While N > 1
      Prod *= N
      N -= 1
    Loop
    Return Prod
  End Function

  Private Function HermitePoly(ByVal X As Double, ByVal Order As Integer) As Double
    ' Implementation of recursive relation for Hermite polynomials.
    Dim H0, H1, H2 As Double
    Dim I, K As Integer

    H0 = 1
    If Order = 0 Then Return H0

    H1 = 2 * X
    If Order = 1 Then Return H1

    K = 1
    For I = 2 To Order
      H2 = 2 * X * H1 - 2 * K * H0
      K += 1
      H0 = H1
      H1 = H2
    Next I

    Return H2
  End Function

  Private Sub fxGaussHermiteQuadrature()
    ' Gauss-Hermite quadratire
    Const MAX_ITER = 1000

    Dim Sum, DeltaX, Toler As Double
    Dim Xa, Fa, Xb, Fb, Fx As Double
    Dim Xrt, Wt, h, Diff As Double
    Dim N, Order, Iter As Integer
    Dim Maxx As Double = 1000
    Dim SqrtPi As Double = Sqrt(PI)

    Toler = Stack(Z_STACK_REG)
    DeltaX = Stack(Y_STACK_REG)
    Order = Fix(Stack(X_STACK_REG))

    Try
      N = Order
      Sum = 0
      Xb = 0
      Fb = HermitePoly(Xb, Order)

      ' found root at x=0 (for odd polynomial orders)?
      If Abs(Fb) < 0.000001 Then
        N = N - 1
        Xrt = Xb
        Wt = 2 ^ (Order - 1) * Factorial(Order) * SqrtPi / Order ^ 2 / (HermitePoly(Xrt, Order - 1)) ^ 2
        Stack(X_STACK_REG) = 0
        Fx = Calc(Alpha)
        Sum += Wt * Fx
      End If

      ' find roots of the Hermite polynomial using the scan range method
      Do
        Xa = Xb
        Fa = Fb
        Xb = Xa + DeltaX
        Fb = HermitePoly(Xb, Order)
        ' root in interval [Xa, Xb]?
        If Fa * Fb < 0 Then
          ' start Newton;s method
          Xrt = (Xa + Xb) / 2
          Iter = 0
          Do
            Iter = Iter + 1
            h = 0.001 * (1 + Abs(Xrt))
            Diff = 2 * h * HermitePoly(Xrt, Order) / (HermitePoly(Xrt + h, Order) - HermitePoly(Xrt - h, Order))
            Xrt -= Diff
          Loop Until Abs(Diff) <= Toler Or Iter > MAX_ITER
          N -= 2 ' decrement 2 ... one for positive-value root and one for it's negative twin

          ' calculate weight for postive and negative roots
          Wt = 2 ^ (Order - 1) * Factorial(Order) * SqrtPi / Order ^ 2 / (HermitePoly(Xrt, Order - 1)) ^ 2
          Stack(X_STACK_REG) = Xrt
          Fx = Calc(Alpha)
          ' update the value of the integral
          Sum += Wt * Fx
          Xrt = -Xrt
          Stack(X_STACK_REG) = Xrt
          Fx = Calc(Alpha)
          ' update the value of the integral
          Sum += Wt * Fx
        End If
      Loop Until N = 0 Or Xa > Maxx

      ' found all roots?
      If N = 0 Then
        Stack(X_STACK_REG) = Sum
      Else
        WriteLine("Error in Gauss-Hermite quadrature calcations.")
        Stack(X_STACK_REG) = 0
      End If
    Catch
      sErrMessage = "Runtime math error in the GAUSSHERQUAD command"
    End Try
    bShowStack = True
  End Sub

  Private Sub fxGaussChebyshevQuadrature()
    ' Gauss-Chebyshev Quadrature
    Dim I, N As Integer
    Dim A, B, X, Y, W, T1, T2, T3 As Double
    Dim Sum As Double = 0

    N = Fix(Stack(Z_STACK_REG))
    A = Stack(X_STACK_REG)
    B = Stack(Y_STACK_REG)

    Try
      W = PI / N
      T1 = (B - A) / 2
      T2 = (B + A) / 2
      T3 = PI / 2 / N

      For I = 1 To N
        Y = Cos((2 * I - 1) * T3)
        X = T1 * Y + T2
        Stack(X_STACK_REG) = X
        Sum += Calc(Alpha) * Sqrt(1 - Y * Y)
      Next
      Stack(X_STACK_REG) = T1 * W * Sum
    Catch
      sErrMessage = "Runtime math error in the GAUSSCHEBQUAD command"
    End Try
    bShowStack = True
  End Sub

  ' ------------- helper functions for Gauss-Kronrod Quadrature
  Private Function abwe1(ByVal n As Integer, ByVal m As Integer,
           ByVal toler As Double, ByVal coeff2 As Double,
           ByVal even As Boolean, ByRef b() As Double,
           ByRef x As Double, ByRef w As Double) As Boolean

    '*****************************************************************************80
    '
    ' ABWE1 calculates a Kronrod abscissa and weight.
    '
    '  Licensing:
    '
    '    This code is distributed under the GNU LGPL license.
    '
    '  Modified:
    '
    '    03 August 2010
    '
    '  Author:
    '
    '    Original FORTRAN77 version by Robert Piessens, Maria Branders.
    '    MATLAB version by John Burkardt. Translated from MATLAB into 
    '    Visual Basic by Namir Shammas
    '
    '  Reference:
    '
    '    Robert Piessens, Maria Branders,
    '    A Note on the Optimal Addition of Abscissas to Quadrature Formulas
    '    of Gauss and Lobatto,
    '    Mathematics of Computation,
    '    Volume 28, Number 125, January 1974, pages 135-139.
    '
    '  Parameters:
    '
    '    Input, integer N, the order of the Gauss rule.
    '
    '    Input, integer M, the value of ( N + 1 ) / 2.
    '
    '    Input, real TOL, the requested absolute accuracy of the
    '    abscissas.
    '
    '    Input, real COEF2, a value needed to compute weights.
    '
    '    Input, logical EVEN, is TRUE if N is even.
    '
    '    Input, real B(M+1), the Chebyshev coefficients.
    '
    '    Input, real X, an estimate for the abscissa.
    '
    '    Output, real X, the abscissa.
    '
    '    Output, real W, the weight.
    '
    Dim i, iter As Integer
    Dim ai, b0, b1, b2, yy, d0, d1, d2, dif, f, fd As Double
    Dim delta, ka As Double

    Try
      If x = 0.0 Then
        ka = 1
      Else
        ka = 0
      End If
      '
      '  Iterative process for the computation of a Kronrod abscissa.
      '
      For iter = 1 To 50

        b1 = 0
        b2 = b(m + 1)
        yy = 4 * x * x - 2
        d1 = 0

        If even Then
          ai = m + m + 1
          d2 = ai * b(m + 1)
          dif = 2
        Else
          ai = m + 1
          d2 = 0
          dif = 1
        End If

        For k = 1 To m
          ai -= dif
          i = m - k + 1
          b0 = b1
          b1 = b2
          d0 = d1
          d1 = d2
          b2 = yy * b1 - b0 + b(i)
          If Not even Then
            i += 1
          End If
          d2 = yy * d1 - d0 + ai * b(i)
        Next

        If even Then
          f = x * (b2 - b1)
          fd = d2 + d1
        Else
          f = (b2 - b0) / 2
          fd = 4 * x * d2
        End If
        '
        '  Newton correction.
        '
        delta = f / fd
        x -= delta

        If ka = 1 Then Exit For

        If Abs(delta) <= toler Then
          ka = 1
        End If

      Next
      '
      '  Catch non-convergence.
      '
      If ka <> 1 Then Return False
      '
      '  Computation of the weight.
      '
      d0 = 1
      d1 = x
      ai = 0.0
      For k = 2 To n
        ai += 1
        d2 = ((2 * ai + 1) * x * d1 - ai * d0) / (ai + 1)
        d0 = d1
        d1 = d2
      Next

      w = coeff2 / (fd * d2)
    Catch
      Return False
    End Try
    Return True
  End Function

  Private Function abwe2(ByVal n As Integer, ByVal m As Integer,
           ByVal toler As Double, ByVal coeff2 As Double,
           ByVal even As Boolean, ByRef b() As Double,
           ByRef x As Double, ByRef w1 As Double, ByRef w2 As Double) As Boolean
    '*****************************************************************************80
    '
    '' ABWE2 calculates a Gaussian abscissa and two weights.
    '
    '  Licensing:
    '
    '    This code is distributed under the GNU LGPL license.
    '
    '  Modified:
    '
    '    30 April 2013
    '
    '  Author:
    '
    '    Original FORTRAN77 version by Robert Piessens, Maria Branders.
    '    MATLAB version by John Burkardt. Translated from MATLAB into 
    '    Visual Basic by Namir Shammas
    '
    '  Reference:
    '
    '    Robert Piessens, Maria Branders,
    '    A Note on the Optimal Addition of Abscissas to Quadrature Formulas
    '    of Gauss and Lobatto,
    '    Mathematics of Computation,
    '    Volume 28, Number 125, January 1974, pages 135-139.
    '
    '  Parameters:
    '
    '    Input, integer N, the order of the Gauss rule.
    '
    '    Input, integer M, the value of ( N + 1 ) / 2.
    '
    '    Input, real TOL, the requested absolute accuracy of the
    '    abscissas.
    '
    '    Input, real COEF2, a value needed to compute weights.
    '
    '    Input, logical EVEN, is TRUE if N is even.
    '
    '    Input, real B(M+1), the Chebyshev coefficients.
    '
    '    Input, real X, an estimate for the abscissa.
    '
    '    Output, real X, the abscissa.
    '
    '    Output, real W1, the Gauss-Kronrod weight.
    '
    '    Output, real W2, the Gauss weight.
    '
    Dim i, k, iter As Integer
    Dim an, ai, b0, b1, b2, yy, d1, d2, dif, f, fd As Double
    Dim p0, p1, p2, pd0, pd1, pd2 As Double
    Dim delta, ka As Double

    Try
      If x = 0.0 Then
        ka = 1
      Else
        ka = 0
      End If
      '
      '  Iterative process for the computation of a Gaussian abscissa.
      '
      For iter = 1 To 50

        p0 = 1
        p1 = x
        pd0 = 0
        pd1 = 1
        '
        '  When N is 1, we need to initialize P2 and PD2 to avoid problems with DELTA.
        '
        If n <= 1 Then
          If Double.MinValue < Abs(x) Then
            p2 = (3 * x * x - 1) / 2
            pd2 = 3 * x
          Else
            p2 = 3 * x
            pd2 = 3
          End If
        End If

        ai = 0
        For k = 2 To n
          ai += 1
          p2 = ((2 * ai + 1) * x * p1 - ai * p0) / (ai + 1)
          pd2 = ((2 * ai + 1) * (p1 + x * pd1) - ai * pd0) / (ai + 1)
          p0 = p1
          p1 = p2
          pd0 = pd1
          pd1 = pd2
        Next
        '
        '  Newton correction.
        '
        delta = p2 / pd2
        x -= delta

        If ka = 1 Then Exit For

        If Abs(delta) <= toler Then
          ka = 1
        End If

      Next
      '
      '  Catch non-convergence.
      '
      If ka <> 1 Then Return False

      '
      '  Computation of the weight.
      '
      an = n

      w2 = 2 / (an * pd2 * p0)

      p1 = 0
      p2 = b(m + 1)
      yy = 4 * x * x - 2
      For k = 1 To m
        i = m - k + 1
        p0 = p1
        p1 = p2
        p2 = yy * p1 - p0 + b(i)
      Next k

      If even Then
        w1 = w2 + coeff2 / (pd2 * x * (p2 - p1))
      Else
        w1 = w2 + 2.0 * coeff2 / (pd2 * (p2 - p0))
      End If
    Catch
      Return False
    End Try

    Return True
  End Function

  Private Function kronrod(ByVal n As Integer, ByVal toler As Double,
                  ByRef x() As Double, ByRef w1() As Double,
                  ByRef w2() As Double) As Boolean

    '*****************************************************************************80
    '
    '' KRONROD adds N+1 points to an N-point Gaussian rule.
    '
    '  Discussion:
    '
    '    This subroutine calculates the abscissas and weights of the 2N+1
    '    point Gauss Kronrod quadrature formula which is obtained from the
    '    N point Gauss quadrature formula by the optimal addition of N+1 points.
    '
    '    The optimally added points are called Kronrod abscissas.  The
    '    abscissas and weights for both the Gauss and Gauss Kronrod rules
    '    are calculated for integration over the interval [-1,+1].
    '
    '    Since the quadrature formula is symmetric with respect to the origin,
    '    only the nonnegative abscissas are calculated.
    '
    '    Note that the code published in Mathematics of Computation
    '    omitted the definition of the variable which is here called COEF2.
    '
    '  Storage:
    '
    '    Given N, let M = ( N + 1 ) / 2.
    '
    '    The Gauss-Kronrod rule will include 2*N+1 points.  However, by symmetry,
    '    only N + 1 of them need to be listed.
    '
    '    The arrays X, W1 and W2 contain the nonnegative abscissas in decreasing
    '    order, and the weights of each abscissa in the Gauss-Kronrod and
    '    Gauss rules respectively.  This means that about half the entries
    '    in W2 are zero.
    '
    '    For instance, if N = 3, the output is:
    '
    '    I      X               W1              W2
    '
    '    1    0.960491        0.104656         0.000000
    '    2    0.774597        0.268488         0.555556
    '    3    0.434244        0.401397         0.000000
    '    4    0.000000        0.450917         0.888889
    '
    '    and if N = 4, (notice that 0 is now a Kronrod abscissa)
    '    the output is
    '
    '    I      X               W1              W2
    '
    '    1    0.976560        0.062977        0.000000
    '    2    0.861136        0.170054        0.347855
    '    3    0.640286        0.266798        0.000000
    '    4    0.339981        0.326949        0.652145
    '    5    0.000000        0.346443        0.000000
    '
    '  Licensing:
    '
    '    This code is distributed under the GNU LGPL license.
    '
    '  Modified:
    '
    '    03 August 2010
    '
    '  Author:
    '
    '    Original FORTRAN77 version by Robert Piessens, Maria Branders.
    '    MATLAB version by John Burkardt. Translated from MATLAB into 
    '    Visual Basic by Namir Shammas
    '
    '  Reference:
    '
    '    Robert Piessens, Maria Branders,
    '    A Note on the Optimal Addition of Abscissas to Quadrature Formulas
    '    of Gauss and Lobatto,
    '    Mathematics of Computation,
    '    Volume 28, Number 125, January 1974, pages 135-139.
    '
    '  Parameters:
    '
    '    Input, integer N, the order of the Gauss rule.
    '
    '    Input, real TOL, the requested absolute accuracy of the
    '    abscissas.
    '
    '    Output, real X(N+1), the abscissas.
    '
    '    Output, real W1(N+1), the weights for the Gauss-Kronrod rule.
    '
    '    Output, real W2(N+1), the weights for the Gauss rule.
    '
    Dim m, k, L, L2 As Integer
    Dim an, ak, coef, coeff2, d, xx As Double
    Dim bb, c, s, x1, y As Double
    Dim Even As Boolean
    Dim tau(), b() As Double

    Try
      m = Fix((n + 1) / 2)
      Even = (2 * m = n)

      d = 2
      an = 0
      For k = 1 To n
        an = an + 1
        d = d * an / (an + 0.5)
      Next
      '
      '  Calculation of the Chebyshev coefficients of the orthogonal polynomial.
      '
      ReDim b(n + 1), tau(n + 1), x(n + 1), w1(n + 1), w2(n + 1)

      tau(1) = (an + 2) / (2 * an + 3)
      b(m) = tau(1) - 1
      ak = an

      For L = 1 To m - 1

        ak += 2
        tau(L + 1) = ((ak - 1.0) * ak - an * (an + 1.0)) * (ak + 2.0) * tau(L) / (ak * ((ak + 3.0) * (ak + 2.0) - an * (an + 1.0)))
        b(m - L) = tau(L + 1)

        For L2 = 1 To L
          b(m - L) += tau(L2) * b(m - L + L2)
        Next

      Next

      b(m + 1) = 1
      '
      '  Calculation of approximate values for the abscissas.
      '
      bb = Sin(PI / 2 / (2 * an + 1))
      x1 = Sqrt(1 - bb * bb)
      s = 2 * bb * x1
      c = Sqrt(1 - s * s)
      coef = 1 - (1 - 1 / an) / (8 * an * an)
      xx = coef * x1
      '
      '  Coefficient needed for weights.
      '
      '  COEF2 = 2^(2*n+1) * n! * n! / (2n+1)!
      '
      coeff2 = 2.0 / (2 * n + 1)
      For i = 1 To n
        coeff2 *= 4 * i / (n + i)
      Next
      '
      '  Calculation of the K-th abscissa (a Kronrod abscissa) and the
      '  corresponding weight.
      '
      For k = 1 To n Step 2

        If Not abwe1(n, m, toler, coeff2, Even, b, xx, w1(k)) Then Return False

        w2(k) = 0

        x(k) = xx
        y = x1
        x1 = y * c - bb * s
        bb = y * s + bb * c

        If k = n Then
          xx = 0.0
        Else
          xx = coef * x1
        End If
        '
        '  Calculation of the K+1 abscissa (a Gaussian abscissa) and the
        '  corresponding weights.
        '
        If Not abwe2(n, m, toler, coeff2, Even, b, xx, w1(k + 1), w2(k + 1)) Then Return False
        x(k + 1) = xx
        y = x1
        x1 = y * c - bb * s
        bb = y * s + bb * c
        xx = coef * x1

      Next
      '
      '  If N is even, we have one more Kronrod abscissa to compute,
      '  namely the origin.
      '
      If Even Then
        xx = 0
        If Not abwe1(n, m, toler, coeff2, Even, b, xx, w1(n + 1)) Then Return False
        w2(n + 1) = 0
        x(n + 1) = xx
      End If
    Catch
      Return False
    End Try

    Return True
  End Function

  Private Function kronrod25(ByRef x() As Double, ByRef w() As Double) As Integer

    Dim i, j, n As Integer

    n = 13 ' only 13 actual points are needed for 25 points

    ReDim x(n), w(n)

    x(1) = 0.9969339225296
    x(2) = 0.9815606342467
    x(3) = 0.9505377959431
    x(4) = 0.9041172563705
    x(5) = 0.8435581241612
    x(6) = 0.7699026741943
    x(7) = 0.68405989547006
    x(8) = 0.5873179542866
    x(9) = 0.48133945047816
    x(10) = 0.36783149899818
    x(11) = 0.2485057483205
    x(12) = 0.12523340851147
    x(13) = 0

    w(1) = 0.0082577114332
    w(2) = 0.023036084039
    w(3) = 0.0389152304693
    w(4) = 0.05369701760776
    w(5) = 0.06725090705084
    w(6) = 0.0799202753336
    w(7) = 0.09154946829505
    w(8) = 0.10164973227906
    w(9) = 0.11002260497764
    w(10) = 0.1167120535018
    w(11) = 0.1216263035239
    w(12) = 0.1245841645362
    w(13) = 0.1255568939055

    Return n
  End Function

  Private Function kronrod51(ByRef x() As Double, ByRef w() As Double) As Integer

    Dim i, j, n As Integer

    n = 26 ' only 26 actual points are needed for 51 points

    ReDim x(n), w(n)

    x(1) = 0.99926210499260981
    x(2) = 0.99555696979049813
    x(3) = 0.98803579453407719
    x(4) = 0.97666392145951753
    x(5) = 0.96161498642584253
    x(6) = 0.94297457122897432
    x(7) = 0.92074711528170161
    x(8) = 0.89499199787827532
    x(9) = 0.8658470652932756
    x(10) = 0.833442628760834
    x(11) = 0.79787379799850011
    x(12) = 0.75925926303735758
    x(13) = 0.71776640681308435
    x(14) = 0.6735663684734684
    x(15) = 0.62681009901031737
    x(16) = 0.577662930241223
    x(17) = 0.52632528433471915
    x(18) = 0.473002731445715
    x(19) = 0.41788538219303772
    x(20) = 0.36117230580938786
    x(21) = 0.30308953893110785
    x(22) = 0.24386688372098844
    x(23) = 0.18371893942104889
    x(24) = 0.1228646926107104
    x(25) = 0.061544483005685081
    x(26) = 0

    w(1) = 0.0019873838923303161
    w(2) = 0.005561932135356714
    w(3) = 0.0094739733861741518
    w(4) = 0.013236229195571676
    w(5) = 0.0168478177091283
    w(6) = 0.020435371145882834
    w(7) = 0.024009945606953215
    w(8) = 0.027475317587851739
    w(9) = 0.030792300167387487
    w(10) = 0.034002130274329335
    w(11) = 0.037116271483415543
    w(12) = 0.040083825504032382
    w(13) = 0.042872845020170053
    w(14) = 0.045502913049921788
    w(15) = 0.047982537138836712
    w(16) = 0.050277679080715669
    w(17) = 0.052362885806407473
    w(18) = 0.054251129888545489
    w(19) = 0.055950811220412316
    w(20) = 0.057437116361567835
    w(21) = 0.058689680022394206
    w(22) = 0.059720340324174059
    w(23) = 0.06053945537604586
    w(24) = 0.061128509717053046
    w(25) = 0.061471189871425316
    w(26) = 0.061580818067832936

    Return n
  End Function

  Private Function kronrod99(ByRef x() As Double, ByRef w() As Double) As Integer

    Dim i, j, n As Integer

    n = 50 ' only 50 actual points are needed for 99 points

    ReDim x(n), w(n)

    x(1) = 0.999804199686
    x(2) = 0.998820150607
    x(3) = 0.9968198143
    x(4) = 0.993788661944
    x(5) = 0.989764238414
    x(6) = 0.984757895914
    x(7) = 0.9787573030312
    x(8) = 0.9717622009016
    x(9) = 0.9637900181364
    x(10) = 0.954853658674
    x(11) = 0.9449550592213
    x(12) = 0.9341002947558
    x(13) = 0.9223054754539
    x(14) = 0.9095856558281
    x(15) = 0.8959496245836
    x(16) = 0.881408445573
    x(17) = 0.8659800043151
    x(18) = 0.8496821198442
    x(19) = 0.8325285014875
    x(20) = 0.8145344273599
    x(21) = 0.7957203207625
    x(22) = 0.7761068943454
    x(23) = 0.7557118903695
    x(24) = 0.7345542542374
    x(25) = 0.712657066705
    x(26) = 0.6900438244251
    x(27) = 0.6667357008416
    x(28) = 0.6427548324192
    x(29) = 0.6181268178009
    x(30) = 0.5928776941089
    x(31) = 0.5670315494954
    x(32) = 0.5406132469917
    x(33) = 0.5136506284017
    x(34) = 0.4861719414525
    x(35) = 0.458203691539
    x(36) = 0.42977299334158
    x(37) = 0.4009095739293
    x(38) = 0.37164350126228
    x(39) = 0.34200319599186
    x(40) = 0.31201753211975
    x(41) = 0.281717708241
    x(42) = 0.25113517861258
    x(43) = 0.2202997655098
    x(44) = 0.1892415924618
    x(45) = 0.1579928776644
    x(46) = 0.1265859972697
    x(47) = 0.09505164998612
    x(48) = 0.06342068498269
    x(49) = 0.03172586345907
    x(50) = 0

    w(1) = 0.0005274769684
    w(2) = 0.00147796392817
    w(3) = 0.00252187657315
    w(4) = 0.0035329557015
    w(5) = 0.0045141331626
    w(6) = 0.0055014203993
    w(7) = 0.00649987253217
    w(8) = 0.00748691027641
    w(9) = 0.0084551856398
    w(10) = 0.0094175722297
    w(11) = 0.0103787329241
    w(12) = 0.0113278438579
    w(13) = 0.0122591748799
    w(14) = 0.013179206821
    w(15) = 0.0140911110473
    w(16) = 0.0149880989347
    w(17) = 0.0158657236989
    w(18) = 0.0167278985538
    w(19) = 0.01757687264125
    w(20) = 0.018407753852
    w(21) = 0.0192169329826
    w(22) = 0.02000706438595
    w(23) = 0.0207798535199
    w(24) = 0.0215314818078
    w(25) = 0.02225891393014
    w(26) = 0.0229641216312
    w(27) = 0.0236484943738
    w(28) = 0.0243089029298
    w(29) = 0.0249427312116
    w(30) = 0.02555156763
    w(31) = 0.026136629573
    w(32) = 0.0266952731072
    w(33) = 0.0272252058088
    w(34) = 0.02772780755086
    w(35) = 0.02820422057002
    w(36) = 0.0286521667109
    w(37) = 0.0290696140122
    w(38) = 0.0294578454056
    w(39) = 0.0298179973237
    w(40) = 0.0301480810668
    w(41) = 0.0304462797397
    w(42) = 0.03071385585774
    w(43) = 0.0309519926156
    w(44) = 0.0311589374966
    w(45) = 0.03133305259123
    w(46) = 0.031475635826999
    w(47) = 0.0315879594587
    w(48) = 0.03166846605661
    w(49) = 0.031715664504
    w(50) = 0.03173093139852

    Return n
  End Function

  Private Sub fxGaussKronrodQuadrature()
    ' Gauss-Kronrod Quadrature
    Dim I, J, N, NumDivs As Integer
    Dim A, B, T1, T2, T3, toler, Xv As Double
    Dim Xa, Xb, DeltaX, SubSum As Double
    Dim X(), W1(), W2() As Double
    Dim Sum As Double = 0

    N = Fix(Stack(T_STACK_REG))
    toler = Stack(T_STACK_REG)
    A = Stack(X_STACK_REG)
    B = Stack(Y_STACK_REG)
    Try
      If kronrod(N, toler, X, W1, W2) Then
        T1 = (B - A) / 2
        T2 = (B + A) / 2

        For I = 1 To N + 1
          Xv = T2 + T1 * X(I)
          Stack(X_STACK_REG) = Xv
          Sum += Calc(Alpha) * W1(I)
        Next
        I = N + 2
        For I = 1 To N
          Xv = T2 - T1 * X(I)
          Stack(X_STACK_REG) = Xv
          Sum += Calc(Alpha) * W1(I)
        Next
        Sum *= T1

      Else
        N = kronrod99(X, W1) - 1
        DeltaX = Abs(B - A)
        If DeltaX < 10 Then
          NumDivs = 1
        ElseIf DeltaX < 100 Then
          NumDivs = 10
        Else
          NumDivs = 100
        End If

        DeltaX /= NumDivs
        Xa = A
        Xb = Xa + DeltaX
        For J = 1 To NumDivs
          T1 = (Xb - Xa) / 2
          T2 = (Xb + Xa) / 2
          ' for node x(i) = 0 located at x(n+1) 
          Stack(X_STACK_REG) = T2 ' + T1 * 0
          SubSum = Calc(Alpha) * W1(N + 1)
          ' calculate sum of terms with non-zero abscissa
          For I = 1 To N
            ' for positive abscissa
            Xv = T2 + T1 * X(I)
            Stack(X_STACK_REG) = Xv
            SubSum += Calc(Alpha) * W1(I)
            ' for negative abscissa with same magnitude
            Xv = T2 - T1 * X(I)
            Stack(X_STACK_REG) = Xv
            SubSum += Calc(Alpha) * W1(I)
          Next
          Sum += T1 * SubSum

          Xa = Xb
          Xb += DeltaX
        Next ' J
      End If
      BackupStack()
      Enter(Sum)

    Catch
      sErrMessage = "Runtime math error in the GAUSSKRONQUAD command"
    End Try
    bShowStack = True
  End Sub

  Private Sub fxGaussKronrodQuadrature2()
    ' Gauss-Kronrod Quadrature
    Dim I, J, N, NumDivs As Integer
    Dim A, B, T1, T2, T3, toler, Xv As Double
    Dim Xa, Xb, DeltaX, SubSum As Double
    Dim X(), W1(), W2() As Double
    Dim Sum As Double = 0

    N = Fix(Stack(T_STACK_REG))
    toler = Stack(T_STACK_REG)
    A = Stack(X_STACK_REG)
    B = Stack(Y_STACK_REG)
    Try
      T1 = (B - A) / 2
      T2 = (B + A) / 2

      If kronrod(N, toler, X, W1, W2) Then
        For I = 1 To N + 1
          Xv = T2 + T1 * X(I)
          Stack(X_STACK_REG) = Xv
          Sum += Calc(Alpha) * W1(I)
        Next
        I = N + 2
        For I = 1 To N
          Xv = T2 - T1 * X(I)
          Stack(X_STACK_REG) = Xv
          Sum += Calc(Alpha) * W1(I)
        Next
      Else
        DeltaX = Abs(B - A)
        If DeltaX < 10 Then
          N = kronrod25(X, W1) - 1
        ElseIf DeltaX < 100 Then
          N = kronrod51(X, W1) - 1
        Else
          N = kronrod99(X, W1) - 1
        End If

        ' for node x(i) = 0 located at x(n+1) 
        Stack(X_STACK_REG) = T2 ' + T1 * 0
        Sum = Calc(Alpha) * W1(N + 1) ' W(n+1)
        ' calculate sum of terms with non-zero abscissa
        For I = 1 To N
          ' for positive abscissa
          Xv = T2 + T1 * X(I)
          Stack(X_STACK_REG) = Xv
          Sum += Calc(Alpha) * W1(I)
          ' for negative abscissa with same magnitude
          Xv = T2 - T1 * X(I)
          Stack(X_STACK_REG) = Xv
          Sum += Calc(Alpha) * W1(I)
        Next
      End If
      Sum *= T1
      BackupStack()
      Enter(Sum)

    Catch
      sErrMessage = "Runtime math error in the GAUSSKRONQUAD2 command"
    End Try
    bShowStack = True
  End Sub

  ' ------------- end of helper functions for Gauss-Kronrod Quadrature

  Private Sub fxIntegrate()
    ' Implement Integrate INTEG command. Use Romberg's method.
    Dim a, b, toler, Sk As Double
    Dim J, K As Integer
    Dim X, X0, Fx, Temp1, DeltaU, Ui As Double
    Dim R(30, 30) As Double

    BackupStack()
    bShowStack = False
    ' get input from the stack
    toler = Stack(Z_STACK_REG)
    b = Stack(X_STACK_REG)
    a = Stack(Y_STACK_REG)
    X0 = Stack(X_STACK_REG)

    ' if a>b then swap a and b
    If a > b Then
      X = a
      a = b
      b = X
    End If

    Try
      Sk = 0
      K = -1
      Do
        Do
          K += +1
          Temp1 = Pow(2, -K)
          DeltaU = 2 * Temp1
          Ui = -1 + Temp1
          Do
            X = (b - a) / 4 * Ui * (3 - Ui ^ 2) + (b + a) / 2
            Stack(X_STACK_REG) = X
            Fx = Calc(Alpha)
            If GetErrMessage() <> "" Then Exit Sub
            Sk = Sk + Fx * (1 - Ui ^ 2)
            Ui = Ui + DeltaU
          Loop While Ui < 1
          R(K, 0) = 3 * (b - a) / 4 * Temp1 * Sk
        Loop While K = 0

        For J = 1 To K
          R(K, J) = R(K, J - 1) + ((R(K, J - 1) - R(K - 1, J - 1)) / (4 ^ J - 1))
        Next J
      Loop While Abs(R(K, K) - R(K - 1, K - 1)) / Abs(R(K, K)) > toler
      Enter(R(K, K))

    Catch
      sErrMessage = "Runtime math error in the INTEG command"
    End Try
    bShowStack = True
  End Sub

  Private Sub fxSolve()
    ' Implement SOLVE command. Use Newton's method
    Dim Toler, X, X0, h, Fx0, Fx1, Diff As Double
    Dim MaxIters, Iters As Double

    BackupStack()
    bShowStack = False
    X = Stack(X_STACK_REG)
    X0 = X
    Toler = Stack(Z_STACK_REG)
    MaxIters = Abs(Fix(Stack(Y_STACK_REG)))
    Iters = 0
    Try
      Do
        Iters += 1
        h = 0.001 * (1 + Abs(X))
        Stack(X_STACK_REG) = X
        Fx0 = Calc(Alpha)
        Stack(X_STACK_REG) = X + h
        Fx1 = Calc(Alpha)
        Diff = h * Fx0 / (Fx1 - Fx0)
        X -= Diff
      Loop Until Abs(Diff) < Toler Or Iters > MaxIters

      ' solution diverged?
      If Iters > MaxIters Then
        WriteLine("Warning! Solution diverged when using SOLVE command")
      End If

      BackupStack()
      Stack(X_STACK_REG) = X
      Fx0 = Calc(Alpha)
      Stack(X_STACK_REG) = X
      Stack(Y_STACK_REG) = Iters
      Stack(Z_STACK_REG) = Fx0
      Stack(T_STACK_REG) = Diff
    Catch
      sErrMessage = "Runtime math error in the SOLVE command"
    End Try
    bShowStack = True
  End Sub

  Private Sub fxSolveHalley()
    ' Implement SOLVE command. Use Halley's method
    Dim Toler, X, X0, h, Fxm, Fx, Fxp, Drv1, Drv2, Diff As Double
    Dim MaxIters, Iters As Double

    BackupStack()
    bShowStack = False
    X = Stack(X_STACK_REG)
    X0 = X
    Toler = Stack(Z_STACK_REG)
    MaxIters = Abs(Fix(Stack(Y_STACK_REG)))
    Iters = 0
    Try
      Do
        Iters += 1
        h = 0.001 * (1 + Abs(X))
        Stack(X_STACK_REG) = X
        Fx = Calc(Alpha)
        Stack(X_STACK_REG) = X + h
        Fxp = Calc(Alpha)
        Stack(X_STACK_REG) = X - h
        Fxm = Calc(Alpha)
        Drv1 = (Fxp - Fxm) / 2 / h
        Drv2 = (Fxp - 2 * Fx + Fxm) / h / h
        Diff = 2 * Fx * Drv1 / (2 * Drv1 ^ 2 - Fx * Drv2)
        X -= Diff
      Loop Until Abs(Diff) < Toler Or Iters > MaxIters

      ' solution diverged?
      If Iters > MaxIters Then
        WriteLine("Warning! Solution diverged when using SOLVEHAL command")
      End If

      BackupStack()
      Stack(X_STACK_REG) = X
      Fx = Calc(Alpha)
      Stack(X_STACK_REG) = X
      Stack(Y_STACK_REG) = Iters
      Stack(Z_STACK_REG) = Fx
      Stack(T_STACK_REG) = Diff
    Catch
      sErrMessage = "Runtime math error in the SOLVEHAL command"
    End Try
    bShowStack = True
  End Sub

  Private Sub fxSolveOstrowski()
    ' Implement SOLVE command. Use Ostrowski's method
    Dim Toler, h, X, X0, Fx, Fxp, Y, Fy, Diff As Double
    Dim MaxIters, Iters As Double

    BackupStack()
    bShowStack = False
    X = Stack(X_STACK_REG)
    X0 = X
    Toler = Stack(Z_STACK_REG)
    MaxIters = Abs(Fix(Stack(Y_STACK_REG)))
    Iters = 0
    Try
      Do
        X0 = X
        Iters += 1
        h = 0.001 * (1 + Abs(X))
        Stack(X_STACK_REG) = X
        Fx = Calc(Alpha)
        Stack(X_STACK_REG) = X + h
        Fxp = Calc(Alpha)
        ' calculate y as the intermediate refinment per iteration
        Y = X - h * Fx / (Fxp - Fx)
        Stack(X_STACK_REG) = Y
        Fy = Calc(Alpha)
        X = Y - Fx * (X - Y) / (Fx - 2 * Fy)
        Diff = X - X0
      Loop Until Abs(Diff) < Toler Or Iters > MaxIters

      ' solution diverged?
      If Iters > MaxIters Then
        WriteLine("Warning! Solution diverged when using SOLVEHAL command")
      End If

      BackupStack()
      Stack(X_STACK_REG) = X
      Fx = Calc(Alpha)
      Stack(X_STACK_REG) = X
      Stack(Y_STACK_REG) = Iters
      Stack(Z_STACK_REG) = Fx
      Stack(T_STACK_REG) = Diff
    Catch
      sErrMessage = "Runtime math error in the SOLVEHAL command"
    End Try
    bShowStack = True
  End Sub

  Private Sub fxSolveBisection()
    ' Implement SOLVE command. Use the Bisection method
    Dim Toler, A, B, C, Fa, Fb, Fc As Double

    BackupStack()
    bShowStack = False
    A = Stack(Y_STACK_REG)
    B = Stack(X_STACK_REG)
    ' make sure A < B
    If A > B Then
      C = A
      A = B
      B = C
    End If
    Toler = Stack(Z_STACK_REG)
    Stack(X_STACK_REG) = A
    Fa = Calc(Alpha)
    Stack(X_STACK_REG) = C
    Fb = Calc(Alpha)

    If Fa * Fb > 0 Then
      Stack(X_STACK_REG) = Double.MaxValue
      WriteLine("Values at A and B have same the function sign using SOLVEBIS command")
      Exit Sub
    End If

    Try
      Do
        C = (A + B) / 2
        Stack(X_STACK_REG) = C
        Fc = Calc(Alpha)
        If Fc * Fa > 0 Then
          A = C
          Fa = Fc
        ElseIf Fc * Fa < 0 Then
          Fb = Fc
          B = C
        Else
          Exit Do
        End If
      Loop Until Abs(B - A) <= Toler
      BackupStack()
      C = (A + B) / 2
      Stack(X_STACK_REG) = C
      Fc = Calc(Alpha)
      Stack(X_STACK_REG) = C
      Stack(Y_STACK_REG) = Fc
    Catch
      sErrMessage = "Runtime math error in the SOLVEBIS command"
    End Try
    bShowStack = True
  End Sub

  '!!!!
  Private Function MySlope(ByVal X As Double, ByVal Fx As Double) As Double
    Dim h As Double
    h = 0.001 * (1 + Abs(X))
    Stack(X_STACK_REG) = X + h
    MySlope = (Calc(Alpha) - Fx) / h
  End Function

  Private Function MySecDeriv(ByVal X As Double) As Double
    Dim h, Fp, Fx, Fm As Double

    h = 0.001 * (1 + Abs(X))
    Stack(X_STACK_REG) = X
    Fx = Calc(Alpha)
    Stack(X_STACK_REG) = X + h
    Fp = Calc(Alpha)
    Stack(X_STACK_REG) = X - h
    Fm = Calc(Alpha)
    MySecDeriv = (Fp - 2 * Fx + Fm) / h / h
  End Function

  Private Function NewtonRoot(ByVal X As Double, ByVal Toler As Double) As Double
    Dim h, Fx, Diff As Double

    Do
      h = 0.001 * (1 + Abs(X))
      Stack(X_STACK_REG) = X
      Fx = Calc(Alpha)
      Stack(X_STACK_REG) = X + h
      Diff = h * Fx / (Calc(Alpha) - Fx)
      X -= Diff
    Loop Until Abs(Diff) < Toler
    NewtonRoot = X
  End Function

  Private Function NewtonRootEx(ByVal A As Double, ByVal B As Double, ByVal Toler As Double) As Double
    Dim X, h, Fx, Fa, Diff As Double

    X = (A + B) / 2
    Do
      h = 0.001 * (1 + Abs(X))
      Stack(X_STACK_REG) = X
      Fx = Calc(Alpha)
      Stack(X_STACK_REG) = X + h
      Diff = h * Fx / (Calc(Alpha) - Fx)
      X -= Diff
      If X < A Or X > B Then
        X = (A + B) / 2
        Stack(X_STACK_REG) = A
        Fa = Calc(Alpha)
        Stack(X_STACK_REG) = X
        Fx = Calc(Alpha)
        If Fa * Fx > 0 Then
          A = X
        Else
          B = X
        End If
        Diff = B - A
      End If
    Loop Until Abs(Diff) <= Toler
    NewtonRootEx = X
  End Function

  Private Function NewtonMinMax(ByVal X As Double, ByVal Toler As Double) As Double
    Dim h, Fx, Diff As Double
    Dim Fp, Fm, Drv1, Drv2 As Double

    Do
      h = 0.001 * (1 + Abs(X))
      Stack(X_STACK_REG) = X
      Fx = Calc(Alpha)
      Stack(X_STACK_REG) = X + h
      Fp = Calc(Alpha)
      Stack(X_STACK_REG) = X - h
      Fm = Calc(Alpha)
      Drv1 = (Fp - Fm) / 2 / h
      Drv2 = (Fp - 2 * Fx + Fm) / h / h
      Diff = Drv1 / Drv2
      X -= Diff
    Loop Until Abs(Diff) < Toler
    NewtonMinMax = X
  End Function

  Private Function NewtonMinMaxEx(ByVal A As Double, ByVal B As Double, ByVal Toler As Double) As Double
    Dim h, Fx, Fa, Diff, X As Double
    Dim Fp, Fm, Drv1, Drv2 As Double

    X = (A + B) / 2
    Do
      h = 0.001 * (1 + Abs(X))

      Stack(X_STACK_REG) = X
      Fx = Calc(Alpha)
      Stack(X_STACK_REG) = X + h
      Fp = Calc(Alpha)
      Stack(X_STACK_REG) = X - h
      Fm = Calc(Alpha)
      Drv1 = (Fp - Fm) / 2 / h
      Drv2 = (Fp - 2 * Fx + Fm) / h / h
      Diff = Drv1 / Drv2
      X -= Diff
      If X < A Or X > B Then
        X = (A + B) / 2
        Stack(X_STACK_REG) = X
        Fx = Calc(Alpha)
        Stack(X_STACK_REG) = A
        Fa = Calc(Alpha)
        If MySlope(A, Fa) * MySlope(X, Fx) > 0 Then
          A = X
        Else
          B = X
        End If
        Diff = B - A
      End If
    Loop Until Abs(Diff) < Toler
    NewtonMinMaxEx = X
  End Function

  Private Sub fxScan()
    ' Scan a range and uses Newton's method to find a root when the sign of the
    ' funtion changes.
    Const MAX_ERROR = 5
    Dim NumSteps, ErrCount, SFa, SDa, SFb, SDb, MemIdx, MemIdx0, CountResults As Integer
    Dim A, B, Stp, Toler, FxToler As Double
    Dim Xa, Fa, Da As Double
    Dim Xb, Fb, Db As Double
    Dim Fx, h, X, Drv2 As Double
    Dim bMoveOneExtraStep As Boolean
    Dim sMsg As String

    B = Stack(X_STACK_REG)
    A = Stack(Y_STACK_REG)
    Stp = Stack(Z_STACK_REG)
    Toler = Stack(T_STACK_REG)
    FxToler = LastStack(X_STACK_REG)
    MemIdx0 = LastStack(Y_STACK_REG)
    MemIdx = MemIdx0 + 1
    CountResults = 0

    Xa = A
    Stack(X_STACK_REG) = A
    Fa = Calc(Alpha)
    Da = MySlope(Xa, Fa)
    SFa = Sign(Fa)
    SDa = Sign(Da)
    NumSteps = 0
    ErrCount = 0

    Do
      Try
        NumSteps += 1
        bMoveOneExtraStep = False
        Xb = A + NumSteps * Stp
        Stack(X_STACK_REG) = Xb
        Fb = Calc(Alpha)
        Db = MySlope(Xb, Fb)
        SFb = Sign(Fb)
        SDb = Sign(Db)
        ' landed on a root??
        If Abs(Fb) = 0 Then
          Drv2 = MySecDeriv(Xb)
          sMsg = "root"
          If SDa * SDb < 0 Then
            If Abs(Drv2) > FxToler Then
              If Drv2 > 0 Then
                sMsg = "root & minimum"
              Else
                sMsg = "root & maximum"
              End If
            Else
              sMsg = "root & saddle point"
            End If
          End If
          WriteLine("M({0}) has a {1} at {2}", MemIdx, sMsg, Output(Xb))
          WriteLine("M({0}) has F(x) = 0", MemIdx + 1)
          WriteLine("----------------------------------------")
          SetMem(MemIdx, Xb)
          SetMem(MemIdx + 1, 0)
          SetMem(MemIdx + 2, 0)
          MemIdx += 3
          CountResults += 1
          SetMem(MemIdx0, CountResults)
          bMoveOneExtraStep = True
          ' Found a range that contains a root?
        ElseIf SFb * SFa < 0 Then
          X = NewtonRoot((Xa + Xb) / 2, Toler)
          SetMem(MemIdx, X)
          Stack(X_STACK_REG) = X
          Fx = Calc(Alpha)
          SetMem(MemIdx + 1, Fx)
          SetMem(MemIdx + 2, 0)
          WriteLine("Result # {0}", CountResults + 1)
          WriteLine("M({0}) has root at {1}", MemIdx, Output(X))
          WriteLine("M({0}) has F(x) = {1}", MemIdx + 1, Output(Fx))
          WriteLine("----------------------------------------")
          MemIdx += 3
          CountResults += 1
          SetMem(MemIdx0, CountResults)
          bMoveOneExtraStep = True
          ' located a range that has a minimum/maximum/root?
        ElseIf SFa * SFb > 0 And SDa * SDb < 0 Then
          X = NewtonMinMax((Xa + Xb) / 2, Toler)
          Drv2 = MySecDeriv(X)
          ' found a root?
          Stack(X_STACK_REG) = X
          Fx = Calc(Alpha)
          SetMem(MemIdx, X)
          SetMem(MemIdx + 1, Fx)
          bMoveOneExtraStep = True
          If Abs(Fx) < FxToler Then
            sMsg = "Root & "
          Else
            sMsg = ""
          End If

          If Drv2 > 0 Then
            sMsg &= "Minimum"
            WriteLine("Result # {0}", CountResults + 1)
            WriteLine("M({0}) has {1} at {2}", MemIdx, sMsg, Output(X))
            WriteLine("M({0}) has F(x) = {1}", MemIdx + 1, Output(Fx))
            WriteLine("----------------------------------------")
            SetMem(MemIdx + 2, -1)
            MemIdx += 3
            CountResults += 1
            SetMem(MemIdx0, CountResults)
          Else
            sMsg &= "Maximum"
            WriteLine("Result # {0}", CountResults + 1)
            WriteLine("M({0}) has {1} at {2}", MemIdx, sMsg, Output(X))
            WriteLine("M({0}) has F(x) = {1}", MemIdx + 1, Output(Fx))
            WriteLine("----------------------------------------")
            SetMem(MemIdx + 2, 1)
            MemIdx += 3
            CountResults += 1
            SetMem(MemIdx0, CountResults)
          End If

        End If

        If bMoveOneExtraStep Then
          NumSteps += 1
          Xa = A + NumSteps * Stp
          Stack(X_STACK_REG) = Xa
          Fa = Calc(Alpha)
          Da = MySlope(Xa, Fa)
          SFa = Sign(Fa)
          SDa = Sign(Da)
        Else
          Xa = Xb
          Fa = Fb
          Da = Db
          SFa = SFb
          SDa = SDb
        End If

      Catch

        ErrCount += +1
        If ErrCount > MAX_ERROR Then
          WriteLine("Reached maximum limit or runtime errors!")
          Enter(CountResults)
          Exit Sub
        End If

        NumSteps += 1
        Xa = A + NumSteps * Stp
        Stack(X_STACK_REG) = Xa
        Fa = Calc(Alpha)
        Da = MySlope(Xa, Fa)
        SFa = Sign(Fa)
        SDa = Sign(Da)

      End Try

    Loop Until Xa >= B
    ' no need to backup stack
    Enter(CountResults)

  End Sub


  Protected Sub fxRSTOcom(ByVal tok As String, ByVal sComName As String)
    Dim s, sCom As String
    Dim I, Len, StrArrLen As Integer
    Dim StrArr() As String
    Dim X As Double

    s = tok.Substring(4).Trim
    ' RRCLx command has no arguments!
    If s.Length = 0 Then
      s = Alpha
      I = Fix(Stack(Y_STACK_REG))
    Else
      ' command has one or two arguments
      StrArr = tok.Split(" ")
      StrArrLen = StrArr.Length - 1
      ' just one argument?
      If StrArrLen = 1 Then
        If IsNumeric(StrArr(1)) Then
          s = Alpha
          I = ParseInt(StrArr(1))
        Else
          s = StrArr(1)
          I = Fix(Stack(Y_STACK_REG))
        End If
      Else
        ' two arguments
        If IsNumeric(StrArr(2)) Then
          s = StrArr(1)
          I = ParseInt(StrArr(2))
        Else
          sErrMessage = "Error in command " & sComName
        End If
      End If
    End If

    If s.Length > 0 Then
      X = Stack(X_STACK_REG) ' objVarArrs.Recall(s, I)
      Len = sComName.Length
      sCom = sComName.Substring(Len - 1, 1)
      Select Case sCom
        Case "+"
          objVarArrs.StorePlus(s, I, X)
        Case "-"
          objVarArrs.StoreMinus(s, I, X)
        Case "*"
          objVarArrs.StoreMult(s, I, X)
        Case "/"
          If X <> 0 Then
            objVarArrs.StoreDiv(s, I, X)
          Else
            sErrMessage = "Divide-by-zero error in RSTO/"
          End If
        Case Else
          objVarArrs.Store(s, I, X)
      End Select
    Else
      sErrMessage = "Missing variable name for " & sComName & " command"
    End If
  End Sub

  Protected Sub fxRSTOUpdateIndex(ByVal tok As String, ByVal sComName As String)
    Dim s, sCom As String
    Dim I, Len As Integer
    Dim X As Double

    s = tok.Substring(4).Trim
    If s.Length = 0 Then s = Alpha
    X = Stack(X_STACK_REG) ' objVarArrs.Recall(s, I)
    Len = sComName.Length
    sCom = sComName.Substring(Len - 1, 1)
    Select Case sCom
      Case "+"
        objVarArrs.StoreIncIndex(s, X)
      Case "-"
        objVarArrs.StoreDecIndex(s, X)
      Case Else
        objVarArrs.StoreKeepIndex(s, X)
    End Select
  End Sub

  Protected Sub fxRRCLcom(ByVal tok As String, ByVal sComName As String)
    Dim s, sCom As String
    Dim I, Len, StrArrLen As Integer
    Dim StrArr() As String
    Dim X As Double

    s = tok.Substring(4).Trim
    ' RRCLx command has no arguments!
    If s.Length = 0 Then
      s = Alpha
      I = Fix(Stack(X_STACK_REG))
    Else
      ' command has one or two arguments
      StrArr = tok.Split(" ")
      StrArrLen = StrArr.Length - 1
      ' just one argument?
      If StrArrLen = 1 Then
        If IsNumeric(StrArr(1)) Then
          s = Alpha
          I = ParseInt(StrArr(1))
        Else
          s = StrArr(1)
          I = Fix(Stack(X_STACK_REG))
        End If
      Else
        ' two arguments
        If IsNumeric(StrArr(2)) Then
          s = StrArr(1)
          I = ParseInt(StrArr(2))
        Else
          sErrMessage = "Error in command " & sComName
        End If
      End If
    End If

    If s.Length > 0 Then
      X = objVarArrs.Recall(s, I)
      Len = sComName.Length
      sCom = sComName.Substring(Len - 1, 1)
      Select Case sCom
        Case "+"
          Stack(X_STACK_REG) += X
        Case "-"
          Stack(X_STACK_REG) -= X
        Case "*"
          Stack(X_STACK_REG) *= X
        Case "/"
          If X <> 0 Then
            Stack(X_STACK_REG) /= X
          Else
            sErrMessage = "Divide-by-zero error in RRCL/"
          End If
        Case Else
          Enter(X)
      End Select
    Else
      sErrMessage = "Missing variable name for " & sComName & " command"
    End If
  End Sub

  Protected Sub fxRRCLUpdateIndex(ByVal tok As String, ByVal sComName As String)
    Dim s, sCom As String
    Dim I, Len As Integer
    Dim X As Double

    s = tok.Substring(4).Trim
    If s.Length = 0 Then s = Alpha
    Len = sComName.Length
    sCom = sComName.Substring(Len - 1, 1)
    Select Case sCom
      Case "+"
        X = objVarArrs.RecallIncIndex(s)
      Case "-"
        X = objVarArrs.RecallDecIndex(s)
      Case Else
        X = objVarArrs.RecallKeepIndex(s)
    End Select
    Stack(X_STACK_REG) = X
  End Sub


  Protected Sub SaveStatRegisters(ByVal sFilename As String)
    Dim fileWriter As System.IO.StreamWriter
    Try
      fileWriter = New StreamWriter(sFilename)
      fileWriter.WriteLine(StatCount)
      fileWriter.WriteLine(SumX)
      fileWriter.WriteLine(SumX2)
      fileWriter.WriteLine(SumY)
      fileWriter.WriteLine(SumY2)
      fileWriter.WriteLine(SumXY)
      fileWriter.WriteLine(If(bHaveRegressionResults, "1", "0"))
      fileWriter.Close()
    Catch
      sErrMessage = "Cannot create file " & sFilename & " In SAVESTAT command"
    End Try
  End Sub

  Protected Sub GetStatRegisters(ByVal sFilename As String)
    Dim TextLine As String

    If My.Computer.FileSystem.FileExists(sFilename) Then
      Dim fileReader As System.IO.StreamReader
      fileReader = New StreamReader(sFilename)
      Try
        TextLine = fileReader.ReadLine()
        StatCount = ParseInt(TextLine)
        TextLine = fileReader.ReadLine()
        SumX = ParseDbl(TextLine)
        TextLine = fileReader.ReadLine()
        SumX2 = ParseDbl(TextLine)
        TextLine = fileReader.ReadLine()
        SumY = ParseDbl(TextLine)
        TextLine = fileReader.ReadLine()
        SumY2 = ParseDbl(TextLine)
        TextLine = fileReader.ReadLine()
        SumXY = ParseDbl(TextLine)
        TextLine = fileReader.ReadLine()
        bHaveRegressionResults = TextLine.StartsWith("1")
        fileReader.Close()
      Catch
        sErrMessage = "Error reading numbers from file " & sFilename & " In GETSTAT command"
      End Try
    Else
      sErrMessage = "Cannot open file " & sFilename & " In GETSTAT command"
    End If
  End Sub


  Protected Sub MergeStatRegisters(ByVal sFilename As String)
    Dim TextLine As String

    If My.Computer.FileSystem.FileExists(sFilename) Then
      Dim fileReader As System.IO.StreamReader
      fileReader = New StreamReader(sFilename)
      Try
        TextLine = fileReader.ReadLine()
        StatCount += ParseInt(TextLine)
        TextLine = fileReader.ReadLine()
        SumX += ParseDbl(TextLine)
        TextLine = fileReader.ReadLine()
        SumX2 += ParseDbl(TextLine)
        TextLine = fileReader.ReadLine()
        SumY += ParseDbl(TextLine)
        TextLine = fileReader.ReadLine()
        SumY2 += ParseDbl(TextLine)
        TextLine = fileReader.ReadLine()
        SumXY += ParseDbl(TextLine)
        bHaveRegressionResults = False
        fileReader.Close()
      Catch
        sErrMessage = "Error reading numbers from file " & sFilename & " In MERGESTAT command"
      End Try
    Else
      sErrMessage = "Cannot open file " & sFilename & " In MERGESTAT command"
    End If
  End Sub

  Public Sub ExecProgramByLabelNumber()
    Dim I As Integer
    Dim s As String

    Do
      ShowLabels()
      WriteLine("------------------------------------------")
      Write("Select label by number: ")
      s = ReadLine()

      Try
        I = CInt(ParseDbl(s))
      Catch
        I = -1
      End Try
    Loop Until I >= 0 And I <= LblsCount
    ExecProgram(sLbls(I))
  End Sub

  Public Sub ExecProgram(ByVal SelLbl As String)
    Dim aLine, tokens() As String
    Dim numLines As Integer = sLines.Length

    ClearFlags()

    If SelLbl = "" Then
      PrgmPtr = 0
    Else
      PrgmPtr = FindLbl(SelLbl.Trim.ToUpper)
      If PrgmPtr > 0 Then
        ' get the line number for the matching label
        PrgmPtr = LblsIndex(PrgmPtr)
      Else
        WriteLine("Label '" & SelLbl & "' is nonexistent.")
        Exit Sub
      End If
    End If
    RtnCount = 0
    bStop = False
    bErr = False
    ClearErrMessage()
    ErrHandlingLine = -1


    Do
      aLine = sLines(PrgmPtr).Trim
      tokens = aLine.Split(" ")
      If IsAnRpnExprToken(tokens(0)) Then
        ParseToken(aLine)
        PrgmPtr += 1
      Else
        ExecPrgmLine(aLine)
      End If

      If sErrMessage.Length > 0 OrElse objVarArrs.GetErrorMessage().Length > 0 Then
        bErr = True
      End If

      If bErr Then
        If Not (bOnErrGoto Or bOnErrReumeNext) Then
          If sErrMessage.Length > 0 Then
            WriteLine("Error: " & sErrMessage & ".")
          Else
            WriteLine("Error: " & objVarArrs.GetErrorMessage() & ".")
          End If
        Else
          WriteLine("Program continues due to active error handler ... ")
          ' resume program execution at the error handling label
          If bOnErrGoto And ErrHandlingLine > -1 Then
            PrgmPtr = ErrHandlingLine
          End If
          sErrMessage = "" ' reset the error message
          objVarArrs.ResetErrMessage()
          bErr = False ' clear error status
        End If
      End If
    Loop Until PrgmPtr >= numLines Or bStop Or bErr

    If Not bErr Then
      If bShowStack Then
        WriteLine("-----------------------------------------------")
        ShowStack()
        WriteLine("-----------------------------------------------")
      End If
      bShowStack = True ' restore show-satck flag
    Else
      WriteLine("Program had runtime Error at or related to the following line:")
      WriteLine(PrgmPtr & ": " & aLine)
      If sErrMessage.Length > 0 Then
        WriteLine(sErrMessage & ".")
      Else
        WriteLine(objVarArrs.GetErrorMessage() & ".")
      End If
    End If
  End Sub

End Class
